#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from Bio import SeqIO
import getopt
import sys
sys.stdout.flush()

class GetTaxonCompositionFromGenBank():

	verbose = False

	def __init__(self):
		self._seqFile = None
		self._seqFileList = None
		self._seqList = []

	def setAttributesFromCmdLine(self):
		try:
			opts, args = getopt.getopt(sys.argv[1:],"vf:l:",["file=","list=","verbose","help"])
		except getopt.GetoptError, err:
			print str(err)
			self.help()
		for o,a in opts:
			if o in ("-v","--verbose"):
				verbose = True
			elif o in ("--help","--usage"):
				self.help()
				sys.exit()
			elif o in ("-f","--file"):
				self._seqFile = a
			elif o in ("-l","--list"):
				self._seqFileList = a
			else:
				assert False, "unhandled option"
				self.help()

		if not self._seqFile:
			print "\n ERROR: missing parameter --file"
			self.help()

	def help(self):
		print
		print "usage: %s [ options ]" % (sys.argv[0])
		print
		print "args (mandatory):"
		print "  -f or --file: genbank file to read"
		print
		print "args (optional):"
		print "  -l or --list: limit info extraction to this list of accessions"
		print "  -v or --verbose"
		print "  --help, --usage"

		sys.exit()

	def _readList(self):
		listFile = open(self._seqFileList, 'r')
		line = listFile.readline().strip()

		while line != "":
			self._seqList.append(line)
			line = listFile.readline().strip()
		listFile.close()

	def exe(self):
		tree = {}
		taxo = {}

		if self._seqFileList:
			self._readList()

		#records = list(SeqIO.parse(self._seqFile, "genbank"))

		#print "Found %i records" % len(records)

		SeqEntries = 0
		lastPrint = 0

		for seq_record in SeqIO.parse(self._seqFile, "genbank"):
			taxonID = seq_record.id
			taxonOrganism = seq_record.annotations['taxonomy']
			taxonOrganism.append(seq_record.annotations['organism'])

			SeqEntries = SeqEntries + 1
			if SeqEntries == lastPrint + 10000 :
				print str(SeqEntries) + " entries read"
				lastPrint = SeqEntries

			if taxonID in self._seqList:

				lval = [taxonOrganism[0]]
				for j in range(1,len(taxonOrganism)):
					lval.append(lval[j-1] +";"+ taxonOrganism[j])

				# set taxa in each Level
				for val in lval:
					if(tree.has_key(val)):
						tree[val].append(seq_record.annotations['accessions'][0])
					else:
						tree[val] = [seq_record.annotations['accessions'][0]]





		# print tree
		lKeys = tree.keys()
		lKeys.sort()

		for key in lKeys:
			print key + "\t" + str(len(tree[key]))


if __name__=="__main__":
	iGetTaxonCompositionFromGenBank = GetTaxonCompositionFromGenBank()
	iGetTaxonCompositionFromGenBank.setAttributesFromCmdLine()
	iGetTaxonCompositionFromGenBank.exe()

