#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser


class LaunchAssignTaxonomyWithBlast(object):

    def __init__(self, inputFileName = "", referenceFileName = "", referenceTaxonMappingFileName = ""):
        self._options = None
        self._inputFileName = inputFileName
        self._referenceFileName = referenceFileName
        self._referenceTaxonMappingFileName = referenceTaxonMappingFileName
        
    def setAttributesFromCmdLine(self):
        description = "LaunchAssignTaxonomyWithBlast assigns taxonomy to each sequence with blast.\n" 
        usage = "LaunchAssignTaxonomyWithBlast.py -i <input file> -r <reference file> -t <taxonomy file>\n"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-i', "--input", dest="inputFileName", action="store", type="string", help="input file (fasta type)")
        parser.add_option('-r', "--reference", dest="reference", action="store", type="string", help="reference file ")
        parser.add_option('-t', "--taxonomy", dest="taxonomy", action="store", type="string", help="taxonomy file (optional)")
        (options, args) = parser.parse_args()
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setInputFileName(options.inputFileName)
        self.setReferenceFileName(options.reference)
        self.setTaxonomyFileName(options.taxonomy)
        
    def setInputFileName(self, inputFileName):
        self._inputFileName = inputFileName
        
    def setReferenceFileName(self, referenceFileName):
        self._referenceFileName = referenceFileName
        
    def setTaxonomyFileName(self, taxonomyFileName):
        self._taxonomyFileName = taxonomyFileName
        
    def checkOptions(self):       
        if self._inputFileName != "":
            if not FileUtils.isRessourceExists(self._inputFileName):
                raise Exception("ERROR: Input file does not exist!")
        else:
            raise Exception("ERROR: No specified -i option!")
        
        if self._referenceFileName != "":
            if not FileUtils.isRessourceExists(self._referenceFileName):
                raise Exception("ERROR: reference file does not exist!")
        else:
            raise Exception("ERROR: No specified -r option!")
        
        if self._taxonomyFileName != "":
            if not FileUtils.isRessourceExists(self._taxonomyFileName):
                raise Exception("ERROR: taxonomy mapping file does not exist!")
        else:
            raise Exception("ERROR: No specified -t option!")
                             
    def run(self):
        self.checkOptions()  
        f = open(".ncbirc", 'w')
        f.close() 
        cmd = 'assign_taxonomy.py -i %s -r %s -t %s -m blast' % (self._inputFileName, self._referenceFileName, self._taxonomyFileName)
        os.system(cmd)
        os.remove(".ncbirc")
            
if __name__ == "__main__":
    iLaunch = LaunchAssignTaxonomyWithBlast()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()