#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser


class LaunchFungalITSExtractor(object):

    def __init__(self, inputFileName = "", workingDir = "", profilesPath = ""):
        self._inputFileName = inputFileName
        self._workingDir = workingDir
        self._profilesPath = profilesPath
        
    def setAttributesFromCmdLine(self):
        description = "LaunchFungalITSExtractor extract ITS sequences from fasta files." 
        usage = "LaunchFungalITSExtractor.py -i <input file> -w <working directory> -p <profile directory path>"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-i', "--input", dest="inputFileName", action="store", type="string", help="input file (fasta type)")
        parser.add_option('-w', "--workingDir", dest="workingDir", action="store", type="string", help="working directory file ")
        parser.add_option('-p', "--profilesPath", dest="profilesPath", action="store", type="string", help="profiles directory path")
        (options, args) = parser.parse_args()
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setInputFileName(options.inputFileName)
        self.setWorkingDirectory(options.workingDir)
        self.setProfilesPath(options.profilesPath)
        
    def setInputFileName(self, inputFileName):
        self._inputFileName = inputFileName
        
    def setWorkingDirectory(self, workingDirectory):
        self._workingDir = workingDirectory
        
    def setProfilesPath(self, profilesPath):
        self._profilesPath = profilesPath
        
    def checkOptions(self):       
        if self._inputFileName != "":
            if not FileUtils.isRessourceExists(self._inputFileName):
                raise Exception("ERROR: Input file does not exist!")
        else:
            raise Exception("ERROR: No specified -i option!")
        
        if self._workingDir == "":
            raise Exception("ERROR: No specified -w option!")
        
        if self._profilesPath != "":
            if not FileUtils.isRessourceExists(self._profilesPath):
                raise Exception("ERROR: profiles directory does not exist!")
        else:
            raise Exception("ERROR: No specified -p option!")
                             
    def run(self):
        self.checkOptions()
        if not FileUtils.isRessourceExists(self._workingDir):
            os.mkdir(self._workingDir)
        cmd = '%s/Gnome_tools/FungalITSextractorURGI.pl %s %s %s' % (os.environ["REPET_PATH"], self._inputFileName, self._workingDir, self._profilesPath)
        os.system(cmd)
            
if __name__ == "__main__":
    iLaunch = LaunchFungalITSExtractor()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()