#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser


class LaunchPickOTUWihtUclust(object):

    def __init__(self, inputFileName = "", identity = 0.97, maxAccepts = 20, maxRejects = 500, optimal = False):
        self._options = None
        self._inputFileName = inputFileName
        self._identity = identity
        self._maxAccepts = maxAccepts
        self._maxRejects = maxRejects
        self._isOptimal = optimal
        
    def setAttributesFromCmdLine(self):
        description = "LaunchPickOTUWihtUclust clusterizes sequences using uclust"
        usage = "LaunchPickOTUWihtUclust.py -i <input file> [-s <similarity> -a <maxaccepts> -r <maxrejects> -o <optimal>]"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-i', "--input", dest="inputFileName", action="store", type="string", help="input file (fasta type)")
        parser.add_option("-s", "--similarity",   dest="identity", action="store", type="float", default=0.97, help="similarity/identity")
        parser.add_option("-a", "--maxaccepts",   dest="maxAccepts", action="store", type="int", default=20, help="Stop searching if n hits have been found, then report the best.")
        parser.add_option("-r", "--maxrejects",   dest="maxRejects", action="store", type="int", default=500, help="Stop searching if n rejects have occurred, then report a failure to find a hit.")
        parser.add_option("-o", "--optimal", dest="isOptimal", action="store_true", default = False, help="Same as --maxrejects 0 --maxaccepts 0")

        (self._options, args) = parser.parse_args()
        self._setAttributesFromOptions(self._options)

    def _setAttributesFromOptions(self, options):
        self.setInputFileName(self._options.inputFileName)
        self.setIdentity(self._options.identity)
        self.setMaxAccepts(self._options.maxAccepts)
        self.setMaxRejects(self._options.maxRejects)
        self.setIsOptimal(self._options.isOptimal)
        
    def setIdentity(self, identity):
        self._identity = identity
        
    def setMaxAccepts(self, maxAccepts):
        self._maxAccepts = maxAccepts
        
    def setMaxRejects(self, maxRejects):
        self._maxRejects = maxRejects
        
    def setInputFileName(self, inputFileName):
        self._inputFileName = inputFileName
        
    def setIsOptimal(self, isOptimal):
        self._isOptimal = isOptimal
            
    def checkOptions(self):       
        if self._inputFileName != "":
            if not FileUtils.isRessourceExists(self._inputFileName):
                raise Exception("ERROR: Input file does not exist!")
        else:
            raise Exception("ERROR: No specified -i option!")
                             
    def run(self):
        self.checkOptions()
        optimal = "--optimal"
        if not self._isOptimal:
            optimal = ""            
        cmd = 'pick_otus.py -m uclust -i %s -s %f --max_accepts %i --max_rejects %i %s ' % (self._inputFileName, self._identity, self._maxAccepts, self._maxRejects, optimal)
        os.system(cmd)
            
if __name__ == "__main__":
    iLaunchUclust = LaunchPickOTUWihtUclust()
    iLaunchUclust.setAttributesFromCmdLine()
    iLaunchUclust.run()