#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser


class LaunchPickRepSet(object):

    def __init__(self, inputFileName = "", OTUclusterFileName = "", outputFileName= ""):
        self._options = None
        self._inputFileName = inputFileName
        self._outputFileName = outputFileName
        self._OTUclusterFileName = OTUclusterFileName
        
    def setAttributesFromCmdLine(self):
        description = "LaunchPickRepSet picks representative set of sequences among OTUs." 
        usage = "LaunchPickRepSet.py -i <input file> -c <OTUs file> [-o <output file>]"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-i', "--input", dest="inputFileName", action="store", type="string", help="input file (fasta type)")
        parser.add_option('-c', "--cluster", dest="clusterFileName", action="store", type="string", help="cluster file ")
        parser.add_option('-o', "--output", dest="outputFileName", action="store", type="string", default="", help="output file (fasta type) (optional)")
        (self._options, args) = parser.parse_args()
        self._setAttributesFromOptions(self._options)

    def _setAttributesFromOptions(self, options):
        self.setInputFileName(self._options.inputFileName)
        self.setOutputFileName(self._options.outputFileName)
        self.setCLusterFileName(self._options.clusterFileName)
        
    def setInputFileName(self, inputFileName):
        self._inputFileName = inputFileName
        
    def setCLusterFileName(self, clusterFileName):
        self._OTUclusterFileName = clusterFileName
        
    def setOutputFileName(self, outputFileName):
        self._outputFileName = outputFileName
        
    def checkOptions(self):       
        if self._inputFileName != "":
            if not FileUtils.isRessourceExists(self._inputFileName):
                raise Exception("ERROR: Input file does not exist!")
        else:
            raise Exception("ERROR: No specified -i option!")
        
        if self._OTUclusterFileName != "":
            if not FileUtils.isRessourceExists(self._OTUclusterFileName):
                raise Exception("ERROR: cluster file does not exist!")
        else:
            raise Exception("ERROR: No specified -c option!")
                             
    def run(self):
        self.checkOptions()         
        outputFileArg = ""
        if self._outputFileName != "":
            outputFileArg = "-o %s" % self._outputFileName 
        cmd = 'pick_rep_set.py -m most_abundant -f %s -i %s %s ' % (self._inputFileName, self._OTUclusterFileName, outputFileArg)
        os.system(cmd)
            
if __name__ == "__main__":
    iLaunchUclust = LaunchPickRepSet()
    iLaunchUclust.setAttributesFromCmdLine()
    iLaunchUclust.run()