#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
import os
from commons.core.seq.BioseqDB import BioseqDB
from commons.core.utils.RepetOptionParser import RepetOptionParser

class TaxonomyClusterToFasta(object):
    
    def setAttributesFromCommandLine(self):
        description = ""
        usage = ""
        examples = "TaxonomyClusterToFasta.py -i fastaFile -t clusterTableFile -c clusterFile -p projectName\n"
        parser = RepetOptionParser(description = description, usage = usage, version = "v1.0", epilog = examples)
        parser.add_option("-c", "--cluster", dest="otu", help="otu file")
        parser.add_option("-t", "--taxonomy_otu_table", dest="taxonomy_otu_table", help="taxonomy_otu_table FILE")
        parser.add_option("-i", "--read_file", dest="read_file", help="read FILE")
        parser.add_option("-p", "--projectName", dest="projectName", help="project name")
        options, args = parser.parse_args()
        return options, args
        
    def checkOptions(self):
        if not self._file:
            raise Exception("ERROR: Option read file is missing!")
        if not self._otu:
            raise Exception("ERROR: Option otu file is missing!")
        if not self._otu_table:
            raise Exception("ERROR: Option otu_table file is missing!")
        if not self._projectName:
            raise Exception("ERROR: Option projectName is missing!")
        
    def setOTUFile(self, otu):
        self._otu = otu
        
    def setReadFile(self, file):
        self._file = file
        
    def setOTU_tableFile(self, otu_table):
        self._otu_table = otu_table
        
    def setProjectName(self, projectName):
        self._projectName = projectName
        
    def writeOutputFastaFiles(self, iBsDB, dTaxaTolClusterId, dClusterId2SeqName):
        if not os.path.isdir("%s_Taxonomy_fasta_files" % self._projectName):
            os.mkdir("%s_Taxonomy_fasta_files" % self._projectName)
        os.chdir("%s_Taxonomy_fasta_files" % self._projectName)   
        
        for taxa in dTaxaTolClusterId.keys():
            lId = dTaxaTolClusterId[taxa]
            lSeqName = []
            for id in lId:
                lSeqName.extend(dClusterId2SeqName[id])
            taxa = taxa.replace("/", "_")
            self._writeOutputFastaFileAccordingToGenus(iBsDB, lSeqName, taxa)
                    
        os.chdir("../")

    def createDictTaxon2lClusterId(self, handler):
        line = handler.readline()
        dTaxa2Cluster = {}
        while line:
            cleanLine = line.rstrip("\n")
            if not cleanLine.startswith("#"):
                clusterNumber = cleanLine.split("\t")[0]
                taxon = cleanLine.split("\t")[2]
                genus = taxon.split(";")[-1]
                if not dTaxa2Cluster.has_key(genus):
                    lClusterId = []
                    lClusterId.append(clusterNumber)
                    dTaxa2Cluster[genus] = lClusterId
                else:
                    lClusterId = dTaxa2Cluster[genus]
                    lClusterId.append(clusterNumber)
                    dTaxa2Cluster[genus] = lClusterId
                    
            line = handler.readline()
        return dTaxa2Cluster
    
    def createDictClusterId2ClusterSeq(self, handler):
        line = handler.readline()
        dClusterId2Seq = {}
        while line:
            cleanLine = line.rstrip("\n")
            if not cleanLine.startswith("#"):
                clusterId = cleanLine.split("\t")[0]
                lSeq = cleanLine.split("\t")[1:]
                dClusterId2Seq[clusterId] = lSeq
            line = handler.readline()
        return dClusterId2Seq

    def run(self):
        self.checkOptions() 
        iBsDB = BioseqDB(self._file)
                
        otuTableHandler = open(self._otu_table, "r")
        dTaxa2lClusterId = self.createDictTaxon2lClusterId(otuTableHandler)
        otuTableHandler.close()
        
        otuFileHandler = open(self._otu, "r")
        dClusterId2SeqName = self.createDictClusterId2ClusterSeq(otuFileHandler)
        otuFileHandler.close()
        
        self.writeOutputFastaFiles(iBsDB, dTaxa2lClusterId, dClusterId2SeqName)
        
    def _writeOutputFastaFileAccordingToGenus(self, iBsDB, lSeqName, genus):
        outputFileName = genus.replace(" ", "_")
        f = open("%s.fasta" % outputFileName , "w")
        for read in lSeqName:
            bioseq = iBsDB.fetch(read)
            f.write(">%s\n" % bioseq.getHeader())
            f.write("%s\n" % bioseq.getSequence())
        f.close()
                
if __name__ == "__main__":
    icreateNewFile =  TaxonomyClusterToFasta() 
    options, args = icreateNewFile.setAttributesFromCommandLine()   
    icreateNewFile.setReadFile(options.read_file)
    icreateNewFile.setOTUFile(options.otu)
    icreateNewFile.setOTU_tableFile(options.taxonomy_otu_table)
    icreateNewFile.setProjectName(options.projectName)
    icreateNewFile.run()
        