#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from Bio import SeqIO
import getopt
import sys
import os
from commons.core.utils.RepetOptionParser import RepetOptionParser
from commons.core.utils.FileUtils import FileUtils

class WriteTaxonomyMappingAndFastaFiles(object):

    def __init__(self):
        self._fastaFileName = ""
        self._genbankFileName = ""
        self._seqList = []
        self._id = 0
        
        
    def setAttributesFromCmdLine(self):
        description = "WriteTaxonomyMappingAndFastaFiles writes taxonomy mapping file and new reference fasta file from reference genbank and fasta files.\n" 
        usage = "WriteTaxonomyMappingAndFastaFiles.py -f <fasta file> -g <genbank file>"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-f', "--fasta", dest="fastaFileName", action="store", type="string", help="input file (fasta type)")
        parser.add_option('-g', "--genbank", dest="genbankFileName", action="store", type="string", help="input file (genbank type)")
        (options, args) = parser.parse_args()
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setFastaFileName(options.fastaFileName)
        self.setGenbankFileName(options.genbankFileName)

    def setFastaFileName(self, file):
        self._fastaFileName = file
        
    def setGenbankFileName(self, file):
        self._genbankFileName = file
        
    def checkOptions(self):       
        if self._fastaFileName != "":
            if not FileUtils.isRessourceExists(self._fastaFileName):
                raise Exception("ERROR: fasta file %s does not exist!" % self._fastaFileName)
        else:
            raise Exception("ERROR: No specified -f option!")
        
        if self._genbankFileName != "":
            if not FileUtils.isRessourceExists(self._genbankFileName):
                raise Exception("ERROR: genbank file %s does not exist!" % self._genbankFileName)
        else:
            raise Exception("ERROR: No specified -g option!")

    def run(self):
        self.checkOptions()
        dIdToTaxonomy = {}
        fastaPath, ext = os.path.splitext(self._fastaFileName)
        
        self._taxonMappingFile = open("%s_taxonMappingID.txt" % fastaPath, "w")
        self._newFastaFile = open("%s_taxonMappingID.fasta" % fastaPath, "w")
        
        for seq_genbank_record in SeqIO.parse(self._genbankFileName, "genbank"):
            tax = seq_genbank_record.annotations["taxonomy"]
            tax.append(seq_genbank_record.annotations["source"])
            dIdToTaxonomy[seq_genbank_record.id] = tax
            
        for seq_record in SeqIO.parse(self._fastaFileName, "fasta"):
            if  dIdToTaxonomy.has_key(seq_record.id):
                taxonomy = ";".join(dIdToTaxonomy[seq_record.id])
                self._writeTaxonomyMappingIdFile(taxonomy)
                self._writeNewFastaFile(seq_record)
                
        self._taxonMappingFile.close()
        self._newFastaFile.close()
        
    def _writeTaxonomyMappingIdFile(self, taxonomy):
        self._id += 1
        self._taxonMappingFile.write("%s\t%s\n" % (self._id, taxonomy))
         
    def _writeNewFastaFile(self, iSeq):
        self._newFastaFile.write(">%s %s\n" % (self._id,iSeq.id)) 
        self._newFastaFile.write("%s\n" % iSeq.seq)


if __name__=="__main__":
    iWTMAFF = WriteTaxonomyMappingAndFastaFiles()
    iWTMAFF.setAttributesFromCmdLine()
    iWTMAFF.run()
