#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import optparse, re

def __main__():
	#Parse Command Line
	parser = optparse.OptionParser()
	parser.add_option("-i", "--input", dest="inputFile", help="input gff file.")
	parser.add_option("-o", "--output", dest="outputFile", help="Out csv file.")
	options = parser.parse_args()[0]
	fh = open(options.inputFile, "r")
	oh = open(options.outputFile, "w")
	
	patternNbOverlaps = re.compile("nbOverlaps=(\d+);")
	patterngene_id = re.compile("gene_id=([^;]*)")
	patternParent = re.compile("Parent=([^;]*)")
	patternName = re.compile("Name=([^;]*)")
	patternID = re.compile("ID=([^;]*)")
	
	line = fh.readline().strip()
	while line:
		geneID = ""
		overlap = ""
		if line.split("\t")[2].lower() in ["mrna", "transcript"]:
			overlap = patternNbOverlaps.search(line).group(1)	
			m = patterngene_id.search(line)
			if not m:
				m = patternParent.search(line)
				if not m:
					m = patternName.search(line)
					if not m:
						m = patternID.search(line)
			if m:
				geneID = m.group(1)
		
			oh.write("%s\t%s\n" % (geneID, overlap))
		
		line = fh.readline().strip()

	fh.close()
	oh.close()

if __name__=="__main__":__main__()
