#! /usr/bin/env python


import optparse, os, sys, tarfile, random
from optparse import OptionParser
from commons.core.launcher.Launcher import Launcher
from commons.core.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory

repetPath = os.environ["REPET_PATH"].rstrip(":") #bugfix because galaxy adds ":" at the end of the path


def stop_err(msg):
	sys.stderr.write('%s\n' % msg)
	sys.exit()

def _map(iLauncher, cmd, cmdStart, cmdFinish ):
	lCmds = []
	lCmds.append(cmd)
	lCmdStart = []
	lCmdStart.append(cmdStart)
	lCmdFinish = []
	lCmdFinish.append(cmdFinish)
	return(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))

def _createCountNumberCommand(iLauncher, inputFile, outputFile):
		lArgs = []
		lArgs.append("-i %s" % inputFile)
		lArgs.append("-o %s" %  outputFile)
		return iLauncher.getSystemCommand("python %s/SMART/DiffExpAnal/countNumber.py " %  repetPath, lArgs)

def __main__():
	#Parse Command Line
	parser = OptionParser()
	parser.add_option("-i", "--input", dest="inputFile", help="input txt file, a list of overlapping results files.")
	parser.add_option("-o", "--output", dest="outputFile", help="Out txt file.")
	(options, args) = parser.parse_args()


	inputFileNames = []
	outCountNames = []
	tar = tarfile.open(options.inputFile, "r")
	workingDir = os.getcwd()
	for tarinfo in tar:	
			name = tarinfo.name
			root = os.path.basename(name).split(".")[0]
			tar.extract(name)
			workingDir = os.getcwd()
			inputFileNames.append("%s/%s" % (workingDir, name))
			outCountNames.append("%s/%s_overlapOut.csv" % (workingDir, root))
	tar.close()

	#Launch on nodes
	acronym = "countNumber"
	jobdb = TableJobAdaptatorFactory.createJobInstance()
	iLauncher = Launcher(jobdb, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
	lCmdsTuples = []
	for i in range(len(inputFileNames)):	#Construct the lines commands
		inputFile = inputFileNames[i]
		outputFile = outCountNames[i]
		cmd2Launch = _createCountNumberCommand(iLauncher, inputFile, outputFile)
		cmdStart = ""
		cmdFinish = ""
		lCmdsTuples.append(_map(iLauncher, cmd2Launch, cmdStart, cmdFinish))

	iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)
	
	tar = tarfile.open(options.outputFile, "w")
	for fileName in outCountNames:
		tar.add(os.path.basename(fileName))
	tar.close()



if __name__=="__main__":__main__()
