#!/usr/bin/env python

import os, tarfile, re, pickle
from optparse import OptionParser

class manageTarForDEA(object):

    def __init__(self):
        self._untar = ""
        self._tar = ""
        self._prgName = ""
        self._workingDir = ""
        self._isPaired = ""
        self._output = ""
        self._ext =""
        
        
    def untar(self, inputTar, workingDir,prgName ,isPaired, ext):
        inputFileNames = []
        outputFileNames = []
        if isPaired:
            inputRevFileNames = []
        tar = tarfile.open(inputTar, "r")
        for tarinfo in tar:    
                name = tarinfo.name
                root = os.path.basename(name).split(".")[0]
                tar.extract(os.path.basename(name), path="%s" % workingDir)
                if re.match(".*paired.*", root) and isPaired:
                    inputRevFileNames.append("%s/%s" % (workingDir, name))
                else:
                    inputFileNames.append("%s/%s" % (workingDir, name))
                    outputFileNames.append("%s/%s_%s.%s" % (workingDir, root, prgName, ext))
        tar.close()
        if isPaired:
            return inputFileNames, inputRevFileNames, outputFileNames
        else:
            return inputFileNames, outputFileNames
    
    def tar(self, outputTar, l2Tar):
        tar = tarfile.open(outputTar, "w")
        for fileName in l2Tar:
            tar.add(fileName)
        tar.close()

    def setAttributesFromCmdLine(self):
        description = "Tar and Untar for DEA."
        parser = OptionParser(description = description)
        parser.add_option("-u", "--untar",      dest = "untar",         action = "store",       type = "string", help = "Tar file to untar",       default = "")
        parser.add_option("-t", "--tar",    dest = "tar",       action = "store",       type = "string", help = "pickle list file to tar", default = "")
        parser.add_option("-p", "--progname",dest = "prgName",   action = "store",       type = "string",    help = "Program Name for untar",default = "")
        parser.add_option("-e", "--ext",dest = "ext",   action = "store",       type = "string",    help = "extension for untar",default = "")
        parser.add_option("-w", "--workingdir", dest = "workingDir",   action = "store",       type = "string",    help = "working dir for untar",default = "")
        parser.add_option("-P", "--paired", dest = "isPaired",   action = "store_true",    help = "Is Data is paired for untar",default = False)
        parser.add_option("-o", "--output", dest = "output",   action = "store",       type = "string",    help = "Output Name for your tar",default = False)
        (options, args) = parser.parse_args()
        self._setAttributesFromOptions(options)
        
    def _setAttributesFromOptions(self, options):
        self.setUntar(options.untar)
        self.setTar(options.tar)
        self.setPrgName(options.prgName)
        self.setWorkingDir(options.workingDir)
        self.setIsPaired(options.isPaired)
        self.setOutput(options.output)
        self.setExt(options.ext)
        
        
    def setExt(self, ext):
        self._ext = ext
        
    def setOutput(self, output):
        self._output = output      
                  
    def setUntar(self, untar):
        self._untar = untar
    
    def setTar(self, tar):
        self._tar = tar

        
    def setPrgName(self, prgName):
        self._prgName = prgName
    
    def setWorkingDir(self, workingDir):
        self._workingDir = workingDir
    
    def setIsPaired(self, isPaired):
        self._isPaired = isPaired
    
    def run(self):
        if self._tar != "":
            l2Tar = pickle.load(file(self._tar))
            self.tar(self._output, l2Tar)
            
        if self._untar != "":
            if self._isPaired == True:
                inputFileNames, inputRevFileNames, outputFileNames = self.untar(self._untar, self._workingDir, self._prgName, self._isPaired, self._ext)
                pickle.dump(inputFileNames, file("%s/inputFile.pickle" % self._workingDir, "w"))
                pickle.dump(inputRevFileNames, file("%s/inputRevFile.pickle" % self._workingDir, "w"))
                pickle.dump(outputFileNames, file("%s/outputFile.pickle" % self._workingDir, "w"))                
            else:
                inputFileNames, outputFileNames = self.untar(self._untar, self._workingDir, self._prgName, self._isPaired, self._ext)
                pickle.dump(inputFileNames, file("%s/inputFile.pickle" % self._workingDir, "w"))
                pickle.dump(outputFileNames, file("%s/outputFile.pickle" % self._workingDir, "w"))      
                
                
                
if __name__ == "__main__":
    iLaunch = manageTarForDEA()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()               
