#
# Copyright INRA-URGI 2009-2010
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
import re
import sys
from commons.core.parsing.MapperParser import MapperParser
from SMART.Java.Python.structure.Mapping import Mapping
from SMART.Java.Python.structure.SubMapping import SubMapping
from SMART.Java.Python.structure.Interval import Interval
from SMART.Java.Python.misc.UnlimitedProgress import UnlimitedProgress
import os

class Sim4Parser(MapperParser):
    """A class that parses the output of SIM4 format (of SSAHA and BLAT)"""

    def __init__(self, fileName, verbosity = 0):
        super(Sim4Parser, self).__init__(fileName, verbosity)
        self._queryInterval = None
        self._targetInterval = None
        
        self._currentMapping = None
        self._finishedMapping = None
        self._currentSubMappings = {}
        
        self._matchPartsFound = False
        self._alignementComplement = False
        
        self._matchinElementLineIndex = 0
        self._exactMatchesPerMatch = 0
        self._gapsPerMatch  = 0
        self._insertionsPerMatch  = 0
        self._deletionsPerMatch = 0
        self._lenalign = 0
        self._target_sequence = ""
        self._target_len = 0
        self._matchpart_indic = 0
        self._match_number = 0 
        self._match_part_number = 0
        
    def __del__(self):
        super(Sim4Parser, self).__del__()


    def getFileFormats():
        return ["sim4"]
    getFileFormats = staticmethod(getFileFormats)


    def getInfos(self):
        self.chromosomes = set()
        self.nbMappings  = 0
        self.size        = 0
        self.reset()
        progress = UnlimitedProgress(100000, "Getting info on SIM4 file, # mappings read:", self.verbosity)
        for line in self.handle:
            progress.inc()
            line = line.strip()
            if line == "":
                continue
            if "seq2" in line:
                self.nbMappings += 1
            #self.chromosomes.add(chromosome)
            #self.size += len(parts[0])
        self.reset()
        progress.done()


    def skipFirstLines(self):
        pass
    
    def getNextMapping(self):
        for line in self.handle:
            mapping = self.parseLine(line)
            self.currentLineNb += 1
            if mapping is not None:
                return mapping
        if self._currentMapping is not None and self._matchinElementLineIndex != 0:
            self._finishedMapping = self._currentMapping
            self._currentMapping = None
            return self._finalizeMapping()
        else:
            return self._resetflags()
            
        return False

    def _computeStarts(self,seqSize,blockSize,start,targetStrand):
        if targetStrand == "+":
            pass
        else:
            start = seqSize-blockSize-start
        return start


    def parseSequenceInfos(self, line, name):
        mappingInfoFields = line.replace(name+" = ","").replace(" ","").replace("bp","").rstrip().split(",")
        seqName,seqSize = mappingInfoFields
        if( "(" in seqName):
            seqName = seqName.split("(")[1].replace(")",'')
        seqName = os.path.basename(seqName)
        seqSize=int(seqSize)
        return seqName, seqSize

    def _finalizeMapping(self):
#         print("exact matches",self._exactMatchesPerMatch)
#         print("mis matches",self._gapsPerMatch)
#         print("insertions",self._insertionsPerMatch)
#         print("deletions",self._deletionsPerMatch)
        
        self._finishedMapping.setNbMismatches(self._gapsPerMatch+self._insertionsPerMatch+self._deletionsPerMatch)
        self._finishedMapping.setNbGaps(self._gapsPerMatch)
        #
        if self._alignementComplement:
            self._finishedMapping.setDirection("-")
        else:
            self._finishedMapping.setDirection("+")
        
        minQ = float("inf")
        maxQ = 0
        minT = float("inf")
        maxT = 0
        
        qAlignLen = 0
        sAlignLen = 0
        
        for subMapping in self._currentSubMappings.values():
            self._finishedMapping.addSubMapping(subMapping)
            qAlignLen+= subMapping.queryInterval.getSize()
            sAlignLen+= subMapping.targetInterval.getSize()
            
            #Query
            if subMapping.queryInterval.getStart()<minQ:
                minQ = subMapping.queryInterval.start
            if subMapping.queryInterval.getEnd()>maxQ:
                maxQ = subMapping.queryInterval.getEnd()

            # Target 
            if subMapping.targetInterval.getStart()<minT:
                minT = subMapping.targetInterval.getStart()
            if subMapping.targetInterval.getEnd()>maxT:
                maxT = subMapping.targetInterval.getEnd()
    
        
#         print("_matchinElementLineIndex", self._matchinElementLineIndex)
#        print("_exactMatchesPerMatch", self._exactMatchesPerMatch)
#        print("_gapsPerMatch", self._gapsPerMatch)
#        print("_insertionsPerMatch", self._insertionsPerMatch)
#        print("_deletionsPerMatch", self._deletionsPerMatch)
        
#         print("test_ligne len", self._lenalign)
#         print("test_ligne len s ", qAlignLen)
#         print("test_ligne len q ", sAlignLen)
#         
#         print("Target_Seq: ", self._target_sequence)

        seqLen = self._targetInterval.getTagValue("target_length")
        total_cover = (self._exactMatchesPerMatch+self._gapsPerMatch+self._insertionsPerMatch+self._deletionsPerMatch+0.0)
        pcoverage= total_cover/seqLen*100
        pidentity= (self._exactMatchesPerMatch+0.0) / total_cover*100
        pcoverage = round(pcoverage,2)
        pidentity = round(pidentity,2)
        nconserved= 0.00
#         print("seqLen", seqLen)
#         print("total_cover",total_cover)
#         print("target", self._targetInterval.name)
#         print("query", self._queryInterval.name)
#         
#         print("pcoverage", pcoverage)
#         print("identity", pidentity)
#         print("new_computing ", self._lenalign)
        
        if self._alignementComplement:
            
            self._finishedMapping.setTagValue("Target", self._targetInterval.name+" "+str(minQ)+" "+str(maxQ))
        else:
            self._finishedMapping.setTagValue("Target", self._targetInterval.name+" "+str(minQ)+" "+str(maxQ))
        
        self._finishedMapping.setTagValue("target_id", self._targetInterval.name)         
        self._finishedMapping.setTagValue("target_pident", "%.2f" % pidentity)
        self._finishedMapping.setTagValue("target_pcover", "%.2f" %pcoverage)
        self._finishedMapping.setTagValue("target_pcons","%.2f" %nconserved ) #TODO : how should this be computed
        self._finishedMapping.setTagValue("evalue", '') #TODO: if this is needed, find a way to inject it
        #print(self._match_number)
        self._finishedMapping.setTagValue("ID", "%s_%s_m%03d" %(self._queryInterval.name,self._targetInterval.name,self._match_number))
        #self._finishedMapping.setTagValue("Name", self._targetInterval.name+"_"+str(self._match_number))
        self._finishedMapping.setTagValue("target_length", seqLen)
        self._finishedMapping.setTagValue("lib", "") #TODO: if this is needed, find a way to inject it
        self._finishedMapping.setTagValue("program", "sim4")

        self._resetflags()
        self._finishedMapping.transcript = None
        #reset all flags
#         self._alignementComplement = False
#         self._matchPartsFound = False
#         self._matchinElementLineIndex = 0
#         
#         self._exactMatchesPerMatch = 0
#         self._gapsPerMatch  = 0
#         self._insertionsPerMatch  = 0
#         self._deletionsPerMatch = 0
#         self._lenalign = 0
#         self._currentSubMappings = {}
        if pcoverage>30 and pidentity>70:
            return self._finishedMapping 
        else:
            return None
    
    
    def _resetflags(self):
        #self._finishedMapping.transcript = None
        self._alignementComplement = False
        self._matchPartsFound = False
        self._matchinElementLineIndex = 0
        
        self._exactMatchesPerMatch = 0
        self._gapsPerMatch  = 0
        self._insertionsPerMatch  = 0
        self._deletionsPerMatch = 0
        self._lenalign = 0
        self._currentSubMappings = {}

    def parseLine(self, line):
        # While alignment didnt start we pass
        if(line==""):
            return
        # Each alignment start with seq1 => Flag initiation and closing previous mapping if existed
        if("seq1" in line):
            self._finishedMapping = self._currentMapping
            self._currentMapping = Mapping()
                
            # Basic sequence hit or query information
            seq_Infos = self.parseSequenceInfos(line,"seq1")
            self._currentMapping.setSize(int(seq_Infos[1]))
            
#             self._currentQueryName = seq_Infos[0]
#             if self._currentRegion != self._currentQueryName:
#                 self._currentRegion = self._currentQueryName
#                 #create new region

            # setting data for query interval object
            self._queryInterval = Interval()
            self._queryInterval.setChromosome(seq_Infos[0])
            self._queryInterval.setName(seq_Infos[0])

            if self._finishedMapping is not None:
                if self._matchinElementLineIndex!=0:
                    self._match_number+=1
                    self._match_part_number=0
                    #FINISHED the mapping, filled its data, return it if not empty
                    return self._finalizeMapping()
                else:
                    # Reset if empty
                    return self._resetflags()
            return
            
        if("seq2" in line ):
            # info on target sequence set in interval object
            seq_Infos = self.parseSequenceInfos(line,"seq2")       
            self._targetInterval = Interval()
            self._targetInterval.setChromosome( seq_Infos[0] )
            self._targetInterval.setName(seq_Infos[0])
            self._targetInterval.setTagValue("target_length", seq_Infos[1])
            return
    
        # If complement found then setting to true
        if "(complement)" in line:
            self._alignementComplement = True
            return
    
        if not self._matchPartsFound:
            # regexp to find exons positions info 
            m = re.search(r"^\s*(\d+)\-(\d+)\s+\((\d+)\-(\d+)\).*$", line)
            if m != None:
                # Query 
                qStart = int(m.group(1))
                qEnd = int(m.group(2))
                
                # Target 
                tStart = int(m.group(3))
                tEnd = int(m.group(4))
                
                if self._alignementComplement:
                    tLen = self._targetInterval.getTagValue("target_length")
                    tEndold = tEnd
                    tEnd = tLen-tStart + 1
                    tStart = tLen-tEndold + 1
                    
                #create a new match part
                subMapping = SubMapping()
                self._match_part_number+=1
                
                queryInterval  = Interval()
                targetInterval = Interval()
    
                #queryInterval.setName( self._queryInterval.name)
                queryInterval.setChromosome( self._queryInterval.getChromosome() )
                queryInterval.setStart(qStart)
                queryInterval.setEnd(qEnd)
                
                if self._alignementComplement:
                    queryInterval.setDirection("-")
                else:
                    queryInterval.setDirection("+")
    
                targetInterval.setName( self._targetInterval.name )
                targetInterval.setChromosome( self._queryInterval.name )
                targetInterval.setStart(tStart)
                targetInterval.setEnd(tEnd)
                
                queryInterval.setTagValue("Target", targetInterval.name +" "+str(targetInterval.getStart())+" "+str(targetInterval.getEnd()))
                #queryInterval.tags["Parent"] = self._queryInterval.name+ self._targetInterval.name+"_1"
                queryInterval.setTagValue("ID", "%s_%s_m%03d_mp%03d" %(self._queryInterval.name,self._targetInterval.name ,self._match_number,self._match_part_number ))
                #queryInterval.setTagValue("Name", self._queryInterval.name+"_"+ self._targetInterval.name+"_m"+str(self._match_number))
                #print("targetInterval", queryInterval.getTagNames())
                #queryInterval.setTagValue("Parent", "%s_%s_m%03d" %(self._queryInterval.name, self._targetInterval.name,self._match_number))
                subMapping.setQueryInterval(targetInterval)
                subMapping.setTargetInterval(queryInterval )
                
                
                self._currentSubMappings["%s_%s:%s_%s" % (qStart,qEnd,tStart,tEnd)] = subMapping
                
                
        if "." in line and ":" in line:
            self._matchPartsFound = True
        
        if self._matchPartsFound:
            match_part = 0
            cleanedLine = line.lstrip().rstrip()
            lineElems = cleanedLine.split()
            test_line = line[8:-1]
            
            if self._matchinElementLineIndex % 5 == 1:
                #print("we are on a query line", line)
                qStart = int(lineElems[0])
                qSeq = " ".join( lineElems[1:] )
                
                #print("query ",qStart, qSeq)
            
            if self._matchinElementLineIndex % 5 == 2:
#                print("line: " ,test_line)
                self._lenalign +=len(test_line)
        
                self._exactMatchesPerMatch += test_line.count("|")
#                 print("line", test_line)
                
                self._gapsPerMatch += test_line.count(" ") 
                self._insertionsPerMatch += test_line.count("-")
#                print("inser_del", test_line.count("-"))
                
                chevronop=test_line.count(">")
                chevronfe=test_line.count("<")
                point=test_line.count(".")
                match_part=chevronop+chevronfe+point
                siEgale=test_line.count("=")
            
                self._matchpart_indic=match_part+siEgale
                
                
                self._deletionsPerMatch += test_line.count("+")
                #print("we are on a pipe line", line)
                
                #print("somme: ",test_line.count("|") +  test_line.count(" ")+test_line.count("-"))
                #print("expected len :" , len(test_line))
                
            if self._matchinElementLineIndex % 5 == 3:
                #print("we are on a target line", line)
                tStart = int(lineElems[0])
                tSeq = " ".join( lineElems[1:] )
                self._target_sequence+=test_line
#                 print("taille tot: ", len(test_line))
#                 print("taille cor: ", len(test_line.replace(" ", "")))
#                 print("taille_mp: ", self._matchpart_indic)
                
                target_delta = len(test_line) - len(test_line.replace(" ", "")) - self._matchpart_indic
#                 print("match_part_indic",self._matchpart_indic)
                self._insertionsPerMatch= self._insertionsPerMatch-target_delta
#                 print("inserbefore: ", self._insertionsPerMatch)
#                 print("inserafter: ", self._insertionsPerMatch-target_delta)
                self._target_len+=len(test_line.replace(" ", ""))
                
                #print("target ",tStart, tSeq)
            self._matchinElementLineIndex +=1

            

            
        return
        #plus or minus strand
#         ->  == + 
#         <-  == -
        #compute coverage etc
#         my $coverage  = grep { !/0/  } @{$hits->{sequence}};
#         my $identity  = grep { /[3]/ } @{$hits->{sequence}};
#         my $conserved = grep { /[2]/ } @{$hits->{sequence}};
#         
#         pcoverage, pidentity, nconserved
#         return(($coverage / $length) * 100, ($identity / $coverage) * 100, ($conserved / $coverage) * 100);
    
        #structure for matches
#         (complement)
#         8610-8955  (40-385)   96% <-
#         9696-9770  (386-460)   100% <-
#         15124-15193  (461-530)   97%
