import unittest
import os
from commons.core.parsing.VarscanToVCF import VarscanToVCF
from commons.core.utils.FileUtils import FileUtils

class Test_F_VarscanToVCF(unittest.TestCase):
    
    def setUp(self):
        self.emptyVarscanFileName = "emptyfile.varscan"
        self.varscanFileName = "%s/commons/core/parsing/test/data/sampleForTestVarscanToVCF.varscan" % os.environ["REPET_PATH"]
        self.expVCFFileName = "%s/commons/core/parsing/test/data/expSampleForTestVarscanToVCF.vcf" % os.environ["REPET_PATH"]
        self.obsVCFFileName = "obsVCF.vcf"
        
    def tearDown(self):
        if os.path.exists(self.emptyVarscanFileName):
            os.remove(self.emptyVarscanFileName)
        os.remove(self.obsVCFFileName)
        
    def test_F_VarscanToVCF(self):
        iVarscanFile = VarscanToVCF(self.varscanFileName, self.obsVCFFileName, doClean = True, verbosity = 5)
        iVarscanFile.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self.expVCFFileName, self.obsVCFFileName))
        
    def test_F_VarscanToVCF_no_header_in_varscan(self):
        self.varscanFileName = "%s/commons/core/parsing/test/data/sampleForTestVarscanToVCF_noHeader.varscan" % os.environ["REPET_PATH"]
        self.expVCFFileName = "%s/commons/core/parsing/test/data/expSampleForTestVarscanToVCF_noHeader.vcf" % os.environ["REPET_PATH"]
        iVarscanFile = VarscanToVCF(self.varscanFileName, self.obsVCFFileName, doClean = True, verbosity = 5)
        iVarscanFile.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self.expVCFFileName, self.obsVCFFileName))
       
    def test_F_VarscanToVCF_empty(self):
        self.expVCFFileName = "expEmptyVCF.vcf"
        self._writeInputFile_empty()
        self._writeExpOutputFile_empty()
        iVarscanFile = VarscanToVCF(self.emptyVarscanFileName, self.obsVCFFileName, doClean = True, verbosity = 5)
        iVarscanFile.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self.expVCFFileName, self.obsVCFFileName))
        os.remove(self.expVCFFileName)
           
    def _writeInputFile_empty(self):
        with open(self.emptyVarscanFileName, "w") as varscanFileName:
            varscanFileName.write("Chrom\tPosition\tRef\tCons\tReads1\tReads2\tVarFreq\tStrands1\tStrands2\tQual1\tQual2\tPvalue\tMapQual1\tMapQual2\tReads1Plus\tReads1Minus\tReads2Plus\tReads2Minus\tVarAllele\n")
            
    def _writeExpOutputFile_empty(self):
        with open(self.expVCFFileName, "w") as vcfFileName:
            vcfFileName.write("##fileformat=VCFv4.1\n")
            vcfFileName.write("##qual=10000 if varscan p-value=0\n")
            vcfFileName.write("##program: VarscanToVCF.py input_file=emptyfile.varscan\n")
            vcfFileName.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n")
   
if __name__ == "__main__":
    unittest.main()
    