from commons.core.parsing.Sim4Parser import Sim4Parser
import unittest, os
from SMART.Java.Python.convertTranscriptFile import ConvertTranscriptFile
from commons.core.writer.Gff3Writer import Gff3Writer


class Test_Sim4Parser(unittest.TestCase):

    def test_simple(self):
        fileName = "data/sim4/input_simple.sim4"
    
        parser = Sim4Parser(fileName, 0)
        self.assertEquals(parser.getNbMappings(), 1)
            
        for mapping in parser.getIterator():
            transcript = mapping.getTranscript()
              
            self.assertEquals(transcript.getName(), "OCV4_rep_c21864")
            self.assertEquals(transcript.getChromosome(), "chunk010081")
            self.assertEquals(transcript.getDirection(), 1)
            self.assertEquals(transcript.getStart(), 1)
            self.assertEquals(transcript.getEnd(),   1357)
            self.assertEquals(transcript.getNbExons(), 2)
            for i, exon in enumerate(transcript.getExons()):
                if i == 0:
                    self.assertEquals(exon.getStart(), 1)
                    self.assertEquals(exon.getEnd(),   307)
                elif i == 1:
                    self.assertEquals(exon.getStart(), 428)
                    self.assertEquals(exon.getEnd(),   1357)
    def test_multiple(self):
        fileName = "data/sim4/input_multiple.sim4"
   
        parser = Sim4Parser(fileName, 0)
        self.assertEquals(parser.getNbMappings(), 2)
         
        for index, mapping in enumerate( parser.getIterator()):
            transcript = mapping.getTranscript()
             
            if index == 0:
                self.assertEquals(transcript.getName(), "OCV4_rep_c5590")
                self.assertEquals(transcript.getChromosome(), "chunk015309")
                self.assertEquals(transcript.getDirection(), -1)
                self.assertEquals(transcript.getStart(), 8610)
                self.assertEquals(transcript.getEnd(),   15193)
                self.assertEquals(transcript.getNbExons(), 3)
                for i, exon in enumerate(transcript.getExons()):
                    if i == 0:
                        self.assertEquals(exon.getStart(), 15124)
                        self.assertEquals(exon.getEnd(),   15193)
                    elif i == 1:
                        self.assertEquals(exon.getStart(), 9696)
                        self.assertEquals(exon.getEnd(),   9770)
            else:
                self.assertEquals(transcript.getName(), "Loc_7915_Tr_7_13_Conf_0.271_Len_2068")
                self.assertEquals(transcript.getChromosome(), "chunk015309")
                self.assertEquals(transcript.getDirection(), -1)
                self.assertEquals(transcript.getStart(), 8545)
                self.assertEquals(transcript.getEnd(),   15193)
                self.assertEquals(transcript.getNbExons(), 2)
                for i, exon in enumerate(transcript.getExons()):
                    if i == 0:
                        self.assertEquals(exon.getStart(), 15124)
                        self.assertEquals(exon.getEnd(),   15193)
                    elif i == 1:
                        self.assertEquals(exon.getStart(), 8545)
                        self.assertEquals(exon.getEnd(),   9770)
            

    
if __name__ == "__main__":
    unittest.main()


