import os
import unittest
from commons.core.parsing.VarscanToVCF import VarscanToVCF

class Test_VarscanToVCF(unittest.TestCase):
    
    def setUp(self):
        self._iVarscanToVCF = VarscanToVCF(doClean = True, verbosity = 2)
    
    def test_convertVarscanLineToVCFLine(self):
        varscanLine = "chr1\t10\tC\tY\t1\t1\t50%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tT"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\tT\t.\t.\tAF=0.5000;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)
        
    def test_convertVarscanLineToVCFLine_false_VarAllele(self):
        varscanLine = "chr1\t10\tC\tY\t1\t1\t50%\t1\t1\t68\t69\t0.49999999999999994\t1\t1\t1\t0\t1\t0\tA"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\tT\t3.010299957\t.\tAF=0.5000;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertVarscanLineToVCFLine_no_IUPAC(self):
        varscanLine = "chr1\t10\tC\tT\t1\t1\t75%\t1\t1\t68\t69\t2.8282406617856754E-8\t1\t1\t1\t0\t1\t0\tT"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\tT\t75.484836381\t.\tAF=0.7500;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertVarscanLineToVCFLine_p_value_0_98(self):
        varscanLine = "chr1\t10\tC\tT\t1\t1\t75%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tT"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\tT\t.\t.\tAF=0.7500;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertVarscanLineToVCFLine_del(self):
        varscanLine = "chr1\t10\tC\tT\t1\t1\t75%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\t-TTT"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\t-TTT\t.\t.\tAF=0.7500;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertVarscanLineToVCFLine_p_value_0(self):
        varscanLine = "chr1\t10\tC\tT\t1\t1\t75%\t1\t1\t68\t69\t0.0\t1\t1\t1\t0\t1\t0\tT"
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord([varscanLine], [1])
        
        expVCFLine = "chr1\t10\t.\tC\tT\t10000\t.\tAF=0.7500;DP=2;RBQ=68;ABQ=69\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertTwoVarscanLineToVCFLine_ATCG_indel_and_SNP(self):
        varscanLine1 = "chr1\t10\tC\tT\t1\t5\t80%\t1\t1\t68\t69\t0\t1\t1\t1\t0\t1\t0\tT"
        varscanLine2 = "chr1\t10\tC\tA\t1\t3\t75%\t1\t1\t68\t70\t0\t1\t1\t1\t0\t1\t0\t-TT"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tC\tT,-TT\t10000\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertTwoVarscanLineToVCFLine_ATCG_pvalue_0(self):
        varscanLine1 = "chr1\t10\tC\tT\t1\t5\t80%\t1\t1\t68\t69\t0\t1\t1\t1\t0\t1\t0\tT"
        varscanLine2 = "chr1\t10\tC\tA\t1\t3\t75%\t1\t1\t68\t70\t0\t1\t1\t1\t0\t1\t0\tT"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tC\tT,A\t10000\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)

    def test_convertTwoVarscanLineToVCFLine_ATCG(self):
        varscanLine1 = "chr1\t10\tC\tT\t1\t5\t80%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tT"
        varscanLine2 = "chr1\t10\tC\tA\t1\t3\t75%\t1\t1\t68\t70\t0.98\t1\t1\t1\t0\t1\t0\tT"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tC\tT,A\t.\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)
        
    def test_convertTwoVarscanLineToVCFLine_IUPAC_first(self):
        varscanLine1 = "chr1\t10\tT\tW\t1\t5\t70%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tA"
        varscanLine2 = "chr1\t10\tT\tC\t1\t3\t80%\t1\t1\t68\t70\t0.98\t1\t1\t1\t0\t1\t0\tA"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tT\tA,C\t.\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)
        
    def test_convertTwoVarscanLineToVCFLine_IUPAC_second(self):
        varscanLine1 = "chr1\t10\tT\tA\t1\t5\t80%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tA"
        varscanLine2 = "chr1\t10\tT\tY\t1\t3\t73%\t1\t1\t68\t70\t0.98\t1\t1\t1\t0\t1\t0\tA"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tT\tA,C\t.\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)
        
    def test_convertTwoVarscanLineToVCFLine_IUPAC_both(self):
        varscanLine1 = "chr1\t10\tT\tW\t1\t5\t70%\t1\t1\t68\t69\t0.98\t1\t1\t1\t0\t1\t0\tA"
        varscanLine2 = "chr1\t10\tT\tY\t1\t3\t73%\t1\t1\t68\t70\t0.98\t1\t1\t1\t0\t1\t0\tA"
        lVarscanLine = [varscanLine1, varscanLine2]
        obsVCFLine = self._iVarscanToVCF._convertVarscanLineToVCFRecord(lVarscanLine, [1,2])
        
        expVCFLine = "chr1\t10\t.\tT\tA,C\t.\t.\tAF=0.5556,0.3333;DP=9;RBQ=68;ABQ=69,70\n"
        self.assertEqual(expVCFLine, obsVCFLine)
        
    def test_hasVarscanHeader_True(self):
        varscanTestFile = "varscan.tmp"
        with open(varscanTestFile, "w") as f:
            f.write("Chrom\tPosition\tRef\tCons\tReads1\tReads2\tVarFreq\tStrands1\tStrands2\tQual1\tQual2\tPvalue\tMapQual1\tMapQual2\tReads1Plus\tReads1Minus\tReads2Plus\tReads2Minus\tVarAllele")
        self._iVarscanToVCF.setVarscanFileName(varscanTestFile)
        
        obsBoolean = self._iVarscanToVCF._hasVarscanHeader()
        self.assertTrue(obsBoolean)
        
    def test_hasVarscanHeader_True_newline(self):
        varscanTestFile = "varscan.tmp"
        with open(varscanTestFile, "w") as f:
            f.write("Chrom\tPosition\tRef\tCons\tReads1\tReads2\tVarFreq\tStrands1\tStrands2\tQual1\tQual2\tPvalue\tMapQual1\tMapQual2\tReads1Plus\tReads1Minus\tReads2Plus\tReads2Minus\tVarAllele\n")
        self._iVarscanToVCF.setVarscanFileName(varscanTestFile)
        
        obsBoolean = self._iVarscanToVCF._hasVarscanHeader()
        os.remove(varscanTestFile)
        self.assertTrue(obsBoolean)
        
    def test_hasVarscanHeader_True_newline_and_sharp(self):
        varscanTestFile = "varscan.tmp"
        with open(varscanTestFile, "w") as f:
            f.write("#Chrom\tPosition\tRef\tCons\tReads1\tReads2\tVarFreq\tStrands1\tStrands2\tQual1\tQual2\tPvalue\tMapQual1\tMapQual2\tReads1Plus\tReads1Minus\tReads2Plus\tReads2Minus\tVarAllele\n")
        self._iVarscanToVCF.setVarscanFileName(varscanTestFile)
        
        obsBoolean = self._iVarscanToVCF._hasVarscanHeader()
        os.remove(varscanTestFile)
        self.assertTrue(obsBoolean)
        
    def test_hasVarscanHeader_False(self):
        varscanTestFile = "varscan.tmp"
        with open(varscanTestFile, "w"):
            pass
        self._iVarscanToVCF.setVarscanFileName(varscanTestFile)
        
        obsBoolean = self._iVarscanToVCF._hasVarscanHeader()
        os.remove(varscanTestFile)
        self.assertFalse(obsBoolean)

    def test_convertVarscanLineToVCFRecord_empty_line(self):
        obsMsg = ""
        try:
            self._iVarscanToVCF._convertVarscanLineToVCFRecord([""], [10])
        except Exception as e:
            obsMsg = e.msg
        
        self.assertEqual("This varscan line (l.10) is not complete", obsMsg)

if __name__ == "__main__":
    unittest.main()