# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.checker.RepetException import RepetException
from commons.core.sql.JobAdaptator import JobAdaptatorSGE
from commons.core.sql.JobAdaptator import JobAdaptatorSLURM
from commons.core.sql.JobAdaptator import JobAdaptatorCOMPSs
from commons.core.sql.JobAdaptator import JobAdaptatorTorque
from commons.core.sql.JobAdaptator import JobAdaptatorRabbitMQ
from commons.core.sql.TableJobAdaptator import TableJobAdaptatorSGE
from commons.core.sql.TableJobAdaptator import TableJobAdaptatorSLURM
from commons.core.sql.TableJobAdaptator import TableJobAdaptatorTorque
from commons.core.sql.TableJobAdaptator import TableJobAdaptatorCOMPSs
from commons.core.sql.TableJobAdaptator import TableJobAdaptatorRabbitMQ

SUPPORTED_JOB_MANAGERS_LIST = ["SGE", "Torque", "SLURM", "RabbitMQ", "COMPSs"]

class TableJobAdaptatorFactory(object):

    @staticmethod
    def createInstance(iDb, jobTableName):
        jobManager = os.environ["REPET_JOB_MANAGER"].lower().replace(":", "")
        
        if not TableJobAdaptatorFactory.isJobManagerSupported(jobManager):
            raise RepetException("ERROR: unknown Job Manager : $REPET_JOB_MANAGER = %s" % os.environ["REPET_JOB_MANAGER"])
            
        iTJA = None
        if jobManager == "sge":
            iTJA = TableJobAdaptatorSGE(iDb, jobTableName)
        elif jobManager == "torque":
            iTJA = TableJobAdaptatorTorque(iDb, jobTableName)
        elif jobManager == "slurm":
            iTJA = TableJobAdaptatorSLURM(iDb, jobTableName)
        elif jobManager == "rabbitmq":
            iTJA = TableJobAdaptatorRabbitMQ(iDb, jobTableName)
        elif jobManager == "compss":
            iTJA = TableJobAdaptatorCOMPSs(iDb, jobTableName)
            
        return iTJA
    
    @staticmethod
    def createJobInstance():
        jobManager = os.environ["REPET_JOB_MANAGER"].lower().replace(":", "")
        
        if not TableJobAdaptatorFactory.isJobManagerSupported(jobManager):
            raise RepetException("ERROR: unknown Job Manager : $REPET_JOB_MANAGER = %s" % os.environ["REPET_JOB_MANAGER"])
            
        iJA = None
        if jobManager == "sge":
            iJA = JobAdaptatorSGE()
        elif jobManager == "torque":
            iJA = JobAdaptatorTorque()
        elif jobManager == "slurm":
            iJA = JobAdaptatorSLURM()
        elif jobManager == "rabbitmq":
            iJA = JobAdaptatorRabbitMQ()
        elif jobManager == "compss":
            iJA = JobAdaptatorCOMPSs()
            
        return iJA
       
    @staticmethod
    def getJobManagersList():
        return SUPPORTED_JOB_MANAGERS_LIST
    
    @staticmethod
    def isJobManagerSupported(jobManager):
        return jobManager.replace(":", "").lower() in [jm.lower() for jm in TableJobAdaptatorFactory.getJobManagersList()]
