import os
import unittest
from commons.core.checker.RepetException import RepetException
from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory

class Test_TableJobAdaptatorFactory(unittest.TestCase):

    def test_createInstance_SGE(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "SGE:"
        instance = TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        obsClassName = instance.__class__.__name__
        expClassName = "TableJobAdaptatorSGE"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createInstance_Torque(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "ToRquE"
        instance = TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        obsClassName = instance.__class__.__name__
        expClassName = "TableJobAdaptatorTorque"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createInstance_SLURM(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "SlUrM"
        instance = TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        obsClassName = instance.__class__.__name__
        expClassName = "TableJobAdaptatorSLURM"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createInstance_RabbitMQ(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "raBbItMq"
        instance = TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        obsClassName = instance.__class__.__name__
        expClassName = "TableJobAdaptatorRabbitMQ"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createInstance_COMPSs(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "cOmPSs"
        instance = TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        obsClassName = instance.__class__.__name__
        expClassName = "TableJobAdaptatorCOMPSs"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createInstance_Error(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "PBS"
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            TableJobAdaptatorFactory.createInstance(DbFactory.createInstance(), "dummyJobTable")
        except RepetException as e:
            isExceptionRaised = True
            obsMessage = e.getMessage()
            
        expMessage = "ERROR: unknown Job Manager : $REPET_JOB_MANAGER = PBS"
        
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, expMessage)
        
    def test_createJobInstance_SGE(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "SGE:"
        instance = TableJobAdaptatorFactory.createJobInstance()
        obsClassName = instance.__class__.__name__
        expClassName = "JobAdaptatorSGE"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createJobInstance_Torque(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "ToRquE"
        instance = TableJobAdaptatorFactory.createJobInstance()
        obsClassName = instance.__class__.__name__
        expClassName = "JobAdaptatorTorque"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createJobInstance_SLURM(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "SlUrM"
        instance = TableJobAdaptatorFactory.createJobInstance()
        obsClassName = instance.__class__.__name__
        expClassName = "JobAdaptatorSLURM"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createJobInstance_RabbitMQ(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "raBbItMq"
        instance = TableJobAdaptatorFactory.createJobInstance()
        obsClassName = instance.__class__.__name__
        expClassName = "JobAdaptatorRabbitMQ"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createJobInstance_COMPSs(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "cOmPSs"
        instance = TableJobAdaptatorFactory.createJobInstance()
        obsClassName = instance.__class__.__name__
        expClassName = "JobAdaptatorCOMPSs"
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertEquals(expClassName, obsClassName)

    def test_createJobInstance_Error(self):
        REPET_JOB_MANAGER_Initial_Value = os.environ["REPET_JOB_MANAGER"]
        os.environ["REPET_JOB_MANAGER"] = "PBS"
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            TableJobAdaptatorFactory.createJobInstance()
        except RepetException as e:
            isExceptionRaised = True
            obsMessage = e.getMessage()
            
        expMessage = "ERROR: unknown Job Manager : $REPET_JOB_MANAGER = PBS"
        
        os.environ["REPET_JOB_MANAGER"] = REPET_JOB_MANAGER_Initial_Value
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, expMessage)
        
    def test_getJobManagersList(self):
        lExpJobManagers = ["SGE", "Torque", "SLURM", "RabbitMQ", "COMPSs"]
        lObsJobManagers = TableJobAdaptatorFactory.getJobManagersList()
        
        self.assertEquals(lObsJobManagers, lExpJobManagers)

    def test_isJobManagerSupported_True(self):
        self.assertTrue(TableJobAdaptatorFactory.isJobManagerSupported("sGe:"))
        
    def test_isJobManagerSupported_False(self):
        self.assertFalse(TableJobAdaptatorFactory.isJobManagerSupported("PBS"))

if __name__ == "__main__":
    unittest.main()