# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import unittest
import os
import time
from collections import OrderedDict
from commons.core.utils.ClassifUtils import ClassifUtils
from commons.core.utils.FileUtils import FileUtils


class Test_ClassifUtils(unittest.TestCase):
    
    def setUp(self):
        self._uniqId = "%s_%s" % (time.strftime("%Y%m%d%H%M%S") , os.getpid())
        
    def tearDown( self ):
        pass
    
    def test_getClassifLineAsDict_miscellaneous(self):
        line = "DHX-comp_Azer\t1234\t.\tok\tII\tHelitron\tcomplete\t\
CI=99; coding=(TE_BLRtx: Foo:Helitron: 86.84%, Bar:Helitron: 80.12%; \
TE_BLRx: Foo:Gypsy: 72.87%; \
profiles: _Foo_Profile: 94.23%(94.23%)); \
struct=(TElength: >4000bps; TermRepeats: termLTR: 290; ORF: >1000bps GAG, >1000bps AP; helitronExtremities: Foo:classII:Helitron: (2e-39, 1, 82)); \
other=(Other_profiles: Foo.Bar_OTHER_P: 188.12%(99.81%); SSRCoverage=0.18); \
like=(bestHitName:DHX-incomp_Qwer_reversed CI:78)\n"
            
        preExpDict = [("name", "DHX-comp_Azer"),
                        ("wCode", "DHX"),
                        ("length", 1234),
                        ("strand", "."),
                        ("chimeric", False),
                        ("class", "II"),
                        ("order", "Helitron"),
                        ("complete", True),
                        ("CI", 99),
                        ("coding", OrderedDict([
                            ("TE_BLRtx", OrderedDict([
                                ("Foo:Helitron", OrderedDict([
                                    ("cov", 86.84)
                                ])),
                                ("Bar:Helitron", OrderedDict([
                                    ("cov", 80.12)
                                ]))
                            ])),
                            ("TE_BLRx", OrderedDict([
                                ("Foo:Gypsy", OrderedDict([
                                    ("cov", 72.87)
                                ]))
                            ])),
                            ("profiles", OrderedDict([
                                ("_Foo_Profile", OrderedDict([
                                    ("cov", 94.23),
                                    ("covOnSubject", 94.23)
                                ]))
                            ]))
                        ])),
                        ("struct", OrderedDict([
                            ("TElength", ">4000bps"),
                            ("TermRepeats", OrderedDict([
                                ("termLTR", 290)
                            ])),
                            ("ORF", [">1000bps GAG", ">1000bps AP"]),
                            ("helitronExtremities", OrderedDict([
                                ("Foo:classII:Helitron", OrderedDict([
                                    ("start", 1),
                                    ("end", 82),
                                    ("eValue", 2e-39)
                                ]))
                            ]))
                        ])),
                        ("other", OrderedDict([
                            ("Other_profiles", OrderedDict([
                                ("Foo.Bar_OTHER_P", OrderedDict([
                                    ("cov", 188.12),
                                    ("covOnSubject", 99.81)
                                ]))
                            ])),
                            ("SSRCoverage", 0.18)
                        ]))
                    ]
        
        expDict = OrderedDict(preExpDict)
        obsDict = ClassifUtils.getClassifLineAsDict(line)
        
        self.assertEqual(expDict, obsDict)
    
    def test_getClassifLineAsDict_coding(self):
        line = "RLX-incomp_Azer\t1234\t+\tok\tI\tLTR\tincomplete\t\
CI=99; coding=(TE_BLRtx: Foo:Helitron: 86.84%, Bar:Helitron: 80.12%; TE_BLRx: Foo:Gypsy: 72.87%; \
profiles: _Foo_Profile: 94.23%(94.23%); Other_profiles: PF01__OTHER_25.0: 100.00%(100.00%); \
rDNA_BLRn: embl|GQ167354|GQ167354 Uncultured eukaryote isolate DGGE gel band B2 18S ribosomal RNA, partial sequence: 100.00%; \
HG_BLRn: Dummy_match: 98.04%)\n"
        
        preExpDict = [("name", "RLX-incomp_Azer"),
                        ("wCode", "RLX"),
                        ("length", 1234),
                        ("strand", "+"),
                        ("chimeric", False),
                        ("class", "I"),
                        ("order", "LTR"),
                        ("complete", False),
                        ("CI", 99),
                        ("coding", OrderedDict([
                            ("TE_BLRtx", OrderedDict([
                                ("Foo:Helitron", OrderedDict([
                                    ("cov", 86.84)
                                ])),
                                ("Bar:Helitron", OrderedDict([
                                    ("cov", 80.12)
                                ]))
                            ])),
                            ("TE_BLRx", OrderedDict([
                                ("Foo:Gypsy", OrderedDict([
                                    ("cov", 72.87)
                                ]))
                            ])),
                            ("profiles", OrderedDict([
                                ("_Foo_Profile", OrderedDict([
                                    ("cov", 94.23),
                                    ("covOnSubject", 94.23)
                                ]))
                            ])),
                            ("Other_profiles", OrderedDict([
                                ("PF01__OTHER_25.0", OrderedDict([
                                    ("cov", 100.00),
                                    ("covOnSubject", 100.00)
                                ]))
                            ])),
                            ("rDNA_BLRn", OrderedDict([
                                ("name", "embl|GQ167354|GQ167354 Uncultured eukaryote isolate DGGE gel band B2 18S ribosomal RNA, partial sequence"),
                                ("cov", 100.0)
                            ])),
                            ("HG_BLRn", OrderedDict([
                                ("name", "Dummy_match"),
                                ("cov", 98.04)
                            ]))
                        ])),
                        ("struct", OrderedDict()),
                        ("other", OrderedDict())
                    ]
        
        expDict = OrderedDict(preExpDict)
        obsDict = ClassifUtils.getClassifLineAsDict(line)
        
        self.assertEqual(expDict, obsDict)
        
    
    def test_getClassifLineAsDict_struct(self):
        line = "DHX-comp_Qwer\t1234\t.\tok\tII\tHelitron\tcomplete\t\
CI=99; struct=(TElength: >4000bps; TermRepeats: termLTR: 290; ORF: >1000bps GAG, >1000bps AP; \
SSR: (AT)30_end; SSRCoverage=0.06; polyAtail; helitronExtremities: Foo:classII:Helitron: (2e-39, 1, 82))\n"
            
        preExpDict = [("name", "DHX-comp_Qwer"),
                        ("wCode", "DHX"),
                        ("length", 1234),
                        ("strand", "."),
                        ("chimeric", False),
                        ("class", "II"),
                        ("order", "Helitron"),
                        ("complete", True),
                        ("CI", 99),
                        ("coding", OrderedDict()),
                        ("struct", OrderedDict([
                            ("TElength", ">4000bps"),
                            ("TermRepeats", OrderedDict([
                                ("termLTR", 290)
                            ])),
                            ("ORF", [">1000bps GAG", ">1000bps AP"]),
                            ("SSR", ["(AT)30_end"]),
                            ("SSRCoverage", 0.06),
                            ("polyAtail", True),
                            ("helitronExtremities", OrderedDict([
                                ("Foo:classII:Helitron", OrderedDict([
                                    ("start", 1),
                                    ("end", 82),
                                    ("eValue", 2e-39)
                                ]))
                            ]))
                        ])),
                        ("other", OrderedDict())
                    ]
        
        expDict = OrderedDict(preExpDict)
        obsDict = ClassifUtils.getClassifLineAsDict(line)

        self.assertEqual(expDict, obsDict)
    
    def test_getClassifLineAsDict_other(self):
        line = "DHX-comp_Abcd\t1234\t-\tchimeric\tII\tHelitron\tcomplete\t\
CI=99; other=(TE_BLRtx: Foo:Helitron: 86.84%, Bar:Helitron: 80.12%; TE_BLRx: Foo:Gypsy: 72.87%; \
profiles: _Foo_Profile: 94.23%(94.23%); Other_profiles: PF01__OTHER_25.0: 100.00%(100.00%); \
rDNA_BLRn: embl|GQ167354|GQ167354 Uncultured eukaryote isolate DGGE gel band B2 18S ribosomal RNA, partial sequence: 100.00%; \
HG_BLRn: Dummy_match: 98.04%; TElength: >4000bps; TermRepeats: termLTR: 290; ORF: >1000bps GAG, >1000bps AP; \
SSR: (AT)30_end; SSRtrf: (AT)30_end; SSRCoverage=0.06; polyAtail; helitronExtremities: Foo:classII:Helitron: (2e-39, 1, 82))\n"
            
        preExpDict = [("name", "DHX-comp_Abcd"),
                        ("wCode", "DHX"),
                        ("length", 1234),
                        ("strand", "-"),
                        ("chimeric", True),
                        ("class", "II"),
                        ("order", "Helitron"),
                        ("complete", True),
                        ("CI", 99),
                        ("coding", OrderedDict()),
                        ("struct", OrderedDict()),
                        ("other", OrderedDict([
                            ("TE_BLRtx", OrderedDict([
                                ("Foo:Helitron", OrderedDict([
                                    ("cov", 86.84)
                                ])),
                                ("Bar:Helitron", OrderedDict([
                                    ("cov", 80.12)
                                ]))
                            ])),
                            ("TE_BLRx", OrderedDict([
                                ("Foo:Gypsy", OrderedDict([
                                    ("cov", 72.87)
                                ]))
                            ])),
                            ("profiles", OrderedDict([
                                ("_Foo_Profile", OrderedDict([
                                    ("cov", 94.23),
                                    ("covOnSubject", 94.23)
                                ]))
                            ])),
                            ("Other_profiles", OrderedDict([
                                ("PF01__OTHER_25.0", OrderedDict([
                                    ("cov", 100.00),
                                    ("covOnSubject", 100.00)
                                ]))
                            ])),
                            ("rDNA_BLRn", OrderedDict([
                                ("name", "embl|GQ167354|GQ167354 Uncultured eukaryote isolate DGGE gel band B2 18S ribosomal RNA, partial sequence"),
                                ("cov", 100.0)
                            ])),
                            ("HG_BLRn", OrderedDict([
                                ("name", "Dummy_match"),
                                ("cov", 98.04)
                            ])),
                            ("TElength", ">4000bps"),
                            ("TermRepeats", OrderedDict([
                                ("termLTR", 290)
                            ])),
                            ("ORF", [">1000bps GAG", ">1000bps AP"]),
                            ("SSR", ["(AT)30_end"]),
                            ("SSRtrf", ["(AT)30_end"]),
                            ("SSRCoverage", 0.06),
                            ("polyAtail", True),
                            ("helitronExtremities", OrderedDict([
                                ("Foo:classII:Helitron", OrderedDict([
                                    ("start", 1),
                                    ("end", 82),
                                    ("eValue", 2e-39)
                                ]))
                            ]))
                        ]))
                    ]
        
        expDict = OrderedDict(preExpDict)
        obsDict = ClassifUtils.getClassifLineAsDict(line)
        
        self.assertEqual(expDict, obsDict)
        
    def test_getClassifLineAsDict_wrongLine(self):
        line = "This\tis\tso\twrong\t.\n"
        
        expDict = OrderedDict()
        obsDict = ClassifUtils.getClassifLineAsDict(line)

        self.assertEqual(expDict, obsDict)
    
    def test_getClassifInfosAsDict(self):
        dummyClassifFile = "dummyClassif_%s.classif" % self._uniqId
        with open(dummyClassifFile, "w") as f:
            f.write("DHX-incomp_DmelChr4-B-R1-Map4\t2367\t.\tok\tII\tHelitron\tincomplete\
\tCI=20; coding=(TE_BLRtx: DNAREP1_DM:ClassII:Helitron:Helitron: 16.84%, DNAREP1_DSim:ClassII:Helitron:Helitron: 10.12%, DNAREP1_DYak:ClassII:Helitron:Helitron: 9.08%); \
struct=(TElength: >700bps); other=(SSRCoverage=0.18)\n")
            f.write("RLX-comp_DmelChr4-L-B6-Map1\t6439\t+\tok\tI\tLTR\tcomplete\
\tCI=92; coding=(TE_BLRtx: QUASIMODO2-I_DM:ClassI:LTR:Gypsy: 98.20%, QUASIMODO2-LTR_DM:ClassI:LTR:Gypsy: 100.00%; \
TE_BLRx: Gypsy-4_DBi-I_1p:ClassI:LTR:Gypsy: 27.74%, Gypsy-6_DSim-I_1p:ClassI:LTR:Gypsy: 59.00%, Gypsy-6_DSim-I_2p:ClassI:LTR:Gypsy: 79.67%; \
profiles: _GAG_17_6_GAG_NA: 96.63%(96.63%), _RNaseH_17_6_RH_NA: 100.00%(100.00%), _AP_17_6_AP_NA: 93.81%(93.81%), _INT_17_6_INT_NA: 100.00%(100.00%), _RT_17_6_RT_NA: 100.00%(100.00%)); \
struct=(TElength: >4000bps; TermRepeats: termLTR: 499; ORF: >1000bps GAG, >1000bps RT RH INT); other=(SSRCoverage=0.16)\n")
        
        preExpDict = [("DHX-incomp_DmelChr4-B-R1-Map4", OrderedDict([
                        ("name", "DHX-incomp_DmelChr4-B-R1-Map4"),
                        ("wCode", "DHX"),
                        ("length", 2367),
                        ("strand", "."),
                        ("chimeric", False),
                        ("class", "II"),
                        ("order", "Helitron"),
                        ("complete", False),
                        ("CI", 20),
                        ("coding", OrderedDict([
                            ("TE_BLRtx", OrderedDict([
                                ("DNAREP1_DM:ClassII:Helitron:Helitron", OrderedDict([
                                    ("cov", 16.84)
                                ])),
                                ("DNAREP1_DSim:ClassII:Helitron:Helitron", OrderedDict([
                                    ("cov", 10.12)
                                ])),
                                ("DNAREP1_DYak:ClassII:Helitron:Helitron", OrderedDict([
                                    ("cov", 9.08)
                                ]))
                            ]))
                        ])),
                        ("struct", OrderedDict([
                            ("TElength", ">700bps")
                        ])),
                        ("other", OrderedDict([
                            ("SSRCoverage", 0.18)
                        ]))
                    ])),
                    ("RLX-comp_DmelChr4-L-B6-Map1", OrderedDict([
                        ("name", "RLX-comp_DmelChr4-L-B6-Map1"),
                        ("wCode", "RLX"),
                        ("length", 6439),
                        ("strand", "+"),
                        ("chimeric", False),
                        ("class", "I"),
                        ("order", "LTR"),
                        ("complete", True),
                        ("CI", 92),
                        ("coding", OrderedDict([
                            ("TE_BLRtx", OrderedDict([
                                ("QUASIMODO2-I_DM:ClassI:LTR:Gypsy", OrderedDict([
                                    ("cov", 98.20)
                                ])),
                                ("QUASIMODO2-LTR_DM:ClassI:LTR:Gypsy", OrderedDict([
                                    ("cov", 100.00)
                                ]))
                            ])),
                            ("TE_BLRx", OrderedDict([
                                ("Gypsy-4_DBi-I_1p:ClassI:LTR:Gypsy", OrderedDict([
                                    ("cov", 27.74)
                                ])),
                                ("Gypsy-6_DSim-I_1p:ClassI:LTR:Gypsy", OrderedDict([
                                    ("cov", 59.00)
                                ])),
                                ("Gypsy-6_DSim-I_2p:ClassI:LTR:Gypsy", OrderedDict([
                                    ("cov", 79.67)
                                ]))
                            ])),
                            ("profiles", OrderedDict([
                                ("_GAG_17_6_GAG_NA", OrderedDict([
                                    ("cov", 96.63),
                                    ("covOnSubject", 96.63)
                                ])),
                                ("_RNaseH_17_6_RH_NA", OrderedDict([
                                    ("cov", 100.00),
                                    ("covOnSubject", 100.00)
                                ])),
                                ("_AP_17_6_AP_NA", OrderedDict([
                                    ("cov", 93.81),
                                    ("covOnSubject", 93.81)
                                ])),
                                ("_INT_17_6_INT_NA", OrderedDict([
                                    ("cov", 100.00),
                                    ("covOnSubject", 100.00)
                                ])),
                                ("_RT_17_6_RT_NA", OrderedDict([
                                    ("cov", 100.00),
                                    ("covOnSubject", 100.00)
                                ]))
                            ]))
                        ])),
                        ("struct", OrderedDict([
                            ("TElength", ">4000bps"),
                            ("TermRepeats", OrderedDict([
                                ("termLTR", 499)
                            ])),
                            ("ORF", [">1000bps GAG", ">1000bps RT RH INT"])
                        ])),
                        ("other", OrderedDict([
                            ("SSRCoverage", 0.16)
                        ]))
                    ]))]
        
        expDict = OrderedDict(preExpDict)
        obsDict = ClassifUtils.getClassifInfosAsDict(dummyClassifFile)

        self.assertEqual(expDict, obsDict)
        os.remove(dummyClassifFile)
        
    def test_getClassifInfosAsDict_wrongFile(self):
        dummyFile = "dummyFile_%s.notClassif" % self._uniqId
        with open(dummyFile, "w") as f:
            f.write("Totally wrong\n")
        
        with self.assertRaises(SystemExit) as catcher:
            ClassifUtils.getClassifInfosAsDict(dummyFile)
        
        expExceptionCode = 1
        obsExceptionCode = catcher.exception.code
        
        self.assertEqual(expExceptionCode, obsExceptionCode)
        os.remove(dummyFile)
    
    def test_getClassifInfosAsDict_emptyFile(self):
        dummyClassifFile = "dummyClassif_%s.classif" % self._uniqId
        with open(dummyClassifFile, "w"):
            pass
        
        expDict = OrderedDict()
        obsDict = ClassifUtils.getClassifInfosAsDict(dummyClassifFile)
        
        self.assertEqual(expDict, obsDict)
        os.remove(dummyClassifFile)
    
    def test_convertClassifToJson(self):
        inputClassifFile = "%s/commons/input_ClassifUtils.classif" % os.environ["REPET_DATA"]
        expJsonFile = "%s/commons/exp_ClassifUtils.json" % os.environ["REPET_DATA"]
        obsJsonFile = "output_convertClassifToJson_%s.json" % self._uniqId
        
        ClassifUtils.convertClassifToJson(inputClassifFile, obsJsonFile)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expJsonFile, obsJsonFile))
        os.remove(obsJsonFile)
        
if __name__ == "__main__":
    unittest.main()
