import unittest
import os
from commons.core.utils.FileUtils import FileUtils
import shutil
from commons.launcher.LaunchJellyfish import LaunchJellyfish
import time


class Test_F_LaunchJellyfish(unittest.TestCase):

    def setUp(self):
        self._inFastaFileName = "%s/Tools/DmelChr4.fa" % os.environ["REPET_DATA"]
        self.obsFileName = "DmelChr4.jellyfish.gff"
        self.testDir = os.path.join(os.getcwd(), "TestLaunchJellyfish%s" % time.strftime("%Y%m%d%H%M%S"))
        os.makedirs(self.testDir)
        os.chdir(self.testDir)
        
    def tearDown(self):
        os.chdir(self.testDir)
        os.chdir("..")
        shutil.rmtree(self.testDir)
       

    def test_run_output_as_gff_cleaned(self):
        iLaunchJellyfish = LaunchJellyfish(self._inFastaFileName,  clean = True, verbosity=5)
        iLaunchJellyfish.run()
        
        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20.gff" % os.environ["REPET_DATA"]
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))
        
#    def test_run_with_remote_dir(self):
#        os.makedirs("sourceDir")
#        changedInFastaFileName = os.path.basename(self._inFastaFileName)
#        changedInFastaFilepath = os.path.join("sourceDir",changedInFastaFileName)
#        shutil.copy2(self._inFastaFileName, changedInFastaFilepath)
#        
#        iLaunchJellyfish = LaunchJellyfish(changedInFastaFilepath,  clean = True, verbosity=2)
#        iLaunchJellyfish.run()
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20.gff" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff"
#        
#        
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        obsSourceDirFiles = os.listdir("sourceDir")
#        expSourceDirFiles = [changedInFastaFileName,]
#        self.assertEquals(expSourceDirFiles, obsSourceDirFiles)
#        
#        os.remove(obsFileName)
#        os.remove("DmelChr4.jellyfish.stat")
#        shutil.rmtree("sourceDir")
#
#
#    def test_run_output_as_gff_cleaned_changed_minOccs2(self):
#        iLaunchJellyfish = LaunchJellyfish(self._inFastaFileName, minOccs=2, clean = True, verbosity=2)
#        iLaunchJellyfish.run()
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20_minOcc2.gff" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff"
#        
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        os.remove(obsFileName)
#        os.remove("DmelChr4.jellyfish.stat")
#        
#    def test_run_output_as_gff_cleaned_relative_input(self):
#        srcFastaFileName = self._inFastaFileName
#        dstFastaFileName = os.path.basename(self._inFastaFileName)
#        shutil.copy2(srcFastaFileName, dstFastaFileName)
#        iLaunchJellyfish = LaunchJellyfish(dstFastaFileName,  clean = True, verbosity=2)
#        iLaunchJellyfish.run()
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20.gff" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff"
#        
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        
#        os.remove(obsFileName)
#        os.remove("DmelChr4.jellyfish.stat")
#        os.remove(dstFastaFileName)
#   
#    def test_run_as_script_output_as_gff_cleaned_relative_input_(self):
#        srcFastaFileName = self._inFastaFileName
#        dstFastaFileName = os.path.basename(self._inFastaFileName)
#        shutil.copy2(srcFastaFileName, dstFastaFileName)
#        
#        cmd = 'LaunchJellyfish.py -i %s -s 20 -c -v 3' % (self._inFastaFileName)
#        os.system(cmd)
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20.gff" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff"
#        
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        
#        os.remove(obsFileName)
#        os.remove("DmelChr4.jellyfish.stat")
#        os.remove(dstFastaFileName)
       
    def test_run_output_as_wig_cleaned(self):
        iLaunchJellyfish = LaunchJellyfish(self._inFastaFileName, outputFormats="wig", clean=True)
        iLaunchJellyfish.run()
        
        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k20.wig" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.jellyfish.wig"
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        

#    def test_run_output_as_gff_and_wig_k15_cleaned(self):
#        iLaunchJellyfish = LaunchJellyfish(self._inFastaFileName, merSize=15, outputFormats="wig,gff3", clean=True)
#        iLaunchJellyfish.run()
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k15.wig" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.wig"
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        os.remove(obsFileName)
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k15.gff3" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff3"
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        os.remove(obsFileName)
#        
#        os.remove("DmelChr4.jellyfish.stat")
#
#    def test_run_as_script_run_output_as_gff_and_wig_k15_cleaned(self):
#        logFileName = "test.log"
#        cmd = 'LaunchJellyfish.py -i %s -s 15 -f gff,wig,vero -c -v 3 > %s' % (self._inFastaFileName, logFileName)
#        os.system(cmd)
#               
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k15.wig" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.wig"
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        os.remove(obsFileName)
#        
#        expFileName = "%s/Tools/DmelChr4.fa.jellyfish_k15.gff3" % os.environ["REPET_DATA"]
#        obsFileName = "DmelChr4.jellyfish.gff"
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        os.remove(obsFileName)
#        
#        expLogFileName = "exp.log"
#        with open(expLogFileName, "w") as f:
#            f.write(" LaunchJellyfish - WARNING - Warning: ignoring invalid formats: <vero>\n")
#            f.write(" LaunchJellyfish - INFO - Starting to run jellyfish search of sequence DmelChr4.fa \n")
#            f.write(" LaunchJellyfish - INFO - Finished running jellyfish scan of sequence DmelChr4.fa \n")
#            f.write(" LaunchJellyfish - INFO - Starting to run jellyfish search to map conversion\n")
#            f.write(" LaunchJellyfish - INFO - Finished jellyfish search to map conversion\n")
#            f.write(" LaunchJellyfish - INFO - Generating wig file\n")
#            f.write(" LaunchJellyfish - INFO - Generating gff file\n")
#        obsLogFileName = "obs.log"
#        os.system("cut -d'-' -f4,5,6 %s > %s" % (logFileName, obsLogFileName))
#        self.assertTrue(FileUtils.are2FilesIdentical(expLogFileName, obsLogFileName))
#        
#        os.remove(logFileName)
#        os.remove(expLogFileName)
#        os.remove(obsLogFileName)
#        os.remove("DmelChr4.jellyfish.stat")
        
if __name__ == "__main__":
    unittest.main()