import os
import time
import shutil
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.launcher.LaunchPindel import LaunchPindel

class Test_F_LaunchPindel(unittest.TestCase):
    
    def setUp(self):
        self._testDir = os.path.join(os.getcwd(), "test_F_LaunchPindel_%s" % time.strftime("%Y%m%d%H%M%S"))
        os.mkdir(self._testDir)
        os.chdir(self._testDir)
        self._fastaFileName = "DummyGenome_modified.fa"
        os.symlink("%s/Tools/GetDeletionsFromCoverage/%s" % (os.environ["REPET_DATA"], self._fastaFileName), self._fastaFileName)
        self._bamFileName = "MutatedDummyGenome_VS_sim60x.bam"
        os.symlink("%s/Tools/GetDeletionsFromCoverage/%s" % (os.environ["REPET_DATA"], self._bamFileName), self._bamFileName)
        self._obsFileName = "DummyGenome_modified_D_LI_SI.vcf"
        self._expVcfFile =  "%s/commons/expPindelResults_D_LI_SI.vcf" % os.environ["REPET_DATA"]
       
        self._verbose = 3
        
    def tearDown(self):
        os.chdir(self._testDir)
        os.chdir("..")
        shutil.rmtree(self._testDir)
        
    def test_run(self):
        iLaunchPindel = LaunchPindel(self._fastaFileName, bamFileName = self._bamFileName, insertSize = 500, nbThreads = 4, doClean = True, verbosity = self._verbose)
        iLaunchPindel.run()
        
        self._outputVcfTrimmed = "%s_trimmed" % self._obsFileName
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT' > %s" % (self._obsFileName, self._outputVcfTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expVcfFile, self._outputVcfTrimmed))

    def test_run_as_script(self):
        self._obsFileName = "observed.vcf"
        cmd = 'LaunchPindel.py -i %s -b %s -s 500 -o %s -t 4 -v %s -c' % (os.path.basename(self._fastaFileName), os.path.basename(self._bamFileName), self._obsFileName, self._verbose)
        os.system(cmd)

        self._outputVcfTrimmed = "%s_trimmed" % self._obsFileName
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT' > %s" % (self._obsFileName, self._outputVcfTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expVcfFile, self._outputVcfTrimmed))
        
if __name__ == "__main__":
    unittest.main()