import os
import glob
import shutil
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.launcher.LaunchRepeatScout import LaunchRepeatScout

class Test_F_LaunchRepeatScout(unittest.TestCase):
    
    def setUp(self):
        self._inputFilePath = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
        self._inputFastaFileName = os.path.basename(self._inputFilePath)
        
        self._expRSFileName = "%s/commons/expDmelChr4_RS.fa" % os.environ["REPET_DATA"]
        self._obsRSFileName = "DmelChr4_RS.fa"

        self._verbosity = 4
        
    def tearDown(self):
        lFolders = glob.glob("LaunchRepeatScout_*")
        for f in lFolders:
            shutil.rmtree(f)
        
        if os.path.exists(self._inputFastaFileName):
            os.remove(self._inputFastaFileName)
        if os.path.exists(self._obsRSFileName):
            os.remove(self._obsRSFileName)

    def test_run_as_script_with_local_input(self):
        os.symlink(self._inputFilePath, self._inputFastaFileName)
        cmd = "LaunchRepeatScout.py -i %s -o %s -l 100 -v %s" % (self._inputFastaFileName, self._obsRSFileName, self._verbosity)
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expRSFileName, self._obsRSFileName))
        
    def test_run_with_abspath_input(self):
        self._obsRSFileName = "DmelChr4_RepeatScoutConsensus.fa"
        iLaunchRepeatScout = LaunchRepeatScout(self._inputFilePath, minLength = 100, verbosity = self._verbosity)
        iLaunchRepeatScout.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self._expRSFileName, self._obsRSFileName))

if __name__ == "__main__":
    unittest.main()