import os
import glob
import shutil
import unittest
from commons.core.seq.FastaUtils import FastaUtils
from commons.launcher.LaunchVelvet import LaunchVelvet

class Test_F_LaunchVelvet(unittest.TestCase):
    
    def setUp(self):
        self._inputRefFilePath = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
        self._inputRefFileName = os.path.basename(self._inputRefFilePath)
        
        self._inputBAMFilePath = "%s/commons/DmelChr4_5x.bam" % os.environ["REPET_DATA"]
        self._inputBAMFileName = os.path.basename(self._inputBAMFilePath)
        
        self._obsContigFileName = "DmelChr4_contigs.fa"
        
        os.symlink(self._inputRefFilePath, self._inputRefFileName)
        os.symlink(self._inputBAMFilePath, self._inputBAMFileName)
        
        self._verbosity = 3
        
    def tearDown(self):
        lFolders = glob.glob("LaunchVelvet_*")
        for f in lFolders:
            shutil.rmtree(f)
        
        if os.path.exists(self._inputRefFileName):
            os.remove(self._inputRefFileName)
        if os.path.exists(self._inputBAMFileName):
            os.remove(self._inputBAMFileName)
        if os.path.exists(self._obsContigFileName):
            os.remove(self._obsContigFileName)

    def test_run_as_script_with_local_input_4_threads(self):
        cmd = "LaunchVelvet.py -i %s -b %s -k 27 -t 4 -v %s" % (self._inputRefFileName, self._inputBAMFileName, self._verbosity)
        os.system(cmd)
        
        lContigsHeaders = FastaUtils.dbHeaders(self._obsContigFileName)

        self.assertTrue(len(lContigsHeaders) > 3300)
        self.assertTrue(len(lContigsHeaders) < 3400)
        self.assertTrue(len(lContigsHeaders) == len([h for h in lContigsHeaders if "NODE" in h]))

    def test_run_as_class_with_local_input_2_threads(self):
        iLV = LaunchVelvet(self._inputRefFileName, "", self._inputBAMFileName, 27, 2, False, self._verbosity)
        iLV.run()
        
        lContigsHeaders = FastaUtils.dbHeaders(self._obsContigFileName)

        self.assertTrue(len(lContigsHeaders) > 3300)
        self.assertTrue(len(lContigsHeaders) < 3400)
        self.assertTrue(len(lContigsHeaders) == len([h for h in lContigsHeaders if "NODE" in h]))
        
    def test_run_with_abspath_input_2_threads(self):
        self._obsContigFileName = "DmelChr4_VelvetContigsReference-GuidedAssembled.fa"
        cmd = "LaunchVelvet.py -i %s -b %s -k 27 -o %s -t 2 -v %s -c" % (self._inputRefFilePath, self._inputBAMFilePath, self._obsContigFileName, self._verbosity)
        os.system(cmd)
        
        lContigsHeaders = FastaUtils.dbHeaders(self._obsContigFileName)
        
        self.assertTrue(len(lContigsHeaders) > 3300)
        self.assertTrue(len(lContigsHeaders) < 3400)
        self.assertTrue(len(lContigsHeaders) == len([h for h in lContigsHeaders if "NODE" in h]))
        
if __name__ == "__main__":
    unittest.main()