#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


import sys
import getopt

from commons.core.seq.BioseqDB import BioseqDB
import pyRepet.util.Stat
from commons.core.utils.FileUtils import FileUtils


def help():
    print
    print "usage: GiveInfoFasta.py [ options ]"
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input file (format='fasta')"
    print "     -l: give stats for each input sequence"
    print "     -o: name of the output file (default=inFileName+.'stats')"
    print "     -v: verbose (default=0/1)"
    print


def main():
    """
    This program outputs some summary statistics on the input fasta file.
    """

    inFileName = ""
    giveStatsPerSeq = False
    outFileName = ""
    verbose = 0

    try:
        opts,args = getopt.getopt( sys.argv[1:], "hi:lo:v:" )
    except getopt.GetoptError, err:
        sys.stderr.write( "%s\n" % str(err) )
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-l":
            giveStatsPerSeq = True
        elif o == "-o":
            outFileName = a
        elif o == "-v":
            verbose = int(a)

    if inFileName == "":
        msg = "ERROR: missing input file name (-i)"
        sys.stderr.write( "%s\n" % msg )
        help()
        sys.exit(1)

    if outFileName == "":
        outFileName = "%s.stats" % ( inFileName )
    outFile = open( outFileName, "w" )

    if verbose > 0:
        print "load the fasta file..."; sys.stdout.flush()
    bsDB = BioseqDB( inFileName )
    if verbose > 0:
        print "done"; sys.stdout.flush()

    stat = pyRepet.util.Stat.Stat()
    for bs in bsDB.db:
        stat.add( bs.getLength() )
        if giveStatsPerSeq:
            string = "sequence '%s': %i bp, GC=%.2f%% N=%.2f%%" % ( bs.header,
                                                                    bs.getLength(),
                                                                    bs.getGCpercentage(),
                                                                    100 * bs.propNt("N") )
            outFile.write( "%s\n" % ( string ) )
            if verbose > 0:
                print string
    sys.stdout.flush()

    string =  "size of the bank (nb of sequences): %i" % ( stat.n )
    outFile.write( "%s\n" % ( string ) )
    if verbose > 0:
        print string

    string = "length of the bank (cumulative sequence lengths): %i bp" % ( stat.sum )
    outFile.write( "%s\n" % ( string ) )
    if verbose > 0:
        print string

    string = "mean sequence length: %i bp (var=%.2f, sd=%.2f, cv=%.2f)" % ( stat.mean(), stat.var(), stat.sd(), stat.cv() )
    outFile.write( "%s\n" % ( string ) )
    if verbose > 0:
        print string

    string = stat.stringQuantiles()
    outFile.write( "%s\n" % ( string ) )
    if verbose > 0:
        print string

    bsDB.upCase()
    dNt2Count = bsDB.countAllNt()
    string = "GC content: %.2f%%" % ( 100 * ( dNt2Count["G"] + dNt2Count["C"] ) / float(stat.sum) )
    outFile.write( "%s\n" % ( string ) )
    if verbose > 0:
        print string
    for nt in ["A","T","G","C","N"]:
        string = "occurrences of %s: %i (%.2f%%)" % ( nt, dNt2Count[nt], 100*dNt2Count[nt]/float(stat.sum) )
        outFile.write( "%s\n" % ( string ) )
        if verbose > 0:
            print string

    try:
        md5sum = FileUtils.getMd5SecureHash( inFileName )
        string = "MD5 secure hash of the bank: %s" % ( md5sum )
        outFile.write( "%s\n" % ( string ) )
        if verbose > 0:
            print string
    except:
        pass

    return 0


if __name__ == "__main__":
    main()
