#!/usr/bin/env python

import re
from commons.pyRepetUnit.profilesDB.Profiles import Profiles
from commons.core.LoggerFactory import LoggerFactory
from commons.core.utils.RepetOptionParser import RepetOptionParser

LOG_DEPTH = "commons.tools"

## Format a profiles DB for pipelines in REPET
#    
class ProfilesDB4Repet(object):
    
    def __init__(self, inputFile = "", outputFile = "", superFamily = "", verbosity = 2):
        self.profile = Profiles()
        self._inputFile = inputFile
        self._outputFile =  outputFile
        
# Beware: superFamily is not always shared by all the profiles in a given bank.
# TODO: try to infer superFamily of each profile, using regexp. In this case, superFamily is not an object attribute anymore.
        self.setSuperFamily(superFamily)
        self._verbosity = verbosity
        self._log = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self._verbosity)
    
    def setAttributesFromCmdLine(self):
        #TODO: Write a tutorial
        usage = "ProfilesDB4Repet.py [options]"
        description = "Format an HMM profiles bank for REPET.\n"
        epilog = "\nExample 1:\n"
        epilog += "\t$ ProfilesDB4Repet.py -i bank.hmm -o bank_REPET.hmm -v 3\n"
        parser = RepetOptionParser(description = description, epilog = epilog, usage = usage)
        parser.add_option("-i", "--in",        dest = "inFileName",  action = "store", type = "string", help = "name of the profiles DB to format for REPET [mandatory]", default = "")
        parser.add_option("-o", "--out",       dest = "outFileName", action = "store", type = "string", help = "name of the output profiles DB for REPET [mandatory]", default = "")
        parser.add_option("-v", "--verbosity", dest = "verbosity",   action = "store", type = "int",    help = "verbosity [optional] [default: 2]", default = 2)
        options = parser.parse_args()[0]
        self._setAttributesFromOptions(options)
        
    def _setAttributesFromOptions(self, options):
        self._inputFile = options.inFileName
        self._outputFile = options.outFileName
        self._verbosity = options.verbosity
    
    ## Set input file name
    #  
    # @param inputFileName string
    #
    def setInputFile(self, inputFileName):
        self._inputFile = inputFileName
        
    ## Set output file name
    #
    # @param outputFileName string
    #
    def setOutputFile(self, outputFileName):
        self._outputFile = outputFileName   
    
    def setSuperFamily(self, superFamily):
        if superFamily == "":
            superFamily = "NA"
        self._superFamily = superFamily    
        
    #TODO: add nb of each domain in log file, verbose...
    def _searchCurrentDomain(self, profile):
        currentDomain = ""
        #TODO: pattern GAGA and GAGE should be excluded !
        #TODO: add new tag like "ORF1_LTR" for ATHILA as key word in Pfam
        #TODO: add new tags from GypsyDB (MOV etc...)
        #RTRH, GAG->ok, AP->ok, INT->ok, RT->ok, RH->ok, YR->ok, ENV->ok, EN->ok, RPA->ok, APE->ok, C-INT->ok, ATP->ok, CYP->ok, POLB->ok, HEL->ok
        
        name = profile[1]
        accession = profile[2]
        description = profile[3]
        
        if (re.search("RTRH_", name)):#SPECIAL case
            currentDomain = "RTRH"
            
        elif (re.search("[gG][aA][Gg]", name) or re.search("[gG][aA][Gg]", description)):
            currentDomain = "GAG"
        elif (re.search("Zinc knuckle", name) or re.search("Zinc knuckle", description)):
            currentDomain = "GAG"
        elif (re.search("PF02813", accession) or re.search("PF01021", accession)):
            currentDomain = "GAG"
        elif (re.search("GAG_", name)):
            currentDomain = "GAG"
        elif (re.search("GAGCOAT_", name)):
            currentDomain = "GAG"
            
        elif ((re.search("[aA]spartic", name) or re.search("[aA]aspartic", description)) and (re.search("[pP]roteinase", name) or re.search("[pP]roteinase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartic", name) or re.search("[aA]spartic", description)) and (re.search("[pP]rotease", name) or re.search("[pP]rotease", description))):
            currentDomain = "AP"
        elif ((re.search("[rR]etrotransposon", name) or re.search("[rR]etrotransposon", description)) and (re.search("[pP]eptidase", name) or re.search("[pP]eptidase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartic", name) or re.search("[aA]spartic", description)) and (re.search("[pP]eptidase", name) or re.search("[pP]eptidase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartic", name) or re.search("[aA]spartic", description)) and (re.search("[eE]ndopeptidase", name) or re.search("[eE]ndopeptidase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartyl", name) or re.search("[aA]spartyl", description)) and (re.search("[pP]roteinase", name) or re.search("[pP]roteinase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartyl", name) or re.search("[aA]spartyl", description)) and (re.search("[pP]rotease", name) or re.search("[pP]rotease", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartyl", name) or re.search("[aA]spartyl", description)) and (re.search("[pP]eptidase", name) or re.search("[pP]eptidase", description))):
            currentDomain = "AP"
        elif ((re.search("[aA]spartyl", name) or re.search("[aA]spartyl", description)) and (re.search("[eE]ndopeptidase", name) or re.search("[eE]ndopeptidase", description))):
            currentDomain = "AP"
        elif ((re.search("AP_", name)) and not (re.search("[eE]ndonuclease", description))):
            currentDomain = "AP"
            
        elif ((re.search("[iI]ntegrase", name) or re.search("[iI]ntegrase", description)) and not (re.search(" C ", name)) and not (re.search(" C ", description))):
            currentDomain = "INT"
        elif (re.search(".*[Cc]hromo.*", name) or re.search(".*[Cc]hromo.*", description)):
            currentDomain = "INT"
        elif (re.search("INT_", name)):
            currentDomain = "INT"
            
        elif ((re.search("[rR]everse", name) or re.search("[Rr]everse", description)) and (re.search("[tT]ranscriptase", name) or re.search("[tT]ranscriptase", description))):
            currentDomain = "RT"
        elif (re.search("RT_", name)):
            currentDomain = "RT"
            
        elif ((re.search("R[nN]ase", name) or re.search("R[nN]ase", description)) and (re.search("H", name) or re.search("H", description))):
            currentDomain = "RH"
        elif ((re.search("R[nN]ase", name) or re.search("R[nN]ase", description)) and (re.search("H ", name) or re.search("H ", description))):
            currentDomain = "RH"
        elif (re.search("RNaseH_", name)):
            currentDomain = "RH"
        elif (re.search("RH_", name)):
            currentDomain = "RH"
            
        elif ((re.search("[eE]nvelope", name) or re.search("[eE]nvelope", description)) and (re.search("[pP]rotein", name) or re.search("[pP]rotein", description))):
            currentDomain = "ENV"
        elif ((re.search("ENV", name) or re.search("ENV", description)) and (re.search("[pP]olyprotein", name) or re.search("[pP]olyprotein", description))):
            currentDomain = "ENV"
        elif ((re.search("[eE]nvelope", name) or re.search("[eE]nvelope", description)) and (re.search("[pP]olyprotein", name) or re.search("[pP]olyprotein", description))):
            currentDomain = "ENV"
        elif ((re.search("[eE]nvelope", name) or re.search("[eE]nvelope", description)) and (re.search("[gG]lycoprotein", name) or re.search("[gG]lycoprotein", description))):
            currentDomain = "ENV"
        elif (re.search("PF07253", accession) or re.search("PF03811", accession)):
            currentDomain = "ENV"
        elif (re.search("ENV_", name)):
            currentDomain = "ENV"
            
        elif ((re.search("[tT]yrosine", name) or re.search("[tT]yrosine", description)) and (re.search("[rR]ecombinase", name) or re.search("[rR]ecombinase", description))):
            currentDomain = "YR"
        elif (re.search("YR_", name)):
            currentDomain = "YR"
            
        elif ((re.search("[eE]ndonuclease", name) or re.search("[eE]ndonuclease", description)) and not (re.search("AP ", name) or re.search("AP ", description)) and not (re.search("[aA]purinic", name) or re.search("[aA]purinic", description))):
            currentDomain = "EN"
        elif (re.search("EN_", name)):
            currentDomain = "EN"
            
        elif ((re.search("[eE]ndonuclease", name) or re.search("[eE]ndonuclease", description)) and (re.search("AP ", name) or re.search("AP ", description))):
            currentDomain = "APE"
        elif ((re.search("[eE]ndonuclease", name) or re.search("[eE]ndonuclease", description)) and (re.search("[aA]purinic", name) or re.search("[aA]purinic", description))):
            currentDomain = "APE"
        elif (re.search("APE_", name)):
            currentDomain = "APE"
            
        elif ((re.search("[tT]ransposase", name) or re.search("[tT]ransposase", description)) and not (re.search("DDE ", name) or re.search("DDE ", description))):
            currentDomain = "Tase"
        elif (re.search("DUF659", name)):
            currentDomain = "Tase"
        elif (re.search("PF01695", accession)) or (re.search("PF02316", accession)) or (re.search("PF09039", accession)) or (re.search("PF04827", accession)) or (re.search("PF05699", accession)) or (re.search("PF10536", accession)):
            currentDomain = "Tase"
            
        elif ((re.search("[tT]ransposase", name) or re.search("[tT]ransposase", description)) and (re.search("DDE ", name) or re.search("DDE ", description))):
            currentDomain = "Tase*"
            
        elif ((re.search("[rR]eplication", name) or re.search("[rR]eplication", description)) and (re.search("[pP]rotein", name) or re.search("[pP]rotein", description)) and (re.search("A ", name) or re.search("A ", description))):
            currentDomain = "RPA"
        elif (re.search("[rR]epA ", name) or re.search("[rR]epA ", description)):
            currentDomain = "RPA"
        elif (re.search("RPA_", name)):
            currentDomain = "RPA"
        elif (re.search("RPA", name) or re.search("RPA", description)):
            currentDomain = "RPA"
            
        elif (re.search("[cC]-integrase", name) or re.search("[cC]-integrase", description)):
            currentDomain = "C-INT"
        elif (re.search("C-INT_", name)):
            currentDomain = "C-INT"
            
        elif ((re.search("[pP]ackaging", name) or re.search("[pP]ackaging", description)) and (re.search("ATPase", name) or re.search("ATPase", description))):
            currentDomain = "ATP"
        elif (re.search("ATP_", name)):
            currentDomain = "ATP"
            
        elif ((re.search("[cC]ysteine", name) or re.search("[cC]ysteine", description)) and (re.search("[pP]rotease", name) or re.search("[pP]rotease", description))):
            currentDomain = "CYP"
        elif ((re.search("[cC]ysteine", name) or re.search("[cC]ysteine", description)) and (re.search("[pP]eptidase", name) or re.search("[pP]eptidase", description))):
            currentDomain = "CYP"
        elif (re.search("[pP]eptidase_C", name) or re.search("[pP]eptidase_C", description)):
            currentDomain = "CYP"
        elif (re.search("PF00559", accession)):
            currentDomain = "CYP"
        elif (re.search("CYP_", name)):
            currentDomain = "CYP"
            
        elif (re.search("[pP]ol\S*_*B", name) or re.search("[pP]ol\S*_*B", description)):
            currentDomain = "POLB"
        elif ((re.search("[pP]olymerase", name) or re.search("[pP]olymerase", description)) and (re.search("B ", name) or re.search("B ", description))):
            currentDomain = "POLB"
        elif (re.search("POLB_", name)):
            currentDomain = "POLB"
            
        elif (re.search("[hH]elicase", name) or re.search("[hH]elicase", description)):
            currentDomain = "HEL"
        elif (re.search("HEL_", name)):
            currentDomain = "HEL"
        
        else :
            currentDomain = "OTHER"
        return currentDomain
    
    ## Replace the old profile name by accession number, name, domain and gather cut off
    #  
    # @param fout file handle
    # @param profile Profiles instance
    # @param currentDomain string
    #
    def _writeModifiedProfile(self, fout, profile, currentDomain):
        for i in xrange(0, len(profile), 1):
            if i != 1:
                fout.write(profile[i])
            else:
                fout.write("NAME  " + self.profile.accNumber + "_"\
                           + self.profile.name + "_"\
                            + self._superFamily+ "_"\
                             + currentDomain + "_"\
                              + str(self.profile.GA_cut_off) + "\n")
        
    ## Read a profiles DB file, parse it and, write a new profiles DB with TE domain information and GA score cut_off placed side by side of the name
    #    
    def run(self):
        LoggerFactory.setLevel(self._log, self._verbosity)
        fileIn = open(self._inputFile)
        fout = open(self._outputFile, "w")
        profile = self.profile.readAndRetrieve(fileIn)
        while profile is not None:
            currentDomain = self._searchCurrentDomain(profile)
            if currentDomain == "OTHER":
                self._log.warning(self.profile.accNumber + " " + self.profile.name + " has no associated domain")
            self._writeModifiedProfile(fout, profile, currentDomain)
            profile = self.profile.read(fileIn)
            
            
if __name__ == "__main__":
    iProfilesDB4Repet = ProfilesDB4Repet()
    iProfilesDB4Repet.setAttributesFromCmdLine()
    iProfilesDB4Repet.run()