import unittest
import os
from commons.core.utils.FileUtils import FileUtils
from commons.tools.formatting.HmmProfileFormatter import HmmProfileFormatter

class Test_F_HmmProfileFormatter(unittest.TestCase):

    def setUp(self):
        self._inputMultAlignFile = "%s/Tools/HmmProfileFormatter/INT_GalEa.fas" % os.environ["REPET_DATA"]
        self._expectedDBFile = "%s/Tools/HmmProfileFormatter/exp_INT_GalEa.fas.hmm" % os.environ["REPET_DATA"]
        self._expectedSuperFamilyDBFile = "exp_INT_GalEa.fas.hmm.supfam"
        self._outputDBFile = "output.hmm"
        
    def tearDown(self):
        try:
            os.remove(self._outputDBFile)
        except:pass
        try:
            os.remove(self._outputDBFile+".build")
        except:pass
        try:
            os.remove(self._expectedSuperFamilyDBFile)
        except:pass

    def test_HmmProfileFormatter(self):
        self._HmmProfileFormatter = HmmProfileFormatter(self._inputMultAlignFile, outFileName = self._outputDBFile, superFamily = "", verbosity = 5)
        self._HmmProfileFormatter.run()
        
        self._replaceData(self._expectedDBFile)
        self._replaceData(self._outputDBFile)
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expectedDBFile, self._outputDBFile))
    
    def test_HmmProfileFormatter_asScript(self):
        cmd = "HmmProfileFormatter.py -i %s -o %s " % (self._inputMultAlignFile, self._outputDBFile)
        os.system(cmd)
        
        self._replaceData(self._expectedDBFile)
        self._replaceData(self._outputDBFile)
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expectedDBFile, self._outputDBFile))
        
    def test_HmmProfileFormatter_superFamily(self):
        self._createSuperFamilyExpected(self._expectedDBFile, self._expectedSuperFamilyDBFile)
        
        self._HmmProfileFormatter= HmmProfileFormatter(self._inputMultAlignFile, outFileName = self._outputDBFile, superFamily = "TEST", verbosity = 5)
        self._HmmProfileFormatter.run()
        
        self._replaceData(self._expectedDBFile)
        self._replaceData(self._outputDBFile)
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expectedSuperFamilyDBFile, self._outputDBFile))
    
    def test_HmmProfileFormatter_asScript_superFamily(self):
        self._createSuperFamilyExpected(self._expectedDBFile, self._expectedSuperFamilyDBFile)
        
        cmd = "HmmProfileFormatter.py -i %s -o %s -s %s" % (self._inputMultAlignFile, self._outputDBFile, "TEST")
        os.system(cmd)
        
        self._replaceData(self._expectedDBFile)
        self._replaceData(self._outputDBFile)
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expectedSuperFamilyDBFile, self._outputDBFile))

    def _replaceData(self, fileName):
        #replace hmmbuild generated timestamp in header with nothing
        cmd = "sed -i 's/DATE.*/ /' %s" % fileName
        os.system(cmd)
        
        #replace hmmer version in header with nothing
        cmd = "sed -i 's/HMMER3.*/ /' %s" % fileName
        os.system(cmd)
    
    def _createSuperFamilyExpected(self, fileName, outputFileName):
        cmd = "sed 's/NAME  _INT_GalEa_NA_INT_NA/NAME  _INT_GalEa_TEST_INT_NA/' %s >> %s" % (fileName,outputFileName)
        os.system(cmd)

#    def test_formateProfilesDB4Repet_GypsyDB(self):
#        self._createHmmFile_GypsyDB()
#        self._createExpectedHmmFile_GypsyDB()
#        self._profilesDB4Repet.setInputFile(self._inputDBFile)
#        self._profilesDB4Repet.setOutputFile(self._outputDBFile)
#        self._profilesDB4Repet.run()
#        self.assertTrue(FileUtils.are2FilesIdentical(self._expectedDBFile, self._outputDBFile))

if __name__ == "__main__":
    unittest.main()