import os
import unittest
import time
from commons.tools.AnnotationStats import AnnotationStats
from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TablePathAdaptator import TablePathAdaptator


class Test_AnnotationStats( unittest.TestCase ):
    
    def setUp(self):
        self._uniqId = time.strftime("%Y%m%d%H%M%S")
        self.iAS = None
        self._configFileName = "DummyConfig_%s" % self._uniqId
        configFile = open( self._configFileName, "w" )
        configFile.write("[repet_env]\n")
        configFile.write( "repet_host: %s\n" % ( os.environ["REPET_HOST"] ) )
        configFile.write( "repet_user: %s\n" % ( os.environ["REPET_USER"] ) )
        configFile.write( "repet_pw: %s\n" % ( os.environ["REPET_PW"] ) )
        configFile.write( "repet_db: %s\n" % ( os.environ["REPET_DB"] ) )
        configFile.write( "repet_port: %s\n" % ( os.environ["REPET_PORT"] ) )
        configFile.close()
        self.iDb = DbFactory.createInstance(self._configFileName)
        
    def tearDown(self):
        self._uniqId = None
        self.iDb.close()
        os.remove(self._configFileName)
        
    def test_getNbFullLengths(self):
        self.iAS = AnnotationStats()
        consensusLength = 100
        lLengths = [12, 100, 95, 105, 106, 94]
        obsNbFullLength = self.iAS.getNbFullLengths(consensusLength,lLengths )
        expNbFullLength = 3
        self.assertEqual( expNbFullLength, obsNbFullLength )
        
    def test_getClusterListFromFile(self):
        clusterFileName = "DummyClusterFile.txt"
        self._writeClusterFile(clusterFileName)
        self.iAS = AnnotationStats(clusterFileName=clusterFileName)
        obslClusters = self.iAS.getClusterListFromFile()
        explClusters = [["Cons1", "Cons2", "Cons3", "Cons4"], ["Cons5", "Cons6", "Cons7"], ["Cons8"], ["Cons9"]]
        self.assertEqual( explClusters, obslClusters )

        os.remove(clusterFileName)

    def test_getCumulCoverage(self):
        self.iAS = AnnotationStats()
        pathFileName = "Dummy%s.path" % self._uniqId
        self._writePath_File(pathFileName)
        pathTableName = "Dummy%s_path" % self._uniqId
        self.iDb.createTable(pathTableName, "path", pathFileName, True)

        self.iAS.setPathTableName(pathTableName)
        iTablePathAdaptator = TablePathAdaptator( self.iDb, pathTableName )
        self.iAS.setTablePathAdaptator(iTablePathAdaptator)
        obsGetCumulCoverage = self.iAS.getCumulCoverage("refTE_227")
        expGetCumulCoverage = 22+22+51
        self.assertEqual( expGetCumulCoverage, obsGetCumulCoverage )
        
        self.iDb.dropTable(pathTableName)
        os.remove(pathFileName)
        
    def _writePath_File(self, fileName):
        f = open(fileName, "w")
        f.write("1\tdmel_chr4\t1\t22\trefTE_227\t128\t145\t0\t148\t69.7\n")#22
        f.write("2\tdmel_chr4\t2\t40\trefTE_324\t648\t686\t0\t767\t85.8\n")#39
        f.write("3\tdmel_chr4\t91\t100\trefTE_617\t41\t48\t0\t580\t85.4\n")#10
        f.write("4\tdmel_chr4\t110\t119\trefTE_617\t41\t48\t0\t743\t84.9\n")#9
        f.write("5\tdmel_chr4\t150\t189\trefTE_200\t50\t89\t0\t221\t95.3\n")#40
        f.write("6\tdmel_chr4\t235\t265\trefTE_878\t13276\t13249\t0\t238\t90.7\n")#31
        f.write("7\tdmel_chr4\t370\t388\trefTE_161\t231\t249\t0\t244\t91.3\n")#19
        f.write("8\tdmel_chrU\t1\t22\trefTE_227\t220\t128\t145\t251\t92.3\n")#22
        f.write("9\tdmel_chrU\t2\t40\trefTE_324\t648\t686\t0\t244\t91.3\n")#39
        f.write("10\tdmel_chrU\t91\t100\trefTE_43\t41\t48\t0\t240\t90.8\n")#10
        f.write("11\tdmel_chrU\t100\t144\trefTE_45\t218\t260\t0\t262\t93.8\n")#45
        f.write("12\tdmel_chrU\t200\t250\trefTE_227\t4182\t4230\t0\t266\t94.4\n")#51
        f.close()

    def _writeClusterFile(self, fileName):
        f = open(fileName, "w")
        f.write("Cons1\tCons2\tCons3\tCons4\n")
        f.write("Cons5\tCons6\tCons7\n")
        f.write("Cons8\n")
        f.write("Cons9\n")
        f.close()

        
if __name__ == "__main__":
        unittest.main()