import unittest
from commons.tools.CreateGFF3sForClassifFeatures import CreateGFF3sForClassifFeatures

class Test_CreateGFF3sForClassifFeatures(unittest.TestCase):
    
    def test_transformHeaders(self):
        iCGCF = CreateGFF3sForClassifFeatures()
        myHeaders = ["RIX-comp_DmelCaf1_test-B-G11-Map3",
                     "RIX-incomp_DmelCaf1_test-B-R119-Map20_reversed",
                     "PotentialHostGene_DmelCaf1_test-B-P13.55-Map4",
                     "RXX-LARD_DmelCaf1_test-L-B103-Map1"]
        
        exps = ["DmelCaf1_test_Blaster_Grouper_11_Map_3",
                "DmelCaf1_test_Blaster_Recon_119_Map_20",
                "DmelCaf1_test_Blaster_Piler_13.55_Map_4",
                "DmelCaf1_test_LTRharvest_Blastclust_103_Map_1"]
        
        obss = [iCGCF._transformHeader(header) for header in myHeaders]
        
        for exp, obs in zip(exps, obss):
            self.assertEquals(exp, obs)
    
    def test_transformHeaders_errorHandling(self):
        iCGCF = CreateGFF3sForClassifFeatures()
        
        wrongHeader = "RIX-comp_DmelCaf1-X-G1-Map1"
        self.assertRaises(Exception, iCGCF._transformHeader(wrongHeader))
        
        wrongHeader = "RIX-comp_DmelCaf1-B-X11-Map1"
        self.assertRaises(Exception, iCGCF._transformHeader(wrongHeader))
        
        wrongHeader = "VeryWrong-Header"
        with self.assertRaises(SystemExit) as se:
            iCGCF._transformHeader(wrongHeader)
        raisedException = se.exception
        self.assertEqual(raisedException.code, 1)
        
if __name__ == "__main__":
    unittest.main()
    