import os
import random
import unittest
import subprocess
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils
from commons.tools.AlignTEOnGenomeAccordingToAnnotation import AlignTEOnGenomeAccordingToAnnotation

class Test_F_AlignTEOnGenomeAccordingToAnnotation(unittest.TestCase):

    def setUp(self):
        uniqID = random.randint(1, 10000)
        pathFileName = "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ["REPET_DATA"]
        queryFileName = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
        subjectFileName = "%s/Tools/DmelChr4_refTEs.fa" % os.environ["REPET_DATA"]
        self.pathTableName = "testDmelChr4_chr_allTEs_nr_noSSR_join_path_%d" % uniqID
        self.queryTableName = "testDmelChr4_chr_seq_%d" % uniqID
        self.subjectTableName = "testDmelChr4_refTEs_seq_%d" % uniqID
        self.iDb = DbFactory.createInstance()
        self.iDb.createTable(self.queryTableName, "seq", queryFileName, True)
        self.iDb.createTable(self.subjectTableName, "seq", subjectFileName, True)
        self.iDb.createTable(self.pathTableName, "path", pathFileName, True)
        
    def tearDown(self):
        self.iDb.dropTable(self.queryTableName)
        self.iDb.dropTable(self.subjectTableName)
        self.iDb.dropTable(self.pathTableName)
        self.iDb.dropTable("%s_align" % self.pathTableName)
        self.iDb.close()
        os.remove(self.obsFileName)

    def test_run_merge_same_paths(self):
        expFileName = "%s/Tools/exptestDmelChr4_chr_allTEs_nr_noSSR_join_path_merge.alignedSeq" % os.environ["REPET_DATA"]
        
        iATOGATA = AlignTEOnGenomeAccordingToAnnotation(self.pathTableName, self.queryTableName, self.subjectTableName, True)
        iATOGATA.run()
        self.obsFileName = "obs%s_merge.alignedSeq" % self.pathTableName
        self.iDb.exportDataToFile("%s_align" % self.pathTableName, self.obsFileName)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))
        
    def test_run_as_script_merge_same_paths(self):
        expFileName = "%s/Tools/exptestDmelChr4_chr_allTEs_nr_noSSR_join_path_merge.alignedSeq" % os.environ["REPET_DATA"]
           
        cmd = "AlignTEOnGenomeAccordingToAnnotation.py -p %s -q %s -s %s -m " % (self.pathTableName, self.queryTableName, self.subjectTableName)
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()
        self.obsFileName = "obs%s_merge.alignedSeq" % self.pathTableName
        self.iDb.exportDataToFile("%s_align" % self.pathTableName, self.obsFileName)
           
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))
           
    def test_run_as_script_without_merge(self):
        expFileName = "%s/Tools/exptestDmelChr4_chr_allTEs_nr_noSSR_join_path.alignedSeq" % os.environ["REPET_DATA"]
           
        cmd = "AlignTEOnGenomeAccordingToAnnotation.py -p %s -q %s -s %s" % (self.pathTableName, self.queryTableName, self.subjectTableName)
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()
        self.obsFileName = "obs%s.alignedSeq" % self.pathTableName
        self.iDb.exportDataToFile("%s_align" % self.pathTableName, self.obsFileName)
           
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))
         
#    def test_run_as_script_merge_same_paths_old(self):
#        pathFileName = "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ["REPET_DATA"]
#        queryFileName = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
#        subjectFileName = "%s/Tools/DmelChr4_refTEs.fa" % os.environ["REPET_DATA"]
#        pathTableName = "testDmelChr4_chr_allTEs_nr_noSSR_join_path"
#        queryTableName = "testDmelChr4_chr_seq"
#        subjectTableName = "testDmelChr4_refTEs_seq"
#        iDb = DbFactory.createInstance()
#        iDb.createTable(queryTableName, "seq", queryFileName, True)
#        iDb.createTable(subjectTableName, "seq", subjectFileName, True)
#        iDb.createTable(pathTableName, "path", pathFileName, True)
#        expFileName = "%s/Tools/exp%s_merge_old.alignedSeq" % (os.environ["REPET_DATA"], pathTableName)
#        
#        cmd = "srptAlignPath.py -m %s -q %s -s %s" % (pathTableName, queryTableName, subjectTableName)
#        process = subprocess.Popen(cmd, shell = True)
#        process.communicate()
#        obsFileName = "obs%s.alignedSeq" % pathTableName
#        iDb.exportDataToFile("%s_align" % pathTableName, obsFileName)
#        
#        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
#        
#        os.remove(obsFileName)
#        iDb.dropTable(queryTableName)
#        iDb.dropTable(subjectTableName)
#        iDb.dropTable(pathTableName)
#        iDb.dropTable("%s_align" % pathTableName)
#        iDb.close()
#        
if __name__ == "__main__":
    unittest.main()