import os
import time
import shutil
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.tools.BenchVelvetKValue import BenchVelvetKValue

class Test_F_BenchVelvetKValue(unittest.TestCase):
    
    SARUMAN_NAME = "compute-2-46.local"
    
    def setUp(self):
        self._inputBAM = "%s/commons/DmelChr4_5x.bam" % os.environ["REPET_DATA"]
        self._inputRef = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
        self._expKfile = "expKFile.txt"
        self._verbosity = 4
        self._nbThreads = 2
        
        self._testDir = os.path.join(os.getcwd(), "test_F_BenchVelvetKValue_%s" % time.strftime("%Y%m%d%H%M%S"))
        os.mkdir(self._testDir)
        os.chdir(self._testDir)
        os.symlink(self._inputBAM, "DmelChr4_5x.bam")
        self._inputBAM = "%s/DmelChr4_5x.bam" % os.getcwd()
        
        if os.getenv("HOSTNAME") == self.SARUMAN_NAME:
            self._resources = "pe multithread %i test=TRUE" % self._nbThreads
        else:
            self._resources = "pe multithread %i mem_free=100M" % self._nbThreads
        
    def tearDown(self):
        os.chdir(self._testDir)
        os.chdir("..")
        shutil.rmtree(self._testDir)

    def test_run(self):
        iBenchVelvetKValue = BenchVelvetKValue()
        iBenchVelvetKValue.setInputBAM(self._inputBAM)
        iBenchVelvetKValue.setInputRef(self._inputRef)
        iBenchVelvetKValue.setDoClean(False)
        iBenchVelvetKValue.setNbThreads(self._nbThreads)
        iBenchVelvetKValue.setResources(self._resources)
        iBenchVelvetKValue.setDoSecPass(True)
        iBenchVelvetKValue.setVerbosity(self._verbosity)
        obsKValue = iBenchVelvetKValue.run()
        
        with open(self._expKfile, "w") as fH:
            fH.write("K-value maximizing assembly quality: 21")
        
        self.assertEqual(obsKValue, 21)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expKfile, "K.txt"))

    def test_run_as_script(self):
        cmd = "BenchVelvetKValue.py"
        cmd += " -i %s" % self._inputBAM
        cmd += " -r %s" % self._inputRef
        cmd += " -o Results.txt"
        cmd += " -t %i" % self._nbThreads
        cmd += " -l '%s'" % self._resources
        cmd += " -s"
        cmd += " -v %i" % self._verbosity
        os.system(cmd)
        
        with open(self._expKfile, "w") as fH:
            fH.write("K-value maximizing assembly quality: 21")
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expKfile, "Results.txt"))

if __name__ == "__main__":
    unittest.main()