import os
import unittest
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils
from commons.tools.ComputeCoverageFromAlignAnnotation import ComputeCoverageFromAlignAnnotation
from commons.core.sql.TableSeqAdaptator import TableSeqAdaptator

class Test_F_ComputeCoverageFromAlignAnnotation(unittest.TestCase):

    def setUp(self):
        self.inAlignFileName = "annotationFile.align"
        self._writeAlign_File()
        self.configFileName = "TEdenovo.cfg"
        self._writeConfigFile()
        self.projectName = "project"
        self.chkMapTableName = "%s_chk_map" % self.projectName
        self.seqTableName = "%s_chr_seq" % self.projectName
        self._writeChk_mapFile()
        self._writeFastaFile()
        iDb = DbFactory.createInstance(self.configFileName)
        iDb.createTable(self.chkMapTableName, "map", "%s.map" % self.chkMapTableName, True)
        iDb.createTable(self.seqTableName, "seq", "sequences.fa", True)
        iDb.close()

        self._writeExpCoverageFile()
           
    def tearDown(self):
        iDb = DbFactory.createInstance(self.configFileName)
        iDb.dropTable(self.chkMapTableName)
        iDb.dropTable(self.seqTableName)
        iDb.close()
        os.remove(self.configFileName)
        os.remove("%s.map" % self.chkMapTableName)
        os.remove("sequences.fa")
        FileUtils.removeFilesByPattern("annotationFile.*")
        FileUtils.removeFilesByPattern("annotationFile*.*")
        FileUtils.removeFilesByPattern("*.txt")
        
    def test_run_as_script(self):
        cmd = 'ComputeCoverageFromAlignAnnotation.py -i %s -C %s -v 3' % (self.inAlignFileName, self.configFileName)
        os.system(cmd)
           
        self.assertTrue(FileUtils.are2FilesIdentical("expCoverage.txt", "coverage.txt"))
           
    def test_run(self):
        iCCFAA = ComputeCoverageFromAlignAnnotation(self.inAlignFileName, self.configFileName)
        iCCFAA.run()
        self.assertTrue(FileUtils.are2FilesIdentical("expCoverage.txt", "coverage.txt"))

    def test_run_onRealData(self):
        self.inAlignFileName = "%s/Tools/annotationFile.align" % os.environ["REPET_DATA"]
        self._writeChk_mapFile2()
        self._writeFastaFile2()
        self._writeExpCoverageFile2()
        iDb = DbFactory.createInstance(self.configFileName)
        iDb.createTable(self.chkMapTableName, "map", "%s.map" % self.chkMapTableName, True)
        iDb.createTable(self.seqTableName, "seq", "sequences.fa", True)
        tSA = TableSeqAdaptator( iDb, self.seqTableName )
        tSA.changeLengthFromAccession('dmel_chr4', 1281640)#dmel_chr4 real length 
        tSA.changeLengthFromAccession('chrU', 11561901)#chrU real length 
        iDb.close()
        
        iCCFAA = ComputeCoverageFromAlignAnnotation(self.inAlignFileName, self.configFileName)
        iCCFAA.run()
        self.assertTrue(FileUtils.are2FilesIdentical("expCoverage2.txt", "coverage.txt"))
         
    def _writeChk_mapFile2(self):
        f = open("%s.map" % self.chkMapTableName, "w")#l tot 1387807pb
        f.write("chunk0502\tdmel_chr4\t1\t200000\n") #batch_30.fa 5328 -->
        f.write("chunk0503\tdmel_chr4\t190001\t390000\n")#batch_29.fa 10288 -->
        f.write("chunk0504\tdmel_chr4\t380001\t580000\n")#batch_29.fa
        f.write("chunk0505\tdmel_chr4\t570001\t770000\n")#batch_29.fa
        f.write("chunk0506\tdmel_chr4\t760001\t960000\n")#batch_29.fa
        f.write("chunk0507\tdmel_chr4\t950001\t1150000\n")#batch_28.fa 6885 -->
        f.write("chunk0508\tdmel_chr4\t1140001\t1220912\n")#batch_32.fa 3226 -->
        f.write("chunk0509\tdmel_chr4\t1221013\t1281640\n")#batch_29.fa
        f.write("chunk0510\tdmel_chrU\t1\t4034\n")#batch_2.fa 6703 -->
        f.write("chunk0511\tdmel_chrU\t8142\t15458\n")#batch_5.fa 5580 -->
        f.write("chunk0512\tdmel_chrU\t19478\t26793\n")#batch_5.fa
        f.write("chunk0513\tdmel_chrU\t28874\t39343\n")#batch_7.fa 4872 -->
        f.write("chunk0514\tdmel_chrU\t39928\t46868\n")#batch_4.fa 4632 -->
        f.write("chunk0515\tdmel_chrU\t47869\t94576\n")#batch_25.fa 4599 -->
        f.write("chunk0516\tdmel_chrU\t95503\t106167\n")#batch_7.fa
        f.close()

    def _writeChk_mapFile(self):
        f = open("%s.map" % self.chkMapTableName, "w")# l dmel_chr4=370, l dmel_chrU=261, l tot= 631pb
        f.write("chunk1\tdmel_chr4\t1\t100\n") 
        f.write("chunk2\tdmel_chr4\t91\t190\n")
        f.write("chunk3\tdmel_chr4\t181\t280\n")
        f.write("chunk4\tdmel_chr4\t271\t370\n")
        f.write("chunk5\tdmel_chrU\t1\t100\n")
        f.write("chunk6\tdmel_chrU\t91\t190\n")
        f.write("chunk7\tdmel_chrU\t210\t280\n")
        f.close()

    def _writeAlign_File(self):
        f = open("annotationFile.align", "w")#coverage avant redondance 414bp
        f.write("chunk1\t1\t22\trefTE_227\t128\t145\t0\t148\t69.7\n")#22
        f.write("chunk1\t2\t40\trefTE_324\t648\t686\t0\t767\t85.8\n")#39
        f.write("chunk1\t91\t100\trefTE_617\t41\t48\t0\t580\t85.4\n")#10
        f.write("chunk2\t1\t9\trefTE_617\t41\t48\t0\t743\t84.9\n")#9  dans l'overlap entre le chunk1 et le chunk2 avec 2 hits est enleve dans gff3
        f.write("chunk2\t50\t89\trefTE_200\t50\t89\t0\t221\t95.3\n")#40
        f.write("chunk3\t3\t20\trefTE_643\t2372\t2390\t0\t760\t92.3\n")#18 dans l'overlap entre le chunk2 et le chunk3 mais il n'y a qu'un hit
        f.write("chunk3\t38\t59\trefTE_385\t1\t11\t0\t251\t98.0\n")#22
        f.write("chunk3\t63\t99\trefTE_582\t451\t485\t0\t221\t95.3\n")#37
        f.write("chunk4\t35\t65\trefTE_878\t13276\t13249\t0\t238\t90.7\n")#31
        f.write("chunk4\t70\t88\trefTE_161\t231\t249\t0\t244\t91.3\n")#19
        f.write("chunk5\t1\t22\trefTE_177\t220\t128\t145\t251\t92.3\n")#22
        f.write("chunk5\t2\t40\trefTE_315\t648\t686\t0\t244\t91.3\n")#39
        f.write("chunk5\t91\t100\trefTE_43\t41\t48\t0\t240\t90.8\n")#10
        f.write("chunk6\t56\t100\trefTE_45\t218\t260\t0\t262\t93.8\n")#45
        f.write("chunk7\t12\t62\trefTE_587\t4182\t4230\t0\t266\t94.4\n")#51
        f.close()

    def _writeFastaFile(self):
        f = open("sequences.fa", "w")
        f.write(">dmel_chr4\n")
        f.write("ATCGCTAGCTAGCTCGATCTAGTCAGTCTGTTTGGATCGCTCTCTGCTCGGAAATCCTCT\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("GTTTGGATCG\n")
        f.write(">chrU\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("ATATTCGCGCATCGATCGATCGGCGGCTATATGCTAGTCAGCTAGCTAGTGTGAGTAGTA\n")
        f.write("GCGATGCAGANNNNNNNNNNNNNNNNNNNNNNNNNNNNNNTATGTGAGGACGGCGGATGC\n")
        f.write("GCGATGCAGAATGTGAGGACGGCGGATGCATTCGGTCTCG\n")
        f.close()

    def _writeFastaFile2(self):
        f = open("sequences.fa", "w")
        f.write(">dmel_chr4\n")
        f.write("A\n")
        f.write(">chrU\n")
        f.write("A\n")
        f.close()


    def _writeConfigFile(self):
        f = open(self.configFileName, "w")
        f.write("[repet_env]\n")
        f.write("repet_host: %s\n" % os.environ["REPET_HOST"])
        f.write("repet_user: %s\n" % os.environ["REPET_USER"])
        f.write("repet_pw: %s\n" % os.environ["REPET_PW"])
        f.write("repet_db: %s\n" % os.environ["REPET_DB"])
        f.write("repet_port: %s\n" % os.environ["REPET_PORT"])
        f.write("[project]\n")
        f.write("project_name: project\n")
        f.close()
        
    def _writeExpCoverageFile(self):
        f = open("expCoverage.txt", "w")
        f.write("Total coverage is 363 bp and pct is 55.85 %\n")
        f.write("dmel_chrU\t146 bp\n")
        f.write("dmel_chr4\t217 bp\n")
        f.close()

    def _writeExpCoverageFile2(self):
        f = open("expCoverage2.txt", "w")
        f.write("Total coverage is 416309 bp and pct is 3.24 %\n")
        f.write("dmel_chrU\t70640 bp\n")
        f.write("dmel_chr4\t345669 bp\n")
        f.close()

if __name__ == "__main__":
    unittest.main()
