import os
import sys
import time
import unittest
import subprocess
from glob import glob
from shutil import rmtree
from commons.core.sql.DbMySql import DbMySql
from commons.core.utils.FileUtils import FileUtils
from commons.tools.GetMultAlignAndPhylogenyPerTErefSeq import GetMultAlignAndPhylogenyPerTErefSeq

SARUMAN_NAME = "compute-2-46.local"

class Test_F_GetMultAlignAndPhylogenyPerTErefSeq(unittest.TestCase):
    
    def setUp(self):
        self._verbosity = 3
        self._curTestDir = os.getcwd()
        
        self._testPrefix = 'test_F_GMAAPPTERS_%s' % time.strftime("%M%S")
        self._configFileName = "%sConfig.cfg" % self._testPrefix
        self._iDb = DbMySql()

        self.inPathTableName = "%sDmelChr4_chr_allTEs_nr_noSSR_join_path" % (self._testPrefix)
        self.inPathFileName = "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ["REPET_DATA"]
        self._iDb.createTable(self.inPathTableName, "path", self.inPathFileName, True)
        
        self.RefTETableName = "%sDmelChr4_refTEs" % (self._testPrefix)
        self.RefTEFileName = "%s/Tools/DmelChr4_refTEs.fa" % os.environ["REPET_DATA"]
        self._iDb.createTable(self.RefTETableName, "seq", self.RefTEFileName, True)
        
        self.genomeTableName = "%sDmelChr4_chr" % (self._testPrefix)
        self.genomeFileName = "%s/Tools/DmelChr4.fa" % os.environ["REPET_DATA"]
        self._iDb.createTable(self.genomeTableName, "seq", self.genomeFileName, True)
        
        self._resource = " test " if os.getenv("HOSTNAME") == SARUMAN_NAME else " mf=1G "

        try:
            os.makedirs(self._testPrefix)
        except:pass
        os.chdir(self._testPrefix)
        self._writeConfigFile()

    def _areTwoFilesIdenticalByScript(self, expFileName, obsFileName, scriptName):
        cmd = "%s -v 1 -r %s -t %s 2>/dev/null" % (scriptName, expFileName, obsFileName)
        log = os.system(cmd)
        print
        sys.stdout.flush()
        if log != 0:
            return False
        else:
            return True

    def tearDown(self):
        print
        self._iDb.dropTable(self.inPathTableName)
        self._iDb.dropTable(self.RefTETableName)
        self._iDb.dropTable(self.genomeTableName)
        self._iDb.close()
        
        os.chdir(self._curTestDir)
        try:
            rmtree(self._testPrefix)
        except:pass
       
    def test_run_refalign(self):
        iGMAAPperTErefSeq = GetMultAlignAndPhylogenyPerTErefSeq(pathTableName= self.inPathTableName, refSeqTableName=self.RefTETableName, genomeSeqTableName= self.genomeTableName, configFileName=self._configFileName, steps="0", verbosity=self._verbosity, clean=False)
        iGMAAPperTErefSeq.setQueue(self._resource)
        iGMAAPperTErefSeq.run()
        
        phylogenyFiles = glob("*_phyml.newick")
        self.assertTrue(len(phylogenyFiles) == 9)
        
        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_refalign.fa_aln.clean_phyml.newick"]
        
        for obsPhyTreeFileName in expFilesList:
            expPhyTreeFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", obsPhyTreeFileName)
            print "\ncomparing:\n%s\n%s" % (obsPhyTreeFileName, expPhyTreeFileName)
            sys.stdout.flush()
            self.assertTrue(FileUtils.are2FilesIdentical(expPhyTreeFileName, obsPhyTreeFileName))

    def test_run_map(self):
        iGMAAPperTErefSeq = GetMultAlignAndPhylogenyPerTErefSeq(pathTableName = self.inPathTableName, refSeqTableName = self.RefTETableName, genomeSeqTableName = self.genomeTableName, configFileName = self._configFileName, steps = "0", mSAmethod = "Map", verbosity = self._verbosity, clean = False)
        iGMAAPperTErefSeq.setQueue(self._resource)
        iGMAAPperTErefSeq.run()

        phylogenyFiles = glob("*_phyml.newick")
        self.assertTrue(len(phylogenyFiles) == 11)

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R12-Map3_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R4-Map5_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_map.fa_aln.clean_phyml.newick"]

        for obsPhyTreeFileName in expFilesList:
            expPhyTreeFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", obsPhyTreeFileName)
            print "\ncomparing:\n%s\n%s" % (obsPhyTreeFileName, expPhyTreeFileName)
            sys.stdout.flush()
            self.assertTrue(FileUtils.are2FilesIdentical(expPhyTreeFileName, obsPhyTreeFileName))
           
    def test_run_step1(self):
        iGMAAPperTErefSeq = GetMultAlignAndPhylogenyPerTErefSeq(pathTableName = self.inPathTableName, refSeqTableName = self.RefTETableName, genomeSeqTableName = self.genomeTableName, configFileName = self._configFileName, steps = "1", verbosity = self._verbosity)
        iGMAAPperTErefSeq.setQueue(self._resource)
        iGMAAPperTErefSeq.run()

        copiesFiles = glob("*_copies.fa")
        self.assertTrue(len(copiesFiles) > 0)
        self.assertTrue(os.stat("DmelChr4-B-R9-Map3_NoCat_copies.fa")[6] != 0)  # check if file is empty

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-G1-Map3_NoCat_copies.fa",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-G9-Map3_NoCat_copies.fa",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-R12-Map3_NoCat_copies.fa",
        "DmelChr4-B-R19-Map4_NoCat_copies.fa",
        "DmelChr4-B-R1-Map4_NoCat_copies.fa",
        "DmelChr4-B-R2-Map6_NoCat_copies.fa",
        "DmelChr4-B-R4-Map5_NoCat_copies.fa",
        "DmelChr4-B-R9-Map3_NoCat_copies.fa"]

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step1", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)

    def test_run_step1Step2Map(self):
        iGMAAPperTErefSeq = GetMultAlignAndPhylogenyPerTErefSeq(pathTableName = self.inPathTableName, refSeqTableName = self.RefTETableName, genomeSeqTableName = self.genomeTableName, mSAmethod = "Map", configFileName = self._configFileName, steps = "1,2", verbosity = self._verbosity)
        iGMAAPperTErefSeq.setQueue(self._resource)
        iGMAAPperTErefSeq.run()

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R12-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R4-Map5_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_map.fa_aln"]

        self.assertTrue(os.stat("DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_map.fa_aln")[6] != 0)

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step2Map", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)

    def test_run_step1Step2RefAlign(self):
        print
        iGMAAPperTErefSeq = GetMultAlignAndPhylogenyPerTErefSeq(pathTableName = self.inPathTableName, refSeqTableName = self.RefTETableName, genomeSeqTableName = self.genomeTableName, mSAmethod = "RefAlign", configFileName = self._configFileName, steps = "1,2", verbosity = self._verbosity)
        iGMAAPperTErefSeq.setQueue(self._resource)
        iGMAAPperTErefSeq.run()

        self.assertTrue(os.stat("DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_refalign.fa_aln")[6] != 0)

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R12-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R4-Map5_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_refalign.fa_aln"]

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step2RefAlign", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)

    def test_run_as_script_step1(self):
        cmd = "GetMultAlignAndPhylogenyPerTErefSeq.py -S 1 -p %s -s %s -g %s -C %s -m map -q %s" % (self.inPathTableName, self.RefTETableName, self.genomeTableName, self._configFileName, self._resource)
        subprocess.call(cmd, shell = True)
        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-G1-Map3_NoCat_copies.fa",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-G9-Map3_NoCat_copies.fa",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_copies.fa",
        "DmelChr4-B-R12-Map3_NoCat_copies.fa",
        "DmelChr4-B-R19-Map4_NoCat_copies.fa",
        "DmelChr4-B-R1-Map4_NoCat_copies.fa",
        "DmelChr4-B-R2-Map6_NoCat_copies.fa",
        "DmelChr4-B-R4-Map5_NoCat_copies.fa",
        "DmelChr4-B-R9-Map3_NoCat_copies.fa"]

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step1", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)

    def test_run_as_script_step1Step2Map(self):
        cmd = "GetMultAlignAndPhylogenyPerTErefSeq.py -S 1,2 -p %s -s %s -g %s -C %s -m map -q %s" % (self.inPathTableName, self.RefTETableName, self.genomeTableName, self._configFileName, self._resource)
        subprocess.call(cmd, shell = True)

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R12-Map3_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R4-Map5_NoCat_all.fa.oriented_map.fa_aln",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_map.fa_aln"]

        self.assertTrue(os.stat("DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_map.fa_aln")[6] != 0)

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step2Map", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)

    def test_run_as_script_step1_step2RefAlign(self):
        cmd = "GetMultAlignAndPhylogenyPerTErefSeq.py -S 1,2 -p %s -s %s -g %s -C %s -m refalign -q %s" % (self.inPathTableName, self.RefTETableName, self.genomeTableName, self._configFileName, self._resource)
        subprocess.call(cmd, shell = True)

        self.assertTrue(os.stat("DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_refalign.fa_aln")[6] != 0)

        expFilesList = ["DmelChr4-B-G11-Map20_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G1-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G7-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-G9-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-P0.0-Map3_classII-TIR-incomp_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R12-Map3_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R19-Map4_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R1-Map4_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R2-Map6_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R4-Map5_NoCat_all.fa.oriented_refalign.fa_aln",
        "DmelChr4-B-R9-Map3_NoCat_all.fa.oriented_refalign.fa_aln"]

        for obsFileName in expFilesList:
            expFileName = os.path.join(os.environ["REPET_DATA"], "Tools", "GetMultAlignAndPhylogenyPerTErefSeq", "Step2RefAlign", obsFileName)
            sys.stdout.flush()
            areSame = self._areTwoFilesIdenticalByScript(obsFileName, expFileName, "CompareTwoFastaFiles.py")
            self.assertTrue(areSame)
    
    def _writeConfigFile(self):
        f = open(self._configFileName, "w")
        f.write("[repet_env]\n")
        f.write("repet_host: %s\n" % os.environ["REPET_HOST"])
        f.write("repet_user: %s\n" % os.environ["REPET_USER"])
        f.write("repet_pw: %s\n" % os.environ["REPET_PW"])
        f.write("repet_db: %s\n" % os.environ["REPET_DB"])
        f.write("repet_port: %s\n" % os.environ["REPET_PORT"])
        f.write("repet_job_manager: SGE\n")
        f.close()
                    
if __name__ == "__main__":
    unittest.main()