import os
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.tools.giveStatsOnTEClusters import giveStatsOnTEClusters

class Test_F_RetrieveInitHeaders(unittest.TestCase):

    def setUp(self):
        self.inputFileName = "bdgpAndDmelCaf1_2_2_refTESH.fa.mcl"
        self.addedFileName = "addedFile_forMCLBenchStats.txt"
        self._writeAddedFile(self.addedFileName)
        self._writeInputFile(self.inputFileName)
        self._expResultsFileName = 'results.statsPerCluster.txt'
        self._writeExpResultsFile(self._expResultsFileName)
        self._writeExpTabResultsFile("%s.tab" % self._expResultsFileName)
        self._obsResultsFileName = 'bdgpAndDmelCaf1_2_2_refTESH.fa.globalStatsPerCluster.txt'

    def tearDown(self):
        os.remove(self.inputFileName)
        os.remove(self.addedFileName)
        FileUtils.removeFilesByPattern("*.txt")
        FileUtils.removeFilesByPattern("*.tab")

    def test_run_as_script(self):
        cmd = "giveStatsOnTEClusters.py -i %s -a %s -v 3" % (self.inputFileName, self.addedFileName)
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expResultsFileName, self._obsResultsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical("%s.tab" % self._expResultsFileName, "%s.tab" % self._obsResultsFileName))

    def test_run_as_script_withoutAdditionnalFile(self):
        self._writeExpResultsFile_withoutAdditionnalFile(self._expResultsFileName)
        self._writeExpTabResultsFile_withoutAdditionnalFile("%s.tab" % self._expResultsFileName)

        cmd = "giveStatsOnTEClusters.py -i %s -v 3" % self.inputFileName
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expResultsFileName, self._obsResultsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical("%s.tab" % self._expResultsFileName, "%s.tab" % self._obsResultsFileName))

    def test_run_withoutAdditionnalFile(self):
        self._writeExpResultsFile_withoutAdditionnalFile(self._expResultsFileName)
        self._writeExpTabResultsFile_withoutAdditionnalFile("%s.tab" % self._expResultsFileName)
        iGSOTEC = giveStatsOnTEClusters()
        iGSOTEC.setClusterFileName(self.inputFileName)
        iGSOTEC.setVerbosity(3)
        iGSOTEC.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self._expResultsFileName, self._obsResultsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical("%s.tab" % self._expResultsFileName, "%s.tab" % self._obsResultsFileName))

    def test_run(self):
        iGSOTEC = giveStatsOnTEClusters()
        iGSOTEC.setClusterFileName(self.inputFileName)
        iGSOTEC.setAddFileName(self.addedFileName)
        iGSOTEC.setVerbosity(3)
        iGSOTEC.run()
        self.assertTrue(FileUtils.are2FilesIdentical(self._expResultsFileName, self._obsResultsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical("%s.tab" % self._expResultsFileName, "%s.tab" % self._obsResultsFileName))

    def _writeInputFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("seq57\tseq266\tseq268\tseq305\tseq470\tseq660\tseq682\tseq169\tseq1121\tseq219\n")
        fG.write("seq12\tseq134\tseq1122\tseq1131\tseq52\tseq614\tseq97\tseq151\tseq204\n")
        fG.write("seq29\tseq129\tseq362\tseq1118\tseq63\tseq746\tseq124\n")
        fG.write("seq39\tseq628\tseq130\tseq159\tseq170\tseq225\n")
        fG.write("seq133\tseq611\tseq207\tseq242\tseq278\tseq334\n")
        fG.write("seq15\tseq258\tseq319\tseq417\tseq595\tseq193\n")
        fG.write("seq2\tseq153\tseq197\tseq711\tseq719\n")
        fG.write("seq24\tseq137\tseq156\tseq218\n")
        fG.write("seq99\tseq98\tseq97\n")
        fG.write("seq4\tseq148\tseq178\n")
        fG.write("seq654\tseq179\n")
        fG.write("seq299\tseq641\n")
        fG.write("seq66\tseq194\n")
        fG.write("seq18\tseq262\n")
        fG.write("seq3\n")
        fG.write("seq885\n")
        fG.write("seq1130\n")
        fG.write("seq706\n")
        fG.write("seq8\n")
        fG.write("seq267\n")
        fG.write("seq9\n")
        fG.close()

    def _writeAddedFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("seq1\nseq2\nseq3\nseq4\nseq5\nseq6\nseq7\nseq8\nseq9\nseq10\nseq11\nseq12\nseq13\nseq14\nseq15\n")
        fG.write("seq16\nseq17\nseq18\nseq19\nseq20\nseq21\nseq22\nseq23\nseq24\nseq25\nseq26\nseq27\nseq28\nseq29\n")
        fG.write("seq30\nseq31\nseq32\nseq33\nseq34\nseq35\nseq36\nseq37\nseq38\nseq39\nseq40\nseq41\nseq42\nseq43\n")
        fG.write("seq44\nseq45\nseq46\nseq47\nseq48\nseq49\nseq50\nseq51\nseq52\nseq53\nseq54\nseq55\nseq56\nseq57\n")
        fG.write("seq58\nseq59\nseq60\nseq61\nseq62\nseq63\nseq64\nseq65\nseq66\nseq67\nseq68\nseq69\nseq70\nseq71\n")
        fG.write("seq72\nseq73\nseq74\nseq75\nseq76\nseq77\nseq78\nseq79\nseq80\nseq81\nseq82\nseq83\nseq84\nseq85\n")
        fG.write("seq86\nseq87\nseq88\nseq89\nseq90\nseq91\nseq92\nseq93\nseq94\nseq95\nseq96\nseq97\nseq98\nseq99\n")
        fG.write("seq100\nseq101\nseq102\nseq103\nseq104\nseq105\nseq106\nseq107\nseq108\nseq109\nseq110\nseq111\n")
        fG.write("seq112\nseq113\nseq114\nseq115\nseq116\nseq117\nseq118\nseq119\nseq120\nseq121\nseq122\nseq123\n")
        fG.write("seq124\nseq125\nseq126\n")
        fG.close()

    def _writeExpResultsFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("nb of sequences: 74\n")
        fG.write("nb of clusters: 21\n")
        fG.write("nb of clusters with 1 sequence: 7\n")
        fG.write("nb of clusters with 2 sequences: 4\n")
        fG.write("nb of clusters with >2 sequences: 10 (with max seq 59)\n")
        fG.write("nb of sequences in the largest cluster: 10\n")
        fG.write("nb of sequences in the smallest cluster: 1\n")
        fG.write("nb true positif: 8\n")
        fG.write("nb false positif: 9\n")
        fG.write("nb false positif with only Denovo consensus: 7\n")
        fG.write("nb false positif with several bdgp: 2\n")
        fG.write("nb false negatif: 4\n")
        fG.write("nb false negatif with only 1 bdgp: 3\n")
        fG.write("nb false negatif with several bdgp: 1\n")
        fG.write("Positive Predictive Value (PPV) TP/(TP+FP): 0.47\n")
        fG.write("Sensitivity TP/(TP+FN): 0.67\n")
        fG.write("Accuracy TP+TN/(TP+FP+TN+FN): 0.38\n")
        fG.close()

    def _writeExpTabResultsFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("74\t21\t7\t4\t10\t10\t1\t8\t9\t7\t2\t4\t3\t1\t0.47\t0.67\t0.38\n")
        fG.close()

    def _writeExpResultsFile_withoutAdditionnalFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("nb of sequences: 74\n")
        fG.write("nb of clusters: 21\n")
        fG.write("nb of clusters with 1 sequence: 7\n")
        fG.write("nb of clusters with 2 sequences: 4\n")
        fG.write("nb of clusters with >2 sequences: 10 (with max seq 59)\n")
        fG.write("nb of sequences in the largest cluster: 10\n")
        fG.write("nb of sequences in the smallest cluster: 1\n")
        fG.close()

    def _writeExpTabResultsFile_withoutAdditionnalFile(self, fileName):
        fG = open(fileName, "w")
        fG.write("74\t21\t7\t4\t10\t10\t1\n")
        fG.close()

if __name__ == "__main__":
    unittest.main()
