import unittest
import os
from commons.core.utils.FileUtils import FileUtils
from commons.tools.LaunchGrouperInParallel import LaunchGrouperInParallel

class Test_F_LaunchGrouperInParallel(unittest.TestCase):
    
    def setUp(self):
        self._configFileName = "TEdenovo_grpParallel_test.cfg"
        self._inFastaFile = "DmelChr4_chunks.fa"
        os.symlink("%s/Tools/Grouper/DmelChr4_chunks.fa" % os.environ["REPET_DATA"], self._inFastaFile)
        self._inAlignFile = "DmelChr4.align.not_over.filtered"
        os.symlink("%s/Tools/Grouper/DmelChr4.align.not_over.filtered" % os.environ["REPET_DATA"], self._inAlignFile)
#        self._expFileSingleGrouper = "DmelChr4_Blaster_Grouper_3elem_20seq.fa"
#        os.symlink("%s/TEdenovo/Step3/DmelChr4_Blaster_Grouper_3elem_20seq.fa" % os.environ["REPET_DATA"], self._expFileSingleGrouper)
#        self._expFileMultiGrouper = "DmelChr4.align.not_over.filtered.match.align.group.c0.95.filtered_Concatenate.fa"
#        os.symlink("%s/Tools/Grouper/DmelChr4.align.not_over.filtered.match.align.group.c0.95.filtered_Concatenate.fa" % os.environ["REPET_DATA"], self._expFileMultiGrouper)
        self._expFileSingleGrouper = "grouperParallelOutput_1grp.fa"
        os.symlink("%s/Tools/Grouper/%s" % (os.environ["REPET_DATA"], self._expFileSingleGrouper), self._expFileSingleGrouper)
        self._expFileMultiGrouper = "grouperParallelOutput_2grp.fa"
        os.symlink("%s/Tools/Grouper/%s" % (os.environ["REPET_DATA"], self._expFileMultiGrouper), self._expFileMultiGrouper)
        self._outputFileName = "grouperParallelOutput.fa"
        
    def tearDown(self):
        try:
            os.remove(self._configFileName)
            os.remove(self._inFastaFile)
            os.remove(self._inAlignFile)
            os.remove(self._expFileSingleGrouper)
            os.remove(self._expFileMultiGrouper)
            os.remove(self._outputFileName)
            if os.path.exists("tmp.fa"):
                os.remove("tmp.fa")
        except:
            pass
        
    def test_run_as_script_2groups(self):
        projectName = self._inAlignFile.split("/")[-1].split(".")[0]
        nbGrouper = 2
        clean = "yes"
        self._writeConfigFile(projectName, nbGrouper, clean)
        
        cmd = "LaunchGrouperInParallel.py"
        cmd += " -C %s" % self._configFileName
        cmd += " -f %s" % self._inFastaFile
        cmd += " -a %s" % self._inAlignFile
        cmd += " -g Test_GrouperInParallel_script_2groups"
        cmd += " -o %s" % self._outputFileName
        cmd += " -v 3"
        os.system(cmd)
        expFile = self._expFileMultiGrouper
        obsFile = self._outputFileName
        self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
        
    def test_run_2groups(self):
        projectName = self._inAlignFile.split("/")[-1].split(".")[0]
        nbGrouper = 2
        clean = "yes"
        self._writeConfigFile(projectName, nbGrouper, clean)
        
        iLGIP = LaunchGrouperInParallel(self._configFileName, self._inFastaFile, self._inAlignFile, self._outputFileName, groupId = "Test_GrouperInParallel_instance_2groups", verbosity = 3)
        iLGIP.run()

        expFile = self._expFileMultiGrouper
        obsFile = self._outputFileName
        self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
    
    def test_run_as_script_1group(self):
        projectName = self._inAlignFile.split("/")[-1].split(".")[0]
        nbGrouper = 1
        clean = "yes"
        self._writeConfigFile(projectName, nbGrouper, clean)
        
        cmd = "LaunchGrouperInParallel.py"
        cmd += " -C %s" % self._configFileName
        cmd += " -f %s" % self._inFastaFile
        cmd += " -a %s" % self._inAlignFile
        cmd += " -g Test_GrouperInParallel_script_1group"
        cmd += " -o %s" % self._outputFileName
        cmd += " -v 3"
        os.system(cmd)
        
        # Adding end of line for the test
        os.system('sed -e "\$a\\\\" %s > tmp.fa' % self._expFileSingleGrouper)
        expFile = "tmp.fa"
        obsFile = self._outputFileName
        self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
        
    def test_run_1group(self):
        projectName = self._inAlignFile.split("/")[-1].split(".")[0]
        nbGrouper = 1
        clean = "yes"
        self._writeConfigFile(projectName, nbGrouper, clean)
        
        iLGIP = LaunchGrouperInParallel(self._configFileName, self._inFastaFile, self._inAlignFile, self._outputFileName, groupId = "Test_GrouperInParallel_instance_1group", verbosity = 3)
        iLGIP.run()
        
        os.system('sed -e "\$a\\\\" %s > tmp.fa' % self._expFileSingleGrouper)
        expFile = "tmp.fa"
        obsFile = self._outputFileName
        self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
    
    def _writeConfigFile(self, projectName, nbGrouper = 1, clean = "yes"):
        configF = open(self._configFileName, "w" )
        configF.write("[repet_env]\n")
        configF.write("repet_host: %s\n" % os.environ["REPET_HOST"])
        configF.write("repet_user: %s\n" % os.environ["REPET_USER"])
        configF.write("repet_pw: %s\n" % os.environ["REPET_PW"])    
        configF.write("repet_db: %s\n" % os.environ["REPET_DB"])
        configF.write("repet_port: %s\n" % os.environ["REPET_PORT"])
        configF.write("\n")
        configF.write("[project]\n")
        configF.write("project_name: %s\n" % projectName)
        configF.write("project_dir: %s\n" % os.getcwd())
        configF.write("\n")
        configF.write("[cluster_HSPs]\n")
        configF.write("resources: \n")
        configF.write("tmpDir: \n")
        configF.write("Grouper_nbGroup: %s\n" % nbGrouper)
        configF.write("Grouper_coverage: 0.95\n")
        configF.write("Grouper_include: 2\n")
        configF.write("Grouper_maxJoinLength: 30000\n")
        configF.write("minNbSeqPerGroup: 3\n")
        configF.write("nbLongestSeqPerGroup: 20\n")
        configF.write("maxSeqLength: 20000\n")
        configF.write("clean: %s\n" % clean)
        configF.write("\n")
        configF.write("[prepare_batches]\n")
        configF.write("chunk_overlap: 10000\n")
        configF.close()

if __name__ == "__main__":
    unittest.main()