import unittest
import os
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils

class Test_F_ListAndDropTables(unittest.TestCase):

    def setUp(self):
        self.obsListTablesFileName = "obsListTables.txt"
        self.expListTablesFileName = "expListTables.txt"
        self.configFileName = "config.cfg"
        self._iDb = DbFactory.createInstance()
        tableName = 'testTable'
        fileName = "%s.path" % tableName
        f = open(fileName, "w")
        f.write("283\tATREP11A:classII:Helitron\t1\t82\tATREP2A:classII:Helitron\t1\t82\t2e-39\t140\t96.3415\n")
        f.close()
        self._iDb.createTable(tableName+'1', "path", fileName, True)
        self._iDb.createTable(tableName+'2', "path", fileName, True)
        os.remove(fileName)

    def tearDown(self):
        os.remove(self.obsListTablesFileName)
        os.remove(self.expListTablesFileName)
          

    def test_run_as_script_listTables(self):
        self._writeExpFile()
        cmd = "ListAndDropTables.py"
        cmd += " -l 'test'"
        cmd += " -v 1 > %s" % self.obsListTablesFileName
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self.expListTablesFileName, self.obsListTablesFileName))
        self._iDb.dropTable('testTable1')    
        self._iDb.dropTable('testTable2')
    
    def test_run_as_script_listTables_withConfigFile(self):
        self._writeConfigFile()
        self._writeExpFile_withConfigFile()
        cmd = "ListAndDropTables.py"
        cmd += " -l 'test'"
        cmd += " -C %s" % self.configFileName
        cmd += " -v 1 > %s" % self.obsListTablesFileName
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self.expListTablesFileName, self.obsListTablesFileName))
        self._iDb.dropTable('testTable1')    
        self._iDb.dropTable('testTable2')
        os.remove(self.configFileName)
    
    def test_run_as_script_dropTables(self):
        self._writeExpFile_forDrop()
        cmd = "ListAndDropTables.py"
        cmd += " -d 'toto' > %s" % self.obsListTablesFileName
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self.expListTablesFileName, self.obsListTablesFileName))    
       
    def test_run_as_script_dropTables_withConfigFile(self):
        self._writeConfigFile()
        self._writeExpFile_forDrop()
        cmd = "ListAndDropTables.py"
        cmd += " -d 'toto'"
        cmd += " -C %s" % self.configFileName
        cmd += " > %s" % self.obsListTablesFileName
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self.expListTablesFileName, self.obsListTablesFileName))
        os.remove(self.configFileName)
    
    def _writeExpFile(self):
        f = open(self.expListTablesFileName, "w")
        f.write("START ListAndDropTables.py\n")
        f.write("WARNING: use environment variables to connect to MySQL\n")
        f.write("list of tables:\n")
        f.write("testTable1\n")
        f.write("testTable2\n")
        f.write("2 tables corresponding to 'test'\n")
        f.write("END ListAndDropTables.py\n")
        f.close()

    def _writeExpFile_forDrop(self):
        f = open(self.expListTablesFileName, "w")
        f.write("no table corresponding to 'toto'\n")
        f.close()

    def _writeExpFile_withConfigFile(self):
        f = open(self.expListTablesFileName, "w")
        f.write("START ListAndDropTables.py\n")
        f.write("list of tables:\n")
        f.write("testTable1\n")
        f.write("testTable2\n")
        f.write("2 tables corresponding to 'test'\n")
        f.write("END ListAndDropTables.py\n")
        f.close()


    def _writeConfigFile(self):
        configF = open(self.configFileName, "w" )
        configF.write("[repet_env]\n")
        configF.write("repet_host: %s\n" % os.environ["REPET_HOST"])
        configF.write("repet_user: %s\n" % os.environ["REPET_USER"])
        configF.write("repet_pw: %s\n" % os.environ["REPET_PW"])
        configF.write("repet_db: %s\n" % os.environ["REPET_DB"])
        configF.write("repet_port: %s\n" % os.environ["REPET_PORT"])
        configF.close()
        
if __name__ == "__main__":
    unittest.main()