# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


import unittest
import os
import time
import shutil
from commons.core.utils.FileUtils import FileUtils
from commons.tools.NoCatBestHitClassifier import NoCatBestHitClassifier


class Test_F_NoCatBestHitClassifier( unittest.TestCase ):
    
    def setUp(self):
        self._uniqId = "NoCatBestHitClassifier%s_%s" % ( time.strftime("%Y%m%d%H%M%S") , os.getpid() )
        self._inFastaFileName = "%s/Tools/consensus_classif_noCatBestHit.fa" % os.environ["REPET_DATA"]
        self._inClassifFileName = "%s/Tools/consensus_classif_noCatBestHit.classif" % os.environ["REPET_DATA"]
        os.makedirs(self._uniqId)
        
        self.curDir = os.getcwd()
        os.chdir(self._uniqId)

    def tearDown( self ):
        os.chdir(self.curDir)
        shutil.rmtree(self._uniqId)
        self._uniqId = None

        
    def test_run(self):
        iNoCatBestHitClassifier = NoCatBestHitClassifier(self._inFastaFileName, self._inClassifFileName)
        iNoCatBestHitClassifier.run()
        
        expClassifFileName = "%s/Tools/exp_consensus_classif_noCatBestHit.classif" % os.environ["REPET_DATA"]
        obsClassifFileName = "%s_noCatBestHit.classif" % os.path.splitext(os.path.basename(self._inClassifFileName))[0]
        
        self.assertTrue(FileUtils.are2FilesIdentical(expClassifFileName, obsClassifFileName))
        
        expFastaFileName = "%s/Tools/exp_consensus_classif_noCatBestHit_noCatBestHit.fa" % os.environ["REPET_DATA"]
        obsFastaFileName = "%s_noCatBestHit.fa" % os.path.splitext(os.path.basename(self._inFastaFileName))[0]
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFastaFileName, obsFastaFileName))

    def test_run_as_script(self):

        expFastaFileName = "%s/Tools/exp_consensus_classif_noCatBestHit_noCatBestHit.fa" % os.environ["REPET_DATA"]
        expClassifFileName = "%s/Tools/exp_consensus_classif_noCatBestHit.classif" % os.environ["REPET_DATA"]
        obsFastaFileName = "consensus_classif_NOCATBESTHIT.fa"
        obsClassifFileName = "consensus_classif_NOCATBESTHIT.classif"
        cmd = "NoCatBestHitClassifier.py -i %s -o %s -C %s" % (self._inFastaFileName, obsFastaFileName, self._inClassifFileName)
        os.system(cmd)

        self.assertTrue(FileUtils.are2FilesIdentical(expClassifFileName, obsClassifFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expFastaFileName, obsFastaFileName))
    
if __name__ == "__main__":
    unittest.main()
