import os
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.tools.RenameHeaderClassif import RenameHeaderClassif

class Test_F_RenameHeaderClassif(unittest.TestCase):

    def tearDown(self):
        FileUtils.removeFilesByPattern("*.fa")
        FileUtils.removeFilesByPattern("*.classif")
        FileUtils.removeFilesByPattern("*.json")

    def test_run_small(self):
        inputFastaFileName = "test.fa"
        self._writeFastaFile(inputFastaFileName)
        inputClassifFileName = "test.classif"
        self._writeClassifFile(inputClassifFileName)
        expFileName = "exptest_WickerH.fa"
        self._writeExpFastaFile(expFileName)
        obsFileName = "test_WickerH.fa"
        
        iRHC = RenameHeaderClassif()
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.setOutputFileName(obsFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        
    def test_run_small_but_real_data(self):
        inputFastaFileName = "test_real_data.fa"
        self._writeRealFastaFile(inputFastaFileName)
        inputClassifFileName = "test.classif_real_data.fa"
        self._writeRealClassifFile(inputClassifFileName)
        expFileName = "exptest_WickerH_real_data.fa"
        self._writeExpRealFastaFile(expFileName)
        obsFileName = "test_WickerH_real_data.fa"
        
        iRHC = RenameHeaderClassif()
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.setOutputFileName(obsFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        
    def test_run_withShortenOption(self):
        inputClassifFileName = "test.classif"
        self._writeClassifFileWithShortenOption(inputClassifFileName)
        inputFastaFileName = "test.fa"
        self._writeFastaFileWithShortenOption(inputFastaFileName)
        expFileName = "exptest_WickerH.fa"
        self._writeExpFileWithShortenOption(expFileName)
        obsFileName = "obs_WickerH.fa"
        
        iRHC = RenameHeaderClassif()
        iRHC.setShorten(True)
        iRHC.setProjectName("Vigne_1_3_12_1")
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.setOutputFileName(obsFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        
    def test_run_withShortenOption_and_renameInClassif_as_script(self):
        projectName = "Vigne_1_3_12_1"
        inputClassifFileName = "test.classif"
        self._writeClassifFileWithShortenOption(inputClassifFileName)
        inputFastaFileName = "test.fa"
        self._writeFastaFileWithShortenOption(inputFastaFileName)
        expFileName = "exptest_WickerH.fa"
        self._writeExpFileWithShortenOption(expFileName)
        expClassifFileName = "exptest_WickerH.classif"
        self._writeExpClassifFileWithShortenOption(expClassifFileName)
        obsFileName = "test_WickerHeader.fa"
        obsClassifFileName = "test_WickerHeader.classif"
        
        cmd = "RenameHeaderClassif.py -c %s -i %s --output %s -s -r -p %s" % (inputClassifFileName, inputFastaFileName, obsFileName, projectName)
        os.system(cmd)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expClassifFileName, obsClassifFileName))
        
    def test_run_renameInClassif(self):
        projectName = "Vigne_1_3_12_1"
        inputClassifFileName = "test.classif"
        self._writeClassifFile(inputClassifFileName)
        inputFastaFileName = "test.fa"
        self._writeFastaFile(inputFastaFileName)
        expFileName = "exptest_WickerH.fa"
        self._writeExpFastaFile(expFileName)
        expClassifFileName = "exptest_WickerH.classif"
        self._writeExpClassifFile(expClassifFileName)
        obsFileName = "test_WickerH.fa"
        obsClassifFileName = "test_WickerH.classif"
        
        iRHC = RenameHeaderClassif()
        iRHC.setRenameInClassif(True)
        iRHC.setProjectName(projectName)
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.setOutputFileName(obsFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expClassifFileName, obsClassifFileName))
        
    def test_run_withShortenOption_and_renameInClassifRSHeaders(self):
        projectName = "Vigne_1_3_12_1"
        inputClassifFileName = "test.classif"
        self._writeClassifFileWithShortenOptionRSHeaders(inputClassifFileName)
        inputFastaFileName = "test.fa"
        self._writeFastaFileWithShortenOptionRSHeaders(inputFastaFileName)
        
        obsFileName = "test_WickerH.fa"
        obsClassifFileName = "test_WickerH.classif"
        
        expFileName = "exptest_WickerH.fa"
        self._writeExpFileWithShortenOptionRSHeaders(expFileName)
        expClassifFileName = "exptest_WickerH.classif"
        self._writeExpClassifFileWithShortenOptionRSHeaders(expClassifFileName)
        
        iRHC = RenameHeaderClassif()
        iRHC.setRenameInClassif(True)
        iRHC.setShorten(True)
        iRHC.setProjectName(projectName)
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.setOutputFileName(obsFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expClassifFileName, obsClassifFileName))
    
    def test_run_renameInJsonClassif(self):
        projectName = "inputRenameHeaderClassif"
        inputClassifFileName = "data/inputRenameHeaderClassif.classif"
        inputFastaFileName = "data/inputRenameHeaderClassif.fa"
        obsJsonClassifFileName = "inputRenameHeaderClassifClassif_WickerH.json"
        expJsonClassifFileName = "data/expInputRenameHeaderClassifClassif_WickerH.json"
        outFileName = "test_WickerH.fa"
        
        self._writeClassifFile(inputClassifFileName)
        self._writeFastaFile(inputFastaFileName)
        
        
        iRHC = RenameHeaderClassif()
        iRHC.setRenameInClassif(True)
        iRHC.setProjectName(projectName)
        iRHC.setOutputFileName(outFileName)
        iRHC.setClassifFileName(inputClassifFileName)
        iRHC.setFastaFileName(inputFastaFileName)
        iRHC.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expJsonClassifFileName, obsJsonClassifFileName))
        
    def _writeClassifFile(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("gypsy5\t7369\t+\tok\tI\tLTR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1-B-G3678-Map14_classI-LTR-comp\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("copia\t5143\t+\tPotentialChimeric\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("gypsy7\t5486\t+\tPotentialChimeric\tI\tLTR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DIRS1:classI:LTR_retrotransposon\t7053\t+\tPotentialChimeric\tI\tDIRS\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("PENELOPE:classI:LINE\t2780\t+\tPotentialChimeric\tI\tPLE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DummyConfusedOrderConfusedClassDifferentIC\t2042\t+\tPotentialChimeric\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("ALU\t312\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("1360\t3409\t+\tok\tII\tTIR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Crypton-1_TC:classII:TIR\t1729\t+\tPotentialChimeric\tII\tCrypton\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("INE-1\t611\t+\tok\tII\tHelitron\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Polinton-1_SM:classII:Polinton\t14867\t+\tok\tII\tMaverick\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DIRS_Nvi:classI:LTR_retrotransposon\t5921\t+\tPotentialChimeric\tI\tnoCat\tNA\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Unknown-B-R21164-Map3_NoCat\t755\t+\tok\tII\tnoCat\tNA\tCI=50; coding=(); struct=(); other=()\n")
        f.write("SSR\t780\t.\tPotentialChimeric\tNA\tSSR\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("GQ167354\t204\t.\tok\tNA\trDNA\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("glycosylase\t7174\t.\tok\tNA\tPotentialHostGene\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("UCON3\t172\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=()\n")
        f.write("GQ167353\t200\t.\tPotentialChimeric\tnoCat\tnoCat\tNA\tCI=NA; struct=()\n")
        f.write("dummyWithPipeClassNoCat\t200\t+\tPotentialChimeric\tnoCat\tTRIM|MITE\tNA\tCI=NA; struct=()\n")
        f.write("dummyWithPipeClassI\t200\t+\tPotentialChimeric\tI\tLTR|LARD\tNA\tCI=NA; struct=()\n")
        f.write("dummyWithPipeClassNA\t90\tNA\tPotentialChimeric\tNA\tPotentialHostGene|SSR\tNA\tCI=NA; struct=()\n")
        f.close()
        
    def _writeFastaFile(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">gypsy5\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1-B-G3678-Map14_classI-LTR-comp\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">copia\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">gypsy7\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DIRS1:classI:LTR_retrotransposon\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">PENELOPE:classI:LINE\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DummyConfusedOrderConfusedClassDifferentIC\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">ALU_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">1360\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Crypton-1_TC:classII:TIR\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">INE-1\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Polinton-1_SM:classII:Polinton\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DIRS_Nvi:classI:LTR_retrotransposon\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Unknown-B-R21164-Map3_NoCat\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">SSR\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">GQ167354\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">glycosylase\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">UCON3\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">GQ167353\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">dummyWithPipeClassNoCat\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">dummyWithPipeClassI\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">dummyWithPipeClassNA\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.close()
        
    def _writeExpClassifFile(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("RLX-incomp_gypsy5\t7369\t+\tok\tI\tLTR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1-B-G3678-Map14_classI-LTR-comp\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp-chim_copia\t5143\t+\tPotentialChimeric\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-incomp-chim_gypsy7\t5486\t+\tPotentialChimeric\tI\tLTR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RYX-comp-chim_DIRS1:classI:LTR_retrotransposon\t7053\t+\tPotentialChimeric\tI\tDIRS\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RPX-incomp-chim_PENELOPE:classI:LINE\t2780\t+\tPotentialChimeric\tI\tPLE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RIX-incomp-chim_DummyConfusedOrderConfusedClassDifferentIC\t2042\t+\tPotentialChimeric\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RSX-comp_ALU_reversed\t312\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DTX-comp_1360\t3409\t+\tok\tII\tTIR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DYX-incomp-chim_Crypton-1_TC:classII:TIR\t1729\t+\tPotentialChimeric\tII\tCrypton\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DHX-incomp_INE-1\t611\t+\tok\tII\tHelitron\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DMX-incomp_Polinton-1_SM:classII:Polinton\t14867\t+\tok\tII\tMaverick\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RXX-chim_DIRS_Nvi:classI:LTR_retrotransposon\t5921\t+\tPotentialChimeric\tI\tnoCat\tNA\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DXX_Unknown-B-R21164-Map3_NoCat\t755\t+\tok\tII\tnoCat\tNA\tCI=50; coding=(); struct=(); other=()\n")
        f.write("SSR-chim_SSR\t780\t.\tPotentialChimeric\tNA\tSSR\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("rDNA_GQ167354\t204\t.\tok\tNA\trDNA\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("PotentialHostGene_glycosylase\t7174\t.\tok\tNA\tPotentialHostGene\tNA\tCI=100; coding=(); struct=(); other=()\n")
        f.write("noCat_UCON3\t172\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=()\n")
        f.write("noCat-chim_GQ167353\t200\t.\tPotentialChimeric\tnoCat\tnoCat\tNA\tCI=NA; struct=()\n")
        f.write("XXX-chim_dummyWithPipeClassNoCat\t200\t+\tPotentialChimeric\tnoCat\tTRIM|MITE\tNA\tCI=NA; struct=()\n")
        f.write("RXX-chim_dummyWithPipeClassI\t200\t+\tPotentialChimeric\tI\tLTR|LARD\tNA\tCI=NA; struct=()\n")
        f.write("nonTE-chim_dummyWithPipeClassNA\t90\tNA\tPotentialChimeric\tNA\tPotentialHostGene|SSR\tNA\tCI=NA; struct=()\n")
        f.close()
        
    def _writeExpFastaFile(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">RLX-incomp_gypsy5\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1-B-G3678-Map14_classI-LTR-comp\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp-chim_copia\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-incomp-chim_gypsy7\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RYX-comp-chim_DIRS1:classI:LTR_retrotransposon\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RPX-incomp-chim_PENELOPE:classI:LINE\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RIX-incomp-chim_DummyConfusedOrderConfusedClassDifferentIC\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RSX-comp_ALU_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DTX-comp_1360\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DYX-incomp-chim_Crypton-1_TC:classII:TIR\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DHX-incomp_INE-1\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DMX-incomp_Polinton-1_SM:classII:Polinton\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RXX-chim_DIRS_Nvi:classI:LTR_retrotransposon\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DXX_Unknown-B-R21164-Map3_NoCat\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">SSR-chim_SSR\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">rDNA_GQ167354\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">PotentialHostGene_glycosylase\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">noCat_UCON3\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">noCat-chim_GQ167353\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">XXX-chim_dummyWithPipeClassNoCat\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RXX-chim_dummyWithPipeClassI\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">nonTE-chim_dummyWithPipeClassNA\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.close()
        
    def _writeClassifFileWithShortenOption(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("Vigne_1_3_12_1_Blaster_Grouper_3678_Map_14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_Blaster_Grouper_3678_Map_15\t2154\t+\tok\tII\tTIR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_Blaster_Recon_3_Map_14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_16\t2154\t+\tok\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_Blaster_Piler_212_Map_14\t2154\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_Blaster_Piler_213_Mafft_14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_Blaster_Piler_214_Mafft_14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.close()
        
    def _writeFastaFileWithShortenOption(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">Vigne_1_3_12_1_Blaster_Grouper_3678_Map_14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_Blaster_Grouper_3678_Map_15_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_Blaster_Recon_3_Map_14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_16_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_Blaster_Piler_212_Map_14_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_Blaster_Piler_213_Mafft_14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_Blaster_Piler_214_Mafft_14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.close()
        
    def _writeExpClassifFileWithShortenOption(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("RLX-comp_Vigne_1_3_12_1-B-G3678-Map14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DTX-incomp_Vigne_1_3_12_1-B-G3678-Map15_reversed\t2154\t+\tok\tII\tTIR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1-B-R3-Map14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RIX-incomp_Vigne_1_3_12_1_RS_16_reversed\t2154\t+\tok\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RSX-comp_Vigne_1_3_12_1-B-P212-Map14_reversed\t2154\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1-B-P213-Mafft14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1-B-P214-Mafft14\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.close()
        
    def _writeExpFileWithShortenOption(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">RLX-comp_Vigne_1_3_12_1-B-G3678-Map14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DTX-incomp_Vigne_1_3_12_1-B-G3678-Map15_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1-B-R3-Map14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RIX-incomp_Vigne_1_3_12_1_RS_16_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RSX-comp_Vigne_1_3_12_1-B-P212-Map14_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1-B-P213-Mafft14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1-B-P214-Mafft14\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.close()
        
    def _writeRealClassifFile(self, fileName):
        f = open(fileName, "w")
        f.write("Vigne_1_3_12_1-B-R13403-Map11_NoCat\t5129\t-\tok\tII\tHelitron\tcomplete\tCI=23; coding=(TE_BLRtx: Helitron-1_Mad:classII:Helitron:Helitron: 34.33%); struct=(TElength: >4000bps; TermRepeats: no termLTR, no termTIR; ORF: >3000bps); other=(HG_BLRn: Vv13s0101g00300; coverage: 84.23%; identity: 99.00%; SSRCoverage=0.05<0.95)\n")
        f.write("Vigne_1_3_12_1-B-R21164-Map3_classI-LTR-comp\t755\t.\tok\tI\tnoCat\tNA\tCI=14; struct=(TermRepeats: termLTR); other=(HG_BLRn: Vv7s0095g00510; coverage: 9.14%; identity: 95.65%; TElength: <700bps; TermRepeats: no termTIR, non-termLTR; SSRCoverage=0.03<0.95)\n")
        f.write("Vigne_1_3_12_1-B-R21665-Map6_classI-LTR-incomp\t309\t.\tok\tNA\tSSR\tNA\tCI=100; struct=(TElength: <100bps; SSRCoverage=0.00<0.95); other=(TermRepeats: no termTIR, termLTR, non-termLTR)\n")
        f.write("Vigne_1_3_12_1-B-R1512-Map7_classI-LINE-comp\t1950\t-\tPotentialChimeric\tI\tLINE\tincomplete\tCI=46; coding=(TE_BLRtx: VLINE6_VV:classI:LINE:L1: 93.85%; TE_BLRx: VLINE3_VV_2p#2:classI:LINE:L1: 98.08%; profiles: RVT_1_RT_35.2: 52.07%); struct=(TElength: >700bps; TermRepeats: no termLTR, no termTIR; SSR: (T)10_start; polyAtail); other=(HG_BLRn: Vv6s0061g01580; coverage: 16.21%; identity: 92.06%; SSRCoverage=0.15<0.95)\n")
        f.close()
        
    def _writeRealFastaFile(self, fileName):
        f = open(fileName, "w")
        f.write(">Vigne_1_3_12_1-B-R21164-Map3_classI-LTR-comp\n")
        f.write("ACGCCAAATGGAAAATGCTAATGGCTCTGAACGCCGAACCGAGAAGAGATAATGGCTCTG\n")
        f.write("AATACCAAATGGGAAATGTTCAGGCTCTAAACACCGAGACTGAGAAGAAATAGTGGCTCT\n")
        f.write("GAACGCCAAATGGAAAATGTTGATGGCTCTGAACGCCAAGACTGAGAAGAAATAGTGGAT\n")
        f.write("CTGAAAGCCAAAAGGAACATGCTGATGGCCCTGAACGCCGAGACTGAGAAGAAATAGTGG\n")
        f.write("CTCTGAATGCCAAGTGAAAAATGCTGATGGCTCTGAATGCCAAGACTGAGAAGAAATAGT\n")
        f.write("GGATCTAAACGCCAAATGGAAAATGATGATGGCTCTGAACGCCAAACTAAGAAGAAATAG\n")
        f.write("TGGCTCTGAACGCCAAATGCGAAATGCTGATGGCTCTGAACACCGATCTAAGAAGAAATA\n")
        f.write("ATGGCTCTGAACGCCAAATGGAAAATACTGATGGCCCTGAACGTCGAGACTAAGAAGAAA\n")
        f.write("TAGTGGCTCTGAACGCCAAATGGAAAATGCTGATGGCCCTAAACGCCAATCTGATAAGAA\n")
        f.write("ATAATGGCTCTAAACGCCAAATGGAAAATGCTGATGGCCCTGAACACCAAGACTAAAAAG\n")
        f.write("AAATAGTGGCTCTGAACGCCAAATGAAAAATTCTGATGGCCCTGAACGCCAAGACTGAGA\n")
        f.write("AGAAATAATGGCTCTGAACGCCAAATGAAAAATGTTGATGGCTATGAATGCCAAACTGAG\n")
        f.write("AAGAAATAGTGGCTCTGAACGCCAAATGAAAAATG\n")
        f.write(">Vigne_1_3_12_1-B-R1512-Map7_classI-LINE-comp\n")
        f.write("TTTTTTTTTTGATAAGTAAGCACAAAATTCAATAAAAAGAGGCAAAATAGCCACAAAGTA\n")
        f.write("TACAGGGGGTATACAAGGCTACTAAGGCCTCACAAGCAAAAAACCAAAAAAACCCACCAC\n")
        f.write("CCCTTATCTGGAGGCTAACCACTCCAGAAAGCCTATGAGGGAAGGCGGCTCCTCTCCAAT\n")
        f.write("ATACAATTTAGCCCAACCCCATATATTACAAACAAAAGAATATTTAAGTCTCTGAATTGC\n")
        f.write("TAACCCCCCCCCCCCCCCCTTAAAAGCTAGTCTATTCCTCTCCTTCCAAACCGTCCAAAA\n")
        f.write("AATAAACAACGGAATGGAATTCCAAAGTTTTTTCCTTTTTTTCCCCACAAAGGATCCCCT\n")
        f.write("CCAACTAAATAAAACCTCCTTTACAGTTTCTGGAAACACCCACTGAACCCCAAACAATCC\n")
        f.write("CAAAACTAAATCCCACAGCACTCTGGCCACTGTACAATGTATGAGAATATGATTTACATT\n")
        f.write("TTCCTCTTCACAACCACACAAAAAACAACGATTGGGAAGCTGCCACCCTCTCTTCTGGAG\n")
        f.write("TCTATCAAGAGTCATAATCTTCCCCCAAGCGGCTTCCCACGCGAAGAACACAATTTTAGT\n")
        f.write("TGGAACTCTCTCCACCCATATGTTATTTTTTGGAAAAACGGCCGTCAAAGGGCTTTCCAA\n")
        f.write("CAGGCTGTATGCTTTCTTTACTCTAAACTGCCCGTTTCTTCCTTCCTTCCAAAAAACCGA\n")
        f.write("ATCTTCCTCCAAAGTTATCCTGTGACCCCTCAAAACATGAAGCAAGTTGCCTACCATATC\n")
        f.write("CAATTCCCAATCGTTAAAATCCCTAATAAACCTCAAATTCCATCCTCCTTCGCCAAAATT\n")
        f.write("CTGGTCCCACATTTCTTCTATTGTCGCATTCCTATGAGCAGCCAAAGTGAAGAGATGCGA\n")
        f.write("GAATCTTTAGGACAACACTGTACCCCCACACCAAAGATCAGTCCAAAACCTGATTTTAGT\n")
        f.write("GCCTTTTCCCACTTTAAACGCCAATTTTTCCCAACACCATCCAACTTCCTTCAAAATCTC\n")
        f.write("CTTCCAAACCCCCACTCCAAACGCACCAAAAGCCTTTTTTGTCCTCCAACCAAAATCCTC\n")
        f.write("TTGCCCATACTTCGCCATGAGCACTTGCTTCCAAAGATCCTCCTTAGCACAAGCAAATCT\n")
        f.write("CCAAATCCATTTGCCAAGCAAGGATTTGTTCAAGAGGGTTAGCTTCCTTATGCCAAGCCC\n")
        f.write("CCCTTTCTCCTTATCCACACAAACCGCCTCCCAATTGACAAGGTGAGCTTTCCTTTCCAG\n")
        f.write("ATTTCCTCCTCCCCACAAAAAATCTCTTTGCAGTTTTTCTAACCTCCTTGCCACAGACTT\n")
        f.write("GGGCATGCTGAAAAAGGACATTTGATATACTGGCATGCTAGCCATCGTGCTCTTAATAAG\n")
        f.write("AGTGATTCTCCCACCTTTTGAAATATATTGGCGTTTCCAAAGGACAAGTCTCCTCCTCAC\n")
        f.write("TCTCTCTTCCACCCCATCCCACAAGGAAGCGGCCTTATTTGGCGCCCCCAGTGGCAACCC\n")
        f.write("CAAGTAGACTGCAGGCAGCTGCCCCACCCTAGCTCCACTGCCATTTCGTCCACCTCTCCA\n")
        f.write("ACCAAGATGATTTCACTTTTAGCCAAATTAATTCTTAACCCTGAAGCAGCTTCAAACCAA\n")
        f.write("AAAAGAATCCAACTCAAATGCGTTAAATGCTCATTCTTTACATCACAAAAAACGATTGTG\n")
        f.write("TCATCAGCAAAGAGCAACTGTGAAATGTGGACAGCCCACCCTCTACCACGCCGAATGTTA\n")
        f.write("CACCCCGAAATAAAACCCCCTTCCACAGCCCTTCTGATTAAAACGCTCAACACTTCCATT\n")
        f.write("CCCATGACAAAAAGATAGAGGGAAAGAGGGTCCCCTTGGCGCAGCCCCTTCGAAATTGAG\n")
        f.write("AAGAAACCAACTGGAACTCCGTTCACCAAGACTAAGAACTTGGCTATGGAAATGCAACAC\n")
        f.write("CACATCCAACTCAACCACTTTGAACCAAAC\n")
        f.write(">Vigne_1_3_12_1-B-R21665-Map6_classI-LTR-incomp\n")
        f.write("CACCTTAGGCCAACTTAGGCACCTTAGGCCCACTTAGGCACCTTAGGCCCACTTAGGCAC\n")
        f.write("CTTAGGCTCACTTAGGCACCTTAGGCCCACTTAGGCAACTTAGGCCCACTTAGGCACCAC\n")
        f.write("TAGGCCCACTTAGGCACCCTAGGCCCACTTAGGCACCCTAGGCCCACTTAGGTACCCTAG\n")
        f.write("GCCCACTTAGGTACCCTAGGCCCATTTAGGCACCCTAGGCCCACTTAGGCACCCTAAGCC\n")
        f.write("CACTTAGGCACCCTAGGACCACTTAGGCACCCTAGGCTCATTTAGGCACCTTAGGCCCAA\n")
        f.write("TTAGGCACA\n")
        f.write(">Vigne_1_3_12_1-B-R13403-Map11_NoCat\n")
        f.write("GAAAGAAAAGTTTAAGCACAAAGGAATCACCTGACTGTGTATTGGCAGAGTGGCGGAGTG\n")
        f.write("GCACTTTGATTAAAATAGCACTTCATTGATCACTATTTTAGGAATGAGATCTATATAACA\n")
        f.write("CCATTCTTCCCCTTTCTCAAATTGACACCGTTTTTCCAGTAAATGACCTCATTATAACTA\n")
        f.write("CAATAAGTGAAGTGATAATGTCATCTAATTAGTAAACCAAACCAAGTCAATAAACTAGAA\n")
        f.write("CTTCCTCTAGTATAGCCTCTGTGTAAAAGATATTTAAAGGTCTTGATACATTAGATATTG\n")
        f.write("AATTTGTTGGATTAAAATCTCATTTATTGCACCTGGCAAGGTAGAAGTAGTAATGCTCAT\n")
        f.write("TTTTTATTGTGACAGATTATAGAGAAAATTAGACACTGGGAAAATACCAAGCCAGGAGCC\n")
        f.write("ATGTCAAGAGCTGCTTTCTTGTTAAAAATTAAGTCCAATTAGAAAATTACAGTAGCGCGT\n")
        f.write("AAGAGAAATTGAAAGTGGTGTTGAGAAGAGAAAGAGGAAGAATGATGTGTTGGGAATGAG\n")
        f.write("ATGTAATACCTTATTTATTACTCAGATGTGGATTAGTCATTGTATCTCCACACAGGGGAT\n")
        f.write("GTGGGCAATCTTGGGCCAATCATCTCCTTCCTCCTTCGAGTACCTTTGTGTTAGATGTGG\n")
        f.write("GCAACGCCTCACATACAGTCGTGAAAGGGTGTCAGGCAATAGTCCTTCCCTTGGCAAAAT\n")
        f.write("CGACCGGAGCTTAGGGCAACTTTCGATTTCTAGATATTCAAGAGAGGTGAGGGTTTGGAG\n")
        f.write("AGACAGGGAGGCTAGGGATTCCAGATTCTGGAAATGTGAAAGGGTAAGGGAGGTGAGAGT\n")
        f.write("TGTAGGAAAAAGAATCGAGTGGGGGTCATCCGAAAAGGAAGTTGCATCTGGAAACATGCC\n")
        f.write("ACCAATCCAGAGGTCTTTAAGAGAGGTGAGTCTGGAAAGGCCCCATTGGGATAGGGGGGT\n")
        f.write("CTTGATATTCTCACAATTACGAATGAAAAGCGATGTAAGACGAGTGAGGTTTTTTATCTG\n")
        f.write("AGGAAGCAACAATTCCAGATTTTCAAAATCTACAATACGTAGATCTGTGAGGGTGTTGAG\n")
        f.write("GCAATCTGGTAGGGTTTTGAGATTAGGATACCTCCTGAGGGTTAAAGATTGAAGTGAATT\n")
        f.write("ATTAGTAGAGTGAAACATCTCCTCTGAAATTGACTCGAGGTGTTCACAATCCCCAATGTG\n")
        f.write("AAGTCGCTCAAGGGTGGATGGAAACTTGCCTCTTGGGAAGGATGTGAGAGATGGACATTT\n")
        f.write("ACAGATTTCCAAGGACTTGGAGAGCAGCAGCATTGGTGGAATGTTGGTGCATTATTCCCT\n")
        f.write("CTGGTAGAGACTCTAGCCTTCTACAATCACCTATTCTGAGTCTCTTGAGAGTGGCGGGTA\n")
        f.write("ACCCACCTTTCGGTAATCCAATAAGAGAATGACACCTATCTATGAATAAGCCTTCAAGGG\n")
        f.write("CACACATGCCCATCATTCCTTCTGGAAGTGACTTGAGATTTTCACAAGACGATATGCTTA\n")
        f.write("GACTCTTAAGGGTGGTGGGTAATTGCCCTTTTGGAAAGCAAATGAGAGATGGACACGCTG\n")
        f.write("TATATCACTAGCTCTTCAAGGAGACACAAGTTGTTGCTGTCAGTACTGTCGTTCCTCATC\n")
        f.write("TTCAACATCATTTCATCAGGTAGACACTCAAGACCTTCACAATTATCAAGAATAAGATTT\n")
        f.write("CTCAGCATTGGTGGGAAACCTACATCTGGAAATGACGCTAGCTTTGGACAATTCCTGATT\n")
        f.write("GTCAATTCTTCAAGGCATGTTAGACTCTGCCATCCATTTGGAAGCCTCTCTAGTTTATCA\n")
        f.write("CATTTAATTATTTCCAAAGACTGAAGATTGCATCCCAAGGATACAAGTTGATCACAATCT\n")
        f.write("CTAATCTCAAGAGAATGAGAGTTTTCCGATCCAAAACCATCCTCCCACAAATATTCGAGT\n")
        f.write("TCTTCGCATTCCCATACTTTCAAAACCCGAAGCCCTTGCAAAAATTGCACAAATCCTTCA\n")
        f.write("TGCAATTTGATAAGCCCTGAAATCCCAGAAATTGTTAACTTGGTGAGTGAGGTGAGGTCA\n")
        f.write("TTTCCACTGCTCAAGACCGCCTCATTACATCCTCTTACTTGTAATTCCTTCAGCAGTGGA\n")
        f.write("AGTCTTGAAAGTGGAGACTCCAATTTTGGACAAAAGTGAACAGAGAGCTTTGTAAGAGAA\n")
        f.write("GGTAGGTAAGTGGGTAGTTTCATAATCAATTTGGGGCAATCCTCAATTGTAAGCTCATGG\n")
        f.write("AGACAAGGGAATAATGACTCTGTTGAGGAAGACCAATCCTCCCAGTGCTCCCATTCTGAC\n")
        f.write("ATGCTATTAAAATGTAGAGACTCTAGTGATGGGAAAAACTTACCTGCAGAAACACGAGTC\n")
        f.write("TCTCCATAAAACTCTGCACCCACTTTTTTTACTCCATCCATTCCTTGGATCCTCAACTGT\n")
        f.write("TTGAGCGATGGCAACTGCCCCAAGCATGGTAAAGACGTGCACTTTCTGCAATCTATGAGA\n")
        f.write("CTTAGATCCACCATCTTAGAGAACAAGGCATCCCCTATCCAACGTGGGAATTCTGGACCA\n")
        f.write("CCATACAACTGAATGCAGAGTTTGTTCAGATTCAAACAAGGTTGTAGAGAGTCAAGGACA\n")
        f.write("TCCATTTGATTCCTTTCATTCCCTGAACCATCCAGCTCAGAACTCCACTGCATTATCAAG\n")
        f.write("CTTTCAAGGTTACGCTTCAATTTTAAATCGGCATCCCTTGCATCTTGAATATTCACCACA\n")
        f.write("TTTTCCAATTTCGAAATGCAAAGTTCTCCTCGCAGATGTGACATGTCCTTCAACTCCTTG\n")
        f.write("ATTGTCAAACCGTTGTTTTTGTCCACAATGAAATTAGACAATATCCGCAAATCTTTTAGT\n")
        f.write("TTGCCAATTCGTATAGGCATTTCTTGTAATTTTATTGCACCAGCAACATCAAGATGTCGA\n")
        f.write("AGATTGATTAGGTTACCGATGCTAATAGGCAACCTGATAAGCTCTTCACAGCATGATAAT\n")
        f.write("TTCAATGTCTGAAGGTAAAAAAGATTACCAATTGAATCCGGTAACCATTTTATACTGGTG\n")
        f.write("TAGGACAAATTAAGGTATCTCAAATGTTTCAACTTACCAAATGAATCCGGTATCTCACTT\n")
        f.write("ATCATGTAATTAGCCAATGAGAGCACCCTTAAGTGCCCTAACCTTGGTATCAATTCTTCA\n")
        f.write("AGCACCTTATTACTTATGAAGAAAAGAAATCCCCTAGTTGGTTCATCAATCGGCAAAGCT\n")
        f.write("ATGAAAGTGCGCAAGCGCTCTTTCTTATCAAACCTTTCAAATTTTTTAAAGATATCAAAA\n")
        f.write("TGATGATGGATGAATGATGAATGACGAGTATTTTCAGAAACGGGGCATTGCAAATCATTC\n")
        f.write("CACAACTCATCATCCAAATGCAAGCATGTGTCTCCAGCAATAGAATTAGCTAGATCGTTA\n")
        f.write("ATAAGATCATGCATAACGAATCGTGATTTATTGCTGCTCGACGATTGGAAAAATGACCTT\n")
        f.write("GACAATAGTTCACAAAAATAATCATCACCAAGATCTTCCATTTTCTCATCCTCATTTGAT\n")
        f.write("TGTTGAATTAACCCCTCCGCCATCCACAAGAGGATTAGCTCTTCCTTCTTAAACTCATAA\n")
        f.write("TCTTGGGGAAATAATGCGCAATAAGCGAAACACCTTTTTAAATGTGAAGGGAGATCATTA\n")
        f.write("TAGCTCAATCTCAACGCAGGAAGAATACCACACTTATCACCAGGTAAATTCCATATTTTG\n")
        f.write("CTAGCTAGTATAATATTCCACTTATCTTCTCTGTGTTCATGGCGCAGAAGACCACCAAGT\n")
        f.write("GCTTTTGCTGCCAAGGGCAGGCCTCCACACTTCTTTACAATTTCCCTTCCTATCAATGCC\n")
        f.write("AAGTCCGGATGCTCCTTGGTATTTCTGTTTTCAAAAGCATGTTTTTTAAACAACTCCCAA\n")
        f.write("CAATCATTGTCAGACAAATGTTTTAGCTCATAAAAGTTTTTATCTCCTCCCATCATAGTT\n")
        f.write("GCAACATTTTTATTACGAGTGGTGACGAGGATCTTGCTTCCCGGTGCTCCCTCCAAGAGA\n")
        f.write("GGGGAGCGTAGGCTGTCCCATTGATCATATTTCTCATTCCACAAATCGTCAAGAACAATC\n")
        f.write("AAAAATCTTTTTCCCATGGTTTCGTCCCTCAGTTTGCGTTGAATTTGATGAAAGTCCGGC\n")
        f.write("GAATCATTGTTGCCTGGAGCAATGTCCCGTAAAACTGCCCTGGTTATGGTCTCCACATGA\n")
        f.write("AATTGATCAGACACACAAACCCAAGCTTTTAAAGCAAAATGTTTGGATGTCTCCTCATCA\n")
        f.write("TCGTACACTAGACCTGCCAGCGTAGTTTTTCCCATCCCACCCATGGCCACGATCGAGACT\n")
        f.write("ACAGACAGATTGCCACCGAGGGATTCATCATTAAGCATGGCTAGTATCTTGGTTTTATCT\n")
        f.write("TCATCCCTGCCGTATACCCCAGGTTTGAATACCAAAGGTGGTGGAGGGGTTGGGCTTTGA\n")
        f.write("GTAGCTGCTCTGGCGCCTCCGATCTGCACTTTGAGCTTTTCCAAACCGAGCTCAGCCTTC\n")
        f.write("TGAGCTGAAATTTCTTCTAATCGGCGAGTGATGTCCTCTATCTTGGACCCCAGCTTAACA\n")
        f.write("TTCCGCATAGCCTGAATTGGAGTGAAAGTAGTACAACAAGTAGGGATGAATTTGCGTACC\n")
        f.write("TTGCTTGTGCTGGCCGCATCACCCTCCGCCACCAACTTTCTTCTCATCACTTGGTAGCCA\n")
        f.write("AACTCGTCCAAGACGTCCTCCACATCATAAGCCAAATCCCTGAGATGTGCGAGCCATGCT\n")
        f.write("TTCACGTGCTGCTTAGTGATCTGCTTGTCCTCAGCATCGTCGAGCACTTCCCGAATCTCC\n")
        f.write("AACAGTCTTGTCTTCCATTTCTTGAGCTCGGTGTGGACTTGCTCTTGGCGTGCGTACTTC\n")
        f.write("CACAAATCAGAGGAGGCCAGTTTGCTGAACAAGAGTTCAAGGGACACAGAAAGAACAACC\n")
        f.write("TCTCCAACAACTTCCATGGTGATCAGTTTGCAGAGAGAGAGAGAAAGGGGAAGAGGAAGA\n")
        f.write("TGAAAATGGAAGCCTGAGAGCAGGCTTAGTAAGCCATGTGCACCACACAAAATAAACAAA\n")
        f.write("TCAAAATCTCCACTATATTATTTTGCAGACAGCTGTCACATATTTTAATAGATTTAGTCT\n")
        f.write("AACCGAGAGAAGGCAATTTATTAAATTATTGGAGGTGCACGCACATGAAACATGAGTCAT\n")
        f.write("GAACATTGTTACAATTATATTTTTCAAAA\n")
        f.close()
        
    def _writeExpRealFastaFile(self, fileName):
        f = open(fileName, "w")
        f.write(">RXX_Vigne_1_3_12_1-B-R21164-Map3_classI-LTR-comp\n")
        f.write("ACGCCAAATGGAAAATGCTAATGGCTCTGAACGCCGAACCGAGAAGAGATAATGGCTCTG\n")
        f.write("AATACCAAATGGGAAATGTTCAGGCTCTAAACACCGAGACTGAGAAGAAATAGTGGCTCT\n")
        f.write("GAACGCCAAATGGAAAATGTTGATGGCTCTGAACGCCAAGACTGAGAAGAAATAGTGGAT\n")
        f.write("CTGAAAGCCAAAAGGAACATGCTGATGGCCCTGAACGCCGAGACTGAGAAGAAATAGTGG\n")
        f.write("CTCTGAATGCCAAGTGAAAAATGCTGATGGCTCTGAATGCCAAGACTGAGAAGAAATAGT\n")
        f.write("GGATCTAAACGCCAAATGGAAAATGATGATGGCTCTGAACGCCAAACTAAGAAGAAATAG\n")
        f.write("TGGCTCTGAACGCCAAATGCGAAATGCTGATGGCTCTGAACACCGATCTAAGAAGAAATA\n")
        f.write("ATGGCTCTGAACGCCAAATGGAAAATACTGATGGCCCTGAACGTCGAGACTAAGAAGAAA\n")
        f.write("TAGTGGCTCTGAACGCCAAATGGAAAATGCTGATGGCCCTAAACGCCAATCTGATAAGAA\n")
        f.write("ATAATGGCTCTAAACGCCAAATGGAAAATGCTGATGGCCCTGAACACCAAGACTAAAAAG\n")
        f.write("AAATAGTGGCTCTGAACGCCAAATGAAAAATTCTGATGGCCCTGAACGCCAAGACTGAGA\n")
        f.write("AGAAATAATGGCTCTGAACGCCAAATGAAAAATGTTGATGGCTATGAATGCCAAACTGAG\n")
        f.write("AAGAAATAGTGGCTCTGAACGCCAAATGAAAAATG\n")
        f.write(">RIX-incomp-chim_Vigne_1_3_12_1-B-R1512-Map7_classI-LINE-comp\n")
        f.write("TTTTTTTTTTGATAAGTAAGCACAAAATTCAATAAAAAGAGGCAAAATAGCCACAAAGTA\n")
        f.write("TACAGGGGGTATACAAGGCTACTAAGGCCTCACAAGCAAAAAACCAAAAAAACCCACCAC\n")
        f.write("CCCTTATCTGGAGGCTAACCACTCCAGAAAGCCTATGAGGGAAGGCGGCTCCTCTCCAAT\n")
        f.write("ATACAATTTAGCCCAACCCCATATATTACAAACAAAAGAATATTTAAGTCTCTGAATTGC\n")
        f.write("TAACCCCCCCCCCCCCCCCTTAAAAGCTAGTCTATTCCTCTCCTTCCAAACCGTCCAAAA\n")
        f.write("AATAAACAACGGAATGGAATTCCAAAGTTTTTTCCTTTTTTTCCCCACAAAGGATCCCCT\n")
        f.write("CCAACTAAATAAAACCTCCTTTACAGTTTCTGGAAACACCCACTGAACCCCAAACAATCC\n")
        f.write("CAAAACTAAATCCCACAGCACTCTGGCCACTGTACAATGTATGAGAATATGATTTACATT\n")
        f.write("TTCCTCTTCACAACCACACAAAAAACAACGATTGGGAAGCTGCCACCCTCTCTTCTGGAG\n")
        f.write("TCTATCAAGAGTCATAATCTTCCCCCAAGCGGCTTCCCACGCGAAGAACACAATTTTAGT\n")
        f.write("TGGAACTCTCTCCACCCATATGTTATTTTTTGGAAAAACGGCCGTCAAAGGGCTTTCCAA\n")
        f.write("CAGGCTGTATGCTTTCTTTACTCTAAACTGCCCGTTTCTTCCTTCCTTCCAAAAAACCGA\n")
        f.write("ATCTTCCTCCAAAGTTATCCTGTGACCCCTCAAAACATGAAGCAAGTTGCCTACCATATC\n")
        f.write("CAATTCCCAATCGTTAAAATCCCTAATAAACCTCAAATTCCATCCTCCTTCGCCAAAATT\n")
        f.write("CTGGTCCCACATTTCTTCTATTGTCGCATTCCTATGAGCAGCCAAAGTGAAGAGATGCGA\n")
        f.write("GAATCTTTAGGACAACACTGTACCCCCACACCAAAGATCAGTCCAAAACCTGATTTTAGT\n")
        f.write("GCCTTTTCCCACTTTAAACGCCAATTTTTCCCAACACCATCCAACTTCCTTCAAAATCTC\n")
        f.write("CTTCCAAACCCCCACTCCAAACGCACCAAAAGCCTTTTTTGTCCTCCAACCAAAATCCTC\n")
        f.write("TTGCCCATACTTCGCCATGAGCACTTGCTTCCAAAGATCCTCCTTAGCACAAGCAAATCT\n")
        f.write("CCAAATCCATTTGCCAAGCAAGGATTTGTTCAAGAGGGTTAGCTTCCTTATGCCAAGCCC\n")
        f.write("CCCTTTCTCCTTATCCACACAAACCGCCTCCCAATTGACAAGGTGAGCTTTCCTTTCCAG\n")
        f.write("ATTTCCTCCTCCCCACAAAAAATCTCTTTGCAGTTTTTCTAACCTCCTTGCCACAGACTT\n")
        f.write("GGGCATGCTGAAAAAGGACATTTGATATACTGGCATGCTAGCCATCGTGCTCTTAATAAG\n")
        f.write("AGTGATTCTCCCACCTTTTGAAATATATTGGCGTTTCCAAAGGACAAGTCTCCTCCTCAC\n")
        f.write("TCTCTCTTCCACCCCATCCCACAAGGAAGCGGCCTTATTTGGCGCCCCCAGTGGCAACCC\n")
        f.write("CAAGTAGACTGCAGGCAGCTGCCCCACCCTAGCTCCACTGCCATTTCGTCCACCTCTCCA\n")
        f.write("ACCAAGATGATTTCACTTTTAGCCAAATTAATTCTTAACCCTGAAGCAGCTTCAAACCAA\n")
        f.write("AAAAGAATCCAACTCAAATGCGTTAAATGCTCATTCTTTACATCACAAAAAACGATTGTG\n")
        f.write("TCATCAGCAAAGAGCAACTGTGAAATGTGGACAGCCCACCCTCTACCACGCCGAATGTTA\n")
        f.write("CACCCCGAAATAAAACCCCCTTCCACAGCCCTTCTGATTAAAACGCTCAACACTTCCATT\n")
        f.write("CCCATGACAAAAAGATAGAGGGAAAGAGGGTCCCCTTGGCGCAGCCCCTTCGAAATTGAG\n")
        f.write("AAGAAACCAACTGGAACTCCGTTCACCAAGACTAAGAACTTGGCTATGGAAATGCAACAC\n")
        f.write("CACATCCAACTCAACCACTTTGAACCAAAC\n")
        f.write(">SSR_Vigne_1_3_12_1-B-R21665-Map6_classI-LTR-incomp\n")
        f.write("CACCTTAGGCCAACTTAGGCACCTTAGGCCCACTTAGGCACCTTAGGCCCACTTAGGCAC\n")
        f.write("CTTAGGCTCACTTAGGCACCTTAGGCCCACTTAGGCAACTTAGGCCCACTTAGGCACCAC\n")
        f.write("TAGGCCCACTTAGGCACCCTAGGCCCACTTAGGCACCCTAGGCCCACTTAGGTACCCTAG\n")
        f.write("GCCCACTTAGGTACCCTAGGCCCATTTAGGCACCCTAGGCCCACTTAGGCACCCTAAGCC\n")
        f.write("CACTTAGGCACCCTAGGACCACTTAGGCACCCTAGGCTCATTTAGGCACCTTAGGCCCAA\n")
        f.write("TTAGGCACA\n")
        f.write(">DHX-comp_Vigne_1_3_12_1-B-R13403-Map11_NoCat\n")
        f.write("GAAAGAAAAGTTTAAGCACAAAGGAATCACCTGACTGTGTATTGGCAGAGTGGCGGAGTG\n")
        f.write("GCACTTTGATTAAAATAGCACTTCATTGATCACTATTTTAGGAATGAGATCTATATAACA\n")
        f.write("CCATTCTTCCCCTTTCTCAAATTGACACCGTTTTTCCAGTAAATGACCTCATTATAACTA\n")
        f.write("CAATAAGTGAAGTGATAATGTCATCTAATTAGTAAACCAAACCAAGTCAATAAACTAGAA\n")
        f.write("CTTCCTCTAGTATAGCCTCTGTGTAAAAGATATTTAAAGGTCTTGATACATTAGATATTG\n")
        f.write("AATTTGTTGGATTAAAATCTCATTTATTGCACCTGGCAAGGTAGAAGTAGTAATGCTCAT\n")
        f.write("TTTTTATTGTGACAGATTATAGAGAAAATTAGACACTGGGAAAATACCAAGCCAGGAGCC\n")
        f.write("ATGTCAAGAGCTGCTTTCTTGTTAAAAATTAAGTCCAATTAGAAAATTACAGTAGCGCGT\n")
        f.write("AAGAGAAATTGAAAGTGGTGTTGAGAAGAGAAAGAGGAAGAATGATGTGTTGGGAATGAG\n")
        f.write("ATGTAATACCTTATTTATTACTCAGATGTGGATTAGTCATTGTATCTCCACACAGGGGAT\n")
        f.write("GTGGGCAATCTTGGGCCAATCATCTCCTTCCTCCTTCGAGTACCTTTGTGTTAGATGTGG\n")
        f.write("GCAACGCCTCACATACAGTCGTGAAAGGGTGTCAGGCAATAGTCCTTCCCTTGGCAAAAT\n")
        f.write("CGACCGGAGCTTAGGGCAACTTTCGATTTCTAGATATTCAAGAGAGGTGAGGGTTTGGAG\n")
        f.write("AGACAGGGAGGCTAGGGATTCCAGATTCTGGAAATGTGAAAGGGTAAGGGAGGTGAGAGT\n")
        f.write("TGTAGGAAAAAGAATCGAGTGGGGGTCATCCGAAAAGGAAGTTGCATCTGGAAACATGCC\n")
        f.write("ACCAATCCAGAGGTCTTTAAGAGAGGTGAGTCTGGAAAGGCCCCATTGGGATAGGGGGGT\n")
        f.write("CTTGATATTCTCACAATTACGAATGAAAAGCGATGTAAGACGAGTGAGGTTTTTTATCTG\n")
        f.write("AGGAAGCAACAATTCCAGATTTTCAAAATCTACAATACGTAGATCTGTGAGGGTGTTGAG\n")
        f.write("GCAATCTGGTAGGGTTTTGAGATTAGGATACCTCCTGAGGGTTAAAGATTGAAGTGAATT\n")
        f.write("ATTAGTAGAGTGAAACATCTCCTCTGAAATTGACTCGAGGTGTTCACAATCCCCAATGTG\n")
        f.write("AAGTCGCTCAAGGGTGGATGGAAACTTGCCTCTTGGGAAGGATGTGAGAGATGGACATTT\n")
        f.write("ACAGATTTCCAAGGACTTGGAGAGCAGCAGCATTGGTGGAATGTTGGTGCATTATTCCCT\n")
        f.write("CTGGTAGAGACTCTAGCCTTCTACAATCACCTATTCTGAGTCTCTTGAGAGTGGCGGGTA\n")
        f.write("ACCCACCTTTCGGTAATCCAATAAGAGAATGACACCTATCTATGAATAAGCCTTCAAGGG\n")
        f.write("CACACATGCCCATCATTCCTTCTGGAAGTGACTTGAGATTTTCACAAGACGATATGCTTA\n")
        f.write("GACTCTTAAGGGTGGTGGGTAATTGCCCTTTTGGAAAGCAAATGAGAGATGGACACGCTG\n")
        f.write("TATATCACTAGCTCTTCAAGGAGACACAAGTTGTTGCTGTCAGTACTGTCGTTCCTCATC\n")
        f.write("TTCAACATCATTTCATCAGGTAGACACTCAAGACCTTCACAATTATCAAGAATAAGATTT\n")
        f.write("CTCAGCATTGGTGGGAAACCTACATCTGGAAATGACGCTAGCTTTGGACAATTCCTGATT\n")
        f.write("GTCAATTCTTCAAGGCATGTTAGACTCTGCCATCCATTTGGAAGCCTCTCTAGTTTATCA\n")
        f.write("CATTTAATTATTTCCAAAGACTGAAGATTGCATCCCAAGGATACAAGTTGATCACAATCT\n")
        f.write("CTAATCTCAAGAGAATGAGAGTTTTCCGATCCAAAACCATCCTCCCACAAATATTCGAGT\n")
        f.write("TCTTCGCATTCCCATACTTTCAAAACCCGAAGCCCTTGCAAAAATTGCACAAATCCTTCA\n")
        f.write("TGCAATTTGATAAGCCCTGAAATCCCAGAAATTGTTAACTTGGTGAGTGAGGTGAGGTCA\n")
        f.write("TTTCCACTGCTCAAGACCGCCTCATTACATCCTCTTACTTGTAATTCCTTCAGCAGTGGA\n")
        f.write("AGTCTTGAAAGTGGAGACTCCAATTTTGGACAAAAGTGAACAGAGAGCTTTGTAAGAGAA\n")
        f.write("GGTAGGTAAGTGGGTAGTTTCATAATCAATTTGGGGCAATCCTCAATTGTAAGCTCATGG\n")
        f.write("AGACAAGGGAATAATGACTCTGTTGAGGAAGACCAATCCTCCCAGTGCTCCCATTCTGAC\n")
        f.write("ATGCTATTAAAATGTAGAGACTCTAGTGATGGGAAAAACTTACCTGCAGAAACACGAGTC\n")
        f.write("TCTCCATAAAACTCTGCACCCACTTTTTTTACTCCATCCATTCCTTGGATCCTCAACTGT\n")
        f.write("TTGAGCGATGGCAACTGCCCCAAGCATGGTAAAGACGTGCACTTTCTGCAATCTATGAGA\n")
        f.write("CTTAGATCCACCATCTTAGAGAACAAGGCATCCCCTATCCAACGTGGGAATTCTGGACCA\n")
        f.write("CCATACAACTGAATGCAGAGTTTGTTCAGATTCAAACAAGGTTGTAGAGAGTCAAGGACA\n")
        f.write("TCCATTTGATTCCTTTCATTCCCTGAACCATCCAGCTCAGAACTCCACTGCATTATCAAG\n")
        f.write("CTTTCAAGGTTACGCTTCAATTTTAAATCGGCATCCCTTGCATCTTGAATATTCACCACA\n")
        f.write("TTTTCCAATTTCGAAATGCAAAGTTCTCCTCGCAGATGTGACATGTCCTTCAACTCCTTG\n")
        f.write("ATTGTCAAACCGTTGTTTTTGTCCACAATGAAATTAGACAATATCCGCAAATCTTTTAGT\n")
        f.write("TTGCCAATTCGTATAGGCATTTCTTGTAATTTTATTGCACCAGCAACATCAAGATGTCGA\n")
        f.write("AGATTGATTAGGTTACCGATGCTAATAGGCAACCTGATAAGCTCTTCACAGCATGATAAT\n")
        f.write("TTCAATGTCTGAAGGTAAAAAAGATTACCAATTGAATCCGGTAACCATTTTATACTGGTG\n")
        f.write("TAGGACAAATTAAGGTATCTCAAATGTTTCAACTTACCAAATGAATCCGGTATCTCACTT\n")
        f.write("ATCATGTAATTAGCCAATGAGAGCACCCTTAAGTGCCCTAACCTTGGTATCAATTCTTCA\n")
        f.write("AGCACCTTATTACTTATGAAGAAAAGAAATCCCCTAGTTGGTTCATCAATCGGCAAAGCT\n")
        f.write("ATGAAAGTGCGCAAGCGCTCTTTCTTATCAAACCTTTCAAATTTTTTAAAGATATCAAAA\n")
        f.write("TGATGATGGATGAATGATGAATGACGAGTATTTTCAGAAACGGGGCATTGCAAATCATTC\n")
        f.write("CACAACTCATCATCCAAATGCAAGCATGTGTCTCCAGCAATAGAATTAGCTAGATCGTTA\n")
        f.write("ATAAGATCATGCATAACGAATCGTGATTTATTGCTGCTCGACGATTGGAAAAATGACCTT\n")
        f.write("GACAATAGTTCACAAAAATAATCATCACCAAGATCTTCCATTTTCTCATCCTCATTTGAT\n")
        f.write("TGTTGAATTAACCCCTCCGCCATCCACAAGAGGATTAGCTCTTCCTTCTTAAACTCATAA\n")
        f.write("TCTTGGGGAAATAATGCGCAATAAGCGAAACACCTTTTTAAATGTGAAGGGAGATCATTA\n")
        f.write("TAGCTCAATCTCAACGCAGGAAGAATACCACACTTATCACCAGGTAAATTCCATATTTTG\n")
        f.write("CTAGCTAGTATAATATTCCACTTATCTTCTCTGTGTTCATGGCGCAGAAGACCACCAAGT\n")
        f.write("GCTTTTGCTGCCAAGGGCAGGCCTCCACACTTCTTTACAATTTCCCTTCCTATCAATGCC\n")
        f.write("AAGTCCGGATGCTCCTTGGTATTTCTGTTTTCAAAAGCATGTTTTTTAAACAACTCCCAA\n")
        f.write("CAATCATTGTCAGACAAATGTTTTAGCTCATAAAAGTTTTTATCTCCTCCCATCATAGTT\n")
        f.write("GCAACATTTTTATTACGAGTGGTGACGAGGATCTTGCTTCCCGGTGCTCCCTCCAAGAGA\n")
        f.write("GGGGAGCGTAGGCTGTCCCATTGATCATATTTCTCATTCCACAAATCGTCAAGAACAATC\n")
        f.write("AAAAATCTTTTTCCCATGGTTTCGTCCCTCAGTTTGCGTTGAATTTGATGAAAGTCCGGC\n")
        f.write("GAATCATTGTTGCCTGGAGCAATGTCCCGTAAAACTGCCCTGGTTATGGTCTCCACATGA\n")
        f.write("AATTGATCAGACACACAAACCCAAGCTTTTAAAGCAAAATGTTTGGATGTCTCCTCATCA\n")
        f.write("TCGTACACTAGACCTGCCAGCGTAGTTTTTCCCATCCCACCCATGGCCACGATCGAGACT\n")
        f.write("ACAGACAGATTGCCACCGAGGGATTCATCATTAAGCATGGCTAGTATCTTGGTTTTATCT\n")
        f.write("TCATCCCTGCCGTATACCCCAGGTTTGAATACCAAAGGTGGTGGAGGGGTTGGGCTTTGA\n")
        f.write("GTAGCTGCTCTGGCGCCTCCGATCTGCACTTTGAGCTTTTCCAAACCGAGCTCAGCCTTC\n")
        f.write("TGAGCTGAAATTTCTTCTAATCGGCGAGTGATGTCCTCTATCTTGGACCCCAGCTTAACA\n")
        f.write("TTCCGCATAGCCTGAATTGGAGTGAAAGTAGTACAACAAGTAGGGATGAATTTGCGTACC\n")
        f.write("TTGCTTGTGCTGGCCGCATCACCCTCCGCCACCAACTTTCTTCTCATCACTTGGTAGCCA\n")
        f.write("AACTCGTCCAAGACGTCCTCCACATCATAAGCCAAATCCCTGAGATGTGCGAGCCATGCT\n")
        f.write("TTCACGTGCTGCTTAGTGATCTGCTTGTCCTCAGCATCGTCGAGCACTTCCCGAATCTCC\n")
        f.write("AACAGTCTTGTCTTCCATTTCTTGAGCTCGGTGTGGACTTGCTCTTGGCGTGCGTACTTC\n")
        f.write("CACAAATCAGAGGAGGCCAGTTTGCTGAACAAGAGTTCAAGGGACACAGAAAGAACAACC\n")
        f.write("TCTCCAACAACTTCCATGGTGATCAGTTTGCAGAGAGAGAGAGAAAGGGGAAGAGGAAGA\n")
        f.write("TGAAAATGGAAGCCTGAGAGCAGGCTTAGTAAGCCATGTGCACCACACAAAATAAACAAA\n")
        f.write("TCAAAATCTCCACTATATTATTTTGCAGACAGCTGTCACATATTTTAATAGATTTAGTCT\n")
        f.write("AACCGAGAGAAGGCAATTTATTAAATTATTGGAGGTGCACGCACATGAAACATGAGTCAT\n")
        f.write("GAACATTGTTACAATTATATTTTTCAAAA\n")
        f.close()

    def _writeFastaFileWithShortenOptionRSHeaders(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">Vigne_1_3_12_1_RS_1_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_2\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_3\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_4\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_5_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">Vigne_1_3_12_1_RS_15\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        #Note: the next consensus is reversed only in classif file: this tests if it's correctly renamed in resulting fasta file
        f.write(">Vigne_1_3_12_1_RS_16\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.close()
        
    def _writeClassifFileWithShortenOptionRSHeaders(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("Vigne_1_3_12_1_RS_1_reversed\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_2\t2154\t+\tok\tII\tTIR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_3\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_4\t2154\t+\tok\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        #Note: the next consensus is reversed only in fasta file: this tests if it's correctly renamed in resulting classif file
        f.write("Vigne_1_3_12_1_RS_5\t2154\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_15\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("Vigne_1_3_12_1_RS_16_reversed\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.close()
        
    def _writeExpFileWithShortenOptionRSHeaders(self, inputFileName):
        f = open(inputFileName, "w")
        f.write(">RLX-comp_Vigne_1_3_12_1_RS_1_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">DTX-incomp_Vigne_1_3_12_1_RS_2\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1_RS_3\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RIX-incomp_Vigne_1_3_12_1_RS_4\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RSX-comp_Vigne_1_3_12_1_RS_5_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1_RS_15\n")
        f.write("ATCGATCGATGCATGCATCG\n")
        f.write(">RLX-comp_Vigne_1_3_12_1_RS_16_reversed\n")
        f.write("ATCGATCGATGCATGCATCG\n")  
        f.close()
    
    def _writeExpClassifFileWithShortenOptionRSHeaders(self, inputFileName):
        f = open(inputFileName, "w")
        f.write("RLX-comp_Vigne_1_3_12_1_RS_1_reversed\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("DTX-incomp_Vigne_1_3_12_1_RS_2\t2154\t+\tok\tII\tTIR\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1_RS_3\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RIX-incomp_Vigne_1_3_12_1_RS_4\t2154\t+\tok\tI\tLINE\tincomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RSX-comp_Vigne_1_3_12_1_RS_5_reversed\t2154\t+\tok\tI\tSINE\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1_RS_15\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.write("RLX-comp_Vigne_1_3_12_1_RS_16_reversed\t2154\t+\tok\tI\tLTR\tcomplete\tCI=50; coding=(); struct=(); other=()\n")
        f.close()

if __name__ == "__main__":
    unittest.main()