import os
import unittest
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils
from commons.tools.SplicerFromAnnotation import SplicerFromAnnotation

class Test_F_SplicerFromAnnotation(unittest.TestCase):

    def setUp(self):
        self.inFastaFileName = "%s/Tools/Splicer/Splicer.fa" % os.environ["REPET_DATA"]
        self.configFileName = "TEdenovo.cfg"
        self._writeConfigFile()
        iDb = DbFactory.createInstance(self.configFileName)
        pathFileName = "%s/Tools/Splicer/Splicer_chr_allTEs_nr_noSSR_join_path.path" % os.environ["REPET_DATA"]
        self.tableName_path = "Splicer_chr_allTEs_nr_noSSR_join_path"
        iDb.createTable(self.tableName_path, "path", pathFileName, True)
        refTEseqFileName = "%s/Tools/Splicer/Splicer_refTEs_seq.fa" % os.environ["REPET_DATA"]
        self.tableName_refTEseq = "Splicer_refTEs_seq"
        iDb.createTable(self.tableName_refTEseq, "seq", refTEseqFileName, True)
        self.obsOutSplicedFastaFileName = "Splicer.fa.splice"
        iDb.close()
   
    def tearDown(self):
        iDb = DbFactory.createInstance(self.configFileName)
        iDb.dropTable(self.tableName_path)
        iDb.dropTable(self.tableName_refTEseq)
        iDb.dropTable("Splicer_annotationIdentitySup80_path")
        iDb.dropTable("Splicer_annotationToSplice_path")
        iDb.close()
        os.remove(self.configFileName)
        FileUtils.removeFilesByPattern("Splicer_chr_allTEs_nr_noSSR_join_path.*")
        try:
            os.remove("Splicer_annotationToSplice_path.path")
            os.remove(self.obsOutSplicedFastaFileName)
        except:
            pass
        
    def test_run_as_script_FLC(self):
        expOutSplicedFastaFileName = "%s/Tools/Splicer/exp_SplicerFLC.fa.splice" % os.environ["REPET_DATA"]
        cmd = 'SplicerFromAnnotation.py -i %s -C %s -o %s -v 3' % (self.inFastaFileName, self.configFileName, self.obsOutSplicedFastaFileName)
        os.system(cmd)
           
        self.assertTrue(FileUtils.are2FilesIdentical(expOutSplicedFastaFileName, self.obsOutSplicedFastaFileName))
           
    def test_run_as_script_FLF(self):
        expOutSplicedFastaFileName = "%s/Tools/Splicer/exp_SplicerFLF.fa.splice" % os.environ["REPET_DATA"]
        cmd = 'SplicerFromAnnotation.py -i %s -C %s -t 2 -o %s -v 3' % (self.inFastaFileName, self.configFileName, self.obsOutSplicedFastaFileName)
        os.system(cmd)
    
        self.assertTrue(FileUtils.are2FilesIdentical(expOutSplicedFastaFileName, self.obsOutSplicedFastaFileName))
          
    def test_run_FLC(self):
        expOutSplicedFastaFileName = "%s/Tools/Splicer/exp_SplicerFLC.fa.splice" % os.environ["REPET_DATA"]
  
        iSFA = SplicerFromAnnotation()
        iSFA.setInputFileName(self.inFastaFileName)
        iSFA.setConfigFileName(self.configFileName)
        iSFA.setOutputFileName(self.obsOutSplicedFastaFileName)
        iSFA.run()
  
        self.assertTrue(FileUtils.are2FilesIdentical(expOutSplicedFastaFileName, self.obsOutSplicedFastaFileName))
         
    def test_run_FLF(self):
        expOutSplicedFastaFileName = "%s/Tools/Splicer/exp_SplicerFLF.fa.splice" % os.environ["REPET_DATA"]
  
        iSFA = SplicerFromAnnotation(self.inFastaFileName, self.configFileName, copyType = 2, outputFile = self.obsOutSplicedFastaFileName)
        iSFA.run()
  
        self.assertTrue(FileUtils.are2FilesIdentical(expOutSplicedFastaFileName, self.obsOutSplicedFastaFileName))

    def _writeConfigFile(self):
        f = open(self.configFileName, "w")
        f.write("[repet_env]\n")
        f.write("repet_host: %s\n" % os.environ["REPET_HOST"])
        f.write("repet_user: %s\n" % os.environ["REPET_USER"])
        f.write("repet_pw: %s\n" % os.environ["REPET_PW"])
        f.write("repet_db: %s\n" % os.environ["REPET_DB"])
        f.write("repet_port: %s\n" % os.environ["REPET_PORT"])
        f.close()

if __name__ == "__main__":
    unittest.main()