import os
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.core.seq.test.Utils_for_T_FastaUtils import Utils_for_T_FastaUtils

class Test_F_dbChunks(unittest.TestCase):

    def setUp(self):
        self.inFileName = "dummyBigSeqFastaFile.fa"
        self.expChunksFileName = 'exp' + self.inFileName +'_chunks.fa'
        self.expChunksMapFileName = 'exp' + self.inFileName +'_chunks.map'
        self.expCutFileName = 'exp' + self.inFileName +'_cut'
        self.expNStretchFileName = 'exp' + self.inFileName +'.Nstretch.map'
        self.expNStretchStats = 'exp' + self.inFileName +'.NstretchStats.txt'
        Utils_for_T_FastaUtils._createFastaFile_big_sequence(self.inFileName)
        Utils_for_T_FastaUtils._createFastaFile_of_Chunks(self.expChunksFileName)
        Utils_for_T_FastaUtils._createMapFile_of_Chunks(self.expChunksMapFileName)
        Utils_for_T_FastaUtils._createFastaFile_of_cut(self.expCutFileName)
        Utils_for_T_FastaUtils._createFastaFile_of_Nstretch(self.expNStretchFileName)
   
    def tearDown(self):
        FileUtils.removeFilesByPattern("*.fa")
        FileUtils.removeFilesByPattern("*.map")
        FileUtils.removeFilesByPattern("*_cut")
        FileUtils.removeFilesByPattern("*.txt")       
        
    def test_run_as_script(self):
        f = open( self.expNStretchStats, "w" )
        f.write("sequence\tmin: 13\tmax: 120\tcumul: 184\n")
        f.write("sequence2\tmin: 14\tmax: 32\tcumul: 65\n")
        f.write("Total N in stretch length > 11: 249 bp represent  15.89 % of genome\n")
        f.close()
        
        cmd = "dbChunks.py -i %s -l 60 -w 11 -o 10" % (self.inFileName)
        os.system(cmd)
        
        obsChunksFileName = self.inFileName +'_chunks.fa'
        obsChunksMapFileName = self.inFileName +'_chunks.map'
        obsCutFileName = self.inFileName +'_cut'
        obsNStretchFileName = self.inFileName +'.Nstretch.map'
        obsNStretchStats = self.inFileName + '.NstretchStats.txt'
        
        self.assertTrue(FileUtils.are2FilesIdentical(self.expChunksFileName, obsChunksFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self.expChunksMapFileName, obsChunksMapFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self.expCutFileName, obsCutFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self.expNStretchFileName, obsNStretchFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self.expNStretchStats, obsNStretchStats))

    def test_run_as_script_with_empty_NStretch_File(self):
        
        f = open( self.expNStretchStats, "w" )
        f.write("No N in stretch length > 120\n")
        f.close()
 
        Utils_for_T_FastaUtils._createFastaFile_big_sequence(self.inFileName)
        Utils_for_T_FastaUtils._createFastaFile_of_Nstretch(self.expNStretchFileName)

        cmd = "dbChunks.py -i %s -l 125 -w 120 -o 10" % (self.inFileName)
        os.system(cmd)
        
        obsNStretchFileName = self.inFileName +'.Nstretch.map'
        obsNStretchStats = self.inFileName + '.NstretchStats.txt'

        self.assertTrue(FileUtils.are2FilesIdentical(self.expNStretchStats, obsNStretchStats))
        self.assertTrue(FileUtils.isEmpty(obsNStretchFileName))
                
if __name__ == "__main__":
    unittest.main()