import unittest
import os
from commons.core.utils.FileUtils import FileUtils


class Test_MaskSeqFromCoord( unittest.TestCase ):
    
    def test_zRunAsScript_hardMasking( self ):
        cDir = os.getcwd()
        
        inFaFile = "dummyInFile.fa"
        inFaFileHandler = open( inFaFile, "w" )
        inFaFileHandler.write( ">seq1\n" )
        inFaFileHandler.write( "AGCATGCAGCATACGACTACG\n" )
        inFaFileHandler.close()
        
        mapFile = "dummyCoordFile.map"
        mapFileHandler = open( mapFile, "w" )
        mapFileHandler.write( "TE1\tseq1\t2\t5\n" )
        mapFileHandler.close()
        
        expFile = "dummyExpFile.fa"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">seq1\n" )
        expFileHandler.write( "ANNNNGCAGCATACGACTACG\n" )
        expFileHandler.close()
        
        obsFile = "dummyObsFile.fa"
        
        cmd = "python ../MaskSeqFromCoord.py"
        cmd += " -i %s" % ( inFaFile )
        cmd += " -m %s" % ( mapFile )
        cmd += " -f %s" % ( "map" )
        cmd += " -o %s" % ( obsFile )
        cmd += " -v 0"
        os.system( cmd )
        
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        for f in [ inFaFile, mapFile, expFile, obsFile ]:
            os.remove( f )
        os.chdir( cDir )


    def test_zRunAsScript_softMasking( self ):
        cDir = os.getcwd()

        inFaFile = "dummyInFile.fa"
        inFaFileHandler = open( inFaFile, "w" )
        inFaFileHandler.write( ">seq1\n" )
        inFaFileHandler.write( "AGCATGCAGCATACGACTACG\n" )
        inFaFileHandler.close()
        
        mapFile = "dummyCoordFile.map"
        mapFileHandler = open( mapFile, "w" )
        mapFileHandler.write( "TE1\tseq1\t2\t5\n" )
        mapFileHandler.close()
        
        expFile = "dummyExpFile.fa"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">seq1\n" )
        expFileHandler.write( "AgcatGCAGCATACGACTACG\n" )
        expFileHandler.close()
        
        obsFile = "dummyObsFile.fa"
        
        cmd = "python ../MaskSeqFromCoord.py"
        cmd += " -i %s" % ( inFaFile )
        cmd += " -m %s" % ( mapFile )
        cmd += " -f %s" % ( "map" )
        cmd += " -s"
        cmd += " -o %s" % ( obsFile )
        cmd += " -v 0"
        os.system( cmd )
        
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        for f in [ inFaFile, mapFile, expFile, obsFile ]:
            os.remove( f )
        os.chdir( cDir )
        
if __name__ == "__main__":
        unittest.main()