import unittest
import time
import os
from commons.core.utils.FileUtils import FileUtils
from commons.tools.NoCatBestHitClassifier import NoCatBestHitClassifier
import shutil
from commons.core.coord.Path import Path
from commons.core.coord.Range import Range
from commons.core.coord.PathUtils import PathUtils

class Test_NoCatBestHitClassifier(unittest.TestCase):

    def setUp(self):
        self._uniqId = "test_U_NoCatBestHitClassifier%s_%s" % ( time.strftime("%Y%m%d%H%M%S") , os.getpid() )
        self._inFastaFileName = "%s/Tools/consensus_classif_noCatBestHit.fa" % os.environ["REPET_DATA"]
        self._inClassifFileName = "%s/Tools/consensus_classif_noCatBestHit.classif" % os.environ["REPET_DATA"]
        self.curDir = os.getcwd()
        os.makedirs(self._uniqId)
        os.chdir(self._uniqId)
        os.symlink(self._inFastaFileName, "consensus_classif_noCatBestHit.fa")
        os.symlink(self._inClassifFileName, "consensus_classif_noCatBestHit.classif")


    def tearDown(self):
        os.chdir(self.curDir)
        shutil.rmtree(self._uniqId)
        self._uniqId = None


    def test_analyzeInputClassifFile(self):
        iNoCatBestHits = NoCatBestHitClassifier(fastaFileName = "consensus_classif_noCatBestHit.fa", classifFileName="consensus_classif_noCatBestHit.classif")
        iNoCatBestHits._analyzeInputClassifFile()
        
        expNbNotCat = len(iNoCatBestHits._lClassifNoCat)        
        obsNbNotCat = 39
        self.assertEquals(expNbNotCat,obsNbNotCat)
        expNbChim = len(iNoCatBestHits._lClassifChim)
        obsNbChim = 308
        self.assertEquals(expNbChim,obsNbChim)
        expNbOk = len(iNoCatBestHits._lClassifOk)
        obsNbOk = 710
        self.assertEquals(expNbOk,obsNbOk)

    
    def test_sumScoreFromPathListOnSamePath(self):
        iNoCatBestHits = NoCatBestHitClassifier()
        iPath1 = Path( 1, Range("qry1",398,491), Range("sbj1",10,112), 0.0, 10, 98.7 )
        iPath2 = Path( 1, Range("qry1",451,492), Range("sbj1",124,169), 0.0, 10, 98.7 )
        iPath3 = Path( 1, Range("qry1",493,531), Range("sbj1",249,294), 0.0, 10, 98.7 )
        lPaths = [ iPath3, iPath2, iPath1 ]
        
        expPath = Path( 1, Range("qry1",398,531), Range("sbj1",10,294), 0.0, 30, 0.0 )       
        obsPath = iNoCatBestHits.sumScoreFromPathListOnSamePath( lPaths )
        
        self.assertEquals( expPath, obsPath )
 
    def test_sumScoreFromPathListOnSamePath_pathListWithSeveralPathId(self):
        iNoCatBestHits = NoCatBestHitClassifier()
        iPath1 = Path( 1, Range("qry1",398,491), Range("sbj1",10,112), 0.0, 10, 98.7 )
        iPath2 = Path( 1, Range("qry1",451,492), Range("sbj1",124,169), 0.0, 10, 98.7 )
        iPath3 = Path( 2, Range("qry2",493,531), Range("sbj1",249,294), 0.0, 10, 98.7 )
        lPaths = [ iPath3, iPath2, iPath1 ]
        
        expPath = Path()       
        obsPath = iNoCatBestHits.sumScoreFromPathListOnSamePath( lPaths )
        
        self.assertEquals( expPath, obsPath )
        
    def test_findBestPath_basedOnSubjectName(self):
        iNoCatBestHits = NoCatBestHitClassifier()
        iPath1 = Path( 1, Range("qry1",398,491), Range("sbj1",10,112), 0.0, 30, 98.7 )
        iPath2 = Path( 5, Range("qry1",451,492), Range("sbj1",124,169), 0.0, 10, 98.7 )
        iPath3 = Path( 2, Range("qry2",493,531), Range("sbj1",249,294), 0.0, 100, 98.7 )
        iPath4 = Path( 3, Range("qry2",493,531), Range("sbj2",249,294), 0.0, 110, 98.7 )
        iPath5 = Path( 4, Range("qry2",493,531), Range("sbj3",249,294), 0.0, 90, 98.7 )
        lPaths = [ iPath3, iPath2, iPath1, iPath4, iPath5 ]
        dPathIdToQueryLength = {"1" : 94, "2" : 39, "3" : 39, "4" : 39, "5" : 42}
        
        expLPath = [iPath1, iPath3]
        
        obsLPath = iNoCatBestHits.findBestPath_basedOnSubjectName(PathUtils.getDictOfListsWithQueryNameAsKey(lPaths), dPathIdToQueryLength)
        self.assertEquals( expLPath.sort(), obsLPath.sort() )

    def test_findBestPath_basedOnSubjectName_with_score_egality(self):
        iNoCatBestHits = NoCatBestHitClassifier()
        iPath1 = Path( 1, Range("qry1",398,491), Range("sbj1",10,112), 0.0, 30, 98.7 )
        iPath2 = Path( 2, Range("qry1",451,492), Range("sbj1",124,169), 0.0, 10, 98.7 )
        iPath3 = Path( 3, Range("qry2",493,531), Range("sbj1",249,294), 0.0, 100, 98.7 )
        iPath4 = Path( 4, Range("qry2",493,531), Range("sbj2",249,294), 0.0, 110, 98.7 )
        iPath5 = Path( 5, Range("qry2",480,531), Range("sbj3",249,294), 0.0, 110, 98.7 )
        iPath6 = Path( 6, Range("qry2",550,750), Range("sbj4",249,294), 0.0, 110, 98.7 )
        iPath7 = Path( 7, Range("qry1",500,600), Range("sbj1",124,169), 0.0, 10, 98.7 )
        iPath8 = Path( 8, Range("qry3",500,600), Range("sbj1",124,169), 0.0, 1000, 98.7 )
        lPaths = [ iPath1, iPath2, iPath7, iPath3, iPath4, iPath5, iPath6,iPath8 ]
        dPathIdToQueryLength = {1 : 94, 2 : 39, 3 : 42, 4 : 42, 5 : 22, 6 : 200, 7 : 101, 8 : 101}
        
        expLPath = [iPath8, iPath6, iPath1]
        obsLPath = iNoCatBestHits.findBestPath_basedOnSubjectName(PathUtils.getDictOfListsWithQueryNameAsKey(lPaths), dPathIdToQueryLength)
        self.assertItemsEqual(expLPath, obsLPath) 
        
        
if __name__ == "__main__":
    unittest.main()