#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
import re

from distutils.core import setup
from distutils.core import Command
from distutils.util import convert_path

class Devtools(object):

    @staticmethod
    def find_sub_packages(lrootPackage, lExclude=()):
        lPackages = Devtools.find_packages(exclude=lExclude)
        lSubPackage = []
        for package in lPackages:
                for rootPackage in lrootPackage:
                        if package.startswith(rootPackage):
                                lSubPackage.append(package)
        return lSubPackage

    @staticmethod
    def find_packages(where='.', exclude=()):
        out = []
        stack=[(convert_path(where), '')]
        while stack:
            where,prefix = stack.pop(0)
            for name in os.listdir(where):
                fn = os.path.join(where,name)
                if ('.' not in name and os.path.isdir(fn) and os.path.isfile(os.path.join(fn,'__init__.py'))):
                    out.append(prefix+name); stack.append((fn,prefix+name+'.'))
        for pat in list(exclude)+['ez_setup', 'distribute_setup']:
            from fnmatch import fnmatchcase
            out = [item for item in out if not fnmatchcase(item,pat)]
        return out

    @staticmethod
    def findall(directory = os.curdir):
        all_files = []
        for base, dirs, files in os.walk(directory):
            if base==os.curdir or base.startswith(os.curdir+os.sep):
                base = base[2:]
            if base:
                files = [os.path.join(base, f) for f in files]
            all_files.extend(filter(os.path.isfile, files))
        return all_files


class Install(Command):
    description = "Install MetaGPipe"
    user_options = []

    def initialize_options(self):
        """Use this to set option defaults before parsing."""
        pass

    def finalize_options(self):
        """Code to validate/modify command-line/config input goes here."""
        pass

    def _isToLink(self, fileName):
        if re.search("__\w*.py", fileName):
            return False
        elif re.search("Test\w*.py", fileName):
            return False
        elif re.search("\w*.py$", fileName):
            return True
        elif re.search("\w*.sh$", fileName):
            return True
        elif re.search("\w*.pl$", fileName):
            return True
        return False

    def run(self):
        cwd = os.getcwd()
        print "Build MetaGPipe in %s" % (cwd)
        if not os.path.isdir("bin"):
            os.mkdir("bin")
        os.chdir("bin")
        lPackages = ["Gnome_tools","commons/tools"]
        for package in lPackages:
            if os.path.isdir("../%s" % package):
                print "processing %s/..." % package
                [os.symlink(i, os.path.basename(i)) for i in Devtools.findall("../%s" % package) if self._isToLink(os.path.basename(i))]
        os.system("chmod 0755 *")
        print "MetaGPipe is ready to use"


setup(
    name="MetaGPipe",
    version="1.0",
    description="A pipeline to assign taxonomy from NGS sequencing data",
    author="URGI team",
    author_email="nlapalu@versailles.inra.fr",
    url="https://urgi.versailles.inra.fr/Projects/Achieved-projects/FungiGrapeMetaG",
    packages=Devtools.find_sub_packages(["commons","SMART"]),
    data_files=[('Gnome_tools',['Gnome_tools/LaunchFungalITSExtractor.py',
                                'Gnome_tools/CreateAnalysisCardForMetaGAnalysis.py',
                                'Gnome_tools/DownloadEntriesFromGenbank.py',
                                'Gnome_tools/FungalITSextractorURGI.pl',
                                'Gnome_tools/GetFastaFileFromGenBankFile.py',
                                'Gnome_tools/GetTaxonCompositionFromGenBank.py',
                                'Gnome_tools/LaunchAssignTaxonomyWithBlast.py',
                                'Gnome_tools/LaunchFungalITSExtractor.py',
                                'Gnome_tools/LaunchPickOTUWihtUclust.py',
                                'Gnome_tools/LaunchPickRepSet.py',
                                'Gnome_tools/MetaGPipe.py',
                                'Gnome_tools/TaxonomyClusterToFasta.py',
                                'Gnome_tools/Taxonomy.py',
                                'Gnome_tools/__init__.py',
                                'Gnome_tools/ITSExtractorDataBanks.sh',
                                'Gnome_tools/WriteTaxonomyMappingAndFastaFiles.py']),
                ('doc',['doc/MetaGPipe.pdf'])],
    cmdclass={'install':Install}
)
